/* API - Any Device - Any OS */
/* $Workfile: gsc_access.c $ */
/* $Revision: 9 $ */
/* $Date: 10/27/07 10:03a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/api/gsc_access.c $
 *
 * 9     10/27/07 10:03a Dnewbold
 * Ported to Linux.
 *
 * 8     4/05/07 7:48a Dnewbold
 * Updated the way device handles are calculated.
 *
 * 7     4/04/07 9:42a Dnewbold
 * A NULL device access handle is now always invalid.
 *
 * 6     4/02/07 4:47p Dnewbold
 * Added #include "gsc_os_device.h"
 *
 * 5     3/01/07 5:11p Dnewbold
 * Linted, increased error checking, updated for those environs that have
 * a hard coded first device base index.
 *
 * 4     10/12/06 12:45p Dnewbold
 * Correctred a bug in gsc_open().
 *
 * 3     5/05/06 10:35a Dnewbold
 *
 * 2     5/04/06 3:33p Dnewbold
 * Removed key validation code and increased modularity.
 *
 * 1     8/22/05 11:51a Dnewbold
*/

#include "gsc_access.h"

#include "gsc_os_device.h"	/*lint !e537 */

#include "gsc_drv.h"

#include "dev_version.h"



/* variables	**************************************************************/

		U32	api_initialized	= 0;
		U32	api_status		= (U32) GSC_SUCCESS;
		U32	api_status_arg	= 0;

static	U8	_api_base_index	= 0;



/*****************************************************************************
*
*	Function:	gsc_count_boards
*
*	Purpose:
*
*		Count the number of installed boards.
*
*	Arguments:
*
*		count		The count is recorded here upon success.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_count_boards(U8* count)
{
	os_device_t	device;
	int			i;
	U32			status;

	for (i = 0; i < 256; i++)
	{
		status	= os_device_open(	i + _api_base_index,
									API_DEV_PRODUCT_NAME_LOWER,
									&device);

		if (status != (U32) GSC_SUCCESS)
			break;

		os_device_close(&device);
	}

	count[0]	= (U8) i;
	return((U32) GSC_SUCCESS);
}


/*****************************************************************************
*
*	Function:	gsc_compute_base_index
*
*	Purpose:
*
*		Find out if the base index is zero or one. We need this to know which
*		index to use to access the application requested device.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_compute_base_index(void)
{
	os_device_t	device;
	int			hc;		// hard coded?
	U8			hc_base;
	U32			status;

	status	= os_device_hc_base_index(&hc, &hc_base);

	if (status != GSC_SUCCESS)
	{
	}
	else if (hc)
	{
		_api_base_index	= hc_base;
	}
	else
	{
		status	= os_device_open(0, API_DEV_PRODUCT_NAME_LOWER, &device);

		if (status == (U32) GSC_SUCCESS)
		{
			os_device_close(&device);
			_api_base_index	= 0;
		}
		else
		{
			_api_base_index	= 1;
		}
	}
}



/*****************************************************************************
*
*	Function:	gsc_handle_validate
*
*	Purpose:
*
*		Validate the given handle and return the corresponding list index
*		to the caller, if the handle is valid.
*
*	Arguments:
*
*		handle		The handle to validate and translate.
*
*		lst_size	The number of elements in the caller's device list.
*
*		st_size		The is the size of the caller's device data structure.
*
*		device		The is the first device structure in the caller's device
*					list.
*
*		lst_index	If all goes well, we record the list index for the give
*					handle here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_handle_validate(
	void*			handle,
	U8				lst_size,
	U32				st_size,
	gsc_device_t*	device,
	U8*				lst_index)
{
	U32	status	= (U32) GSC_INVALID_API_HANDLE;

	for (;;)	// A convenience loop.
	{
		if (handle == NULL)	// This is always invalid.
			break;

		lst_index[0]	= (U8) (((UL) handle) & 0xFF);

		if (lst_index[0] >= lst_size)
			break;

		device	= (gsc_device_t*) (((UL) device) + (st_size * lst_index[0]));

		if (device->handle != handle)
			break;

		status	= (U32) GSC_SUCCESS;
		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_open
*
*	Purpose:
*
*		Open a channel to the specified board.
*
*	Arguments:
*
*		index		The board index to access.
*
*		lst_index	The index in the device specific code's device list.
*
*		device		The data used to access the device.
*
*		handle		We record the application level handle here. NULL
*					indicates failure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_open(U8 index, U8 lst_index, gsc_device_t* device, void** handle)
{
	static	U32		count	= 1;

	U32				status;

	if (handle)
		handle[0]	= NULL;

	status	= gsc_drv_open(	device,
							(U8) (index + _api_base_index),
							API_DEV_PRODUCT_NAME_UPPER);

	if (status == (U32) GSC_SUCCESS)
	{
		if (count >= 0x00FFFFFFUL)
			count	= 1;
		else
			count++;

		device->dev_index	= index;
		device->lst_index	= lst_index;
		device->count		= count;
		device->handle		= (void*) (UL) (lst_index | ((count & 0x00FFFFFF) << 8));

		if (handle)
			handle[0]		= device->handle;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_close
*
*	Purpose:
*
*		Open a channel to the specified board.
*
*	Arguments:
*
*		device		The OS data used to access the device.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

U32 gsc_close(gsc_device_t* device)
{
	U32	status;

	if (device)
	{
		status				= gsc_drv_close(device);
		device->dev_index	= 0;
		device->lst_index	= 0;
		device->count		= 0;
		device->handle		= NULL;
	}
	else
	{
		status	= (U32) GSC_NULL_PARAM;
	}

	return(status);
}


