/* API - Any Device - Any OS */
/* $Workfile: gsc_event.c $ */
/* $Revision: 8 $ */
/* $Date: 10/29/07 1:51p $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/api/gsc_event.c $
 *
 * 8     10/29/07 1:51p Dnewbold
 * Removed duplicate code to cleanup an event.
 *
 * 7     10/29/07 12:10p Dnewbold
 * Restored some commented out code.
 *
 * 6     10/27/07 10:09a Dnewbold
 * Ported to Linux.
 *
 * 5     4/02/07 4:50p Dnewbold
 * Added a couple #includes for Linux porting.
 *
 * 4     3/01/07 5:14p Dnewbold
 * Linted.
 *
 * 3     10/12/06 3:58p Dnewbold
 * Added code to validate pointers received from outside callers.
 *
 * 2     5/04/06 3:37p Dnewbold
 * Removed key validation code and increased modularity.
 *
 * 1     8/22/05 11:48a Dnewbold
*/

#include <stdlib.h>
#include <string.h>

#include "gsc_event.h"



/*****************************************************************************
*
*	Function:	_event_callback_thread
*
*	Purpose:
*
*		This is an event notification response thread. The objective here is
*		to wait for the event to be triggered, then call the referenced
*		callback function.
*
*	Arguments:
*
*		pvArg	The thread's entry arguemtn.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static U32 _event_callback_thread(void* pvArg)
{
	gsc_event_t*	event		= (gsc_event_t*) pvArg;
	U32				status;
	U32				timeout_ms	= 60000L;
	gsc_event_t		tmp;

	for (;;)	/* Keep working until we can't any longer. */
	{
		status	= os_event_validate(&event->event);
		if (status != (U32) GSC_SUCCESS)		break;

		status	= os_event_wait(&event->event, timeout_ms);
		if (event->terminate)					break;		// Terminate!
		if (status == (U32) GSC_WAIT_TIMEOUT)	continue;	// Continue waiting.
		if (status != (U32) GSC_SUCCESS)		break;		// Program error.

		status	= os_event_validate(&event->event);
		if (status != (U32) GSC_SUCCESS)		break;

		status	= gsc_sem_lock(&event->sem);
		if (status != (U32) GSC_SUCCESS)		break;		// Abort;

		tmp	= event[0];	// Make a temp copy so the callback function can
						// change things while it is running.

		gsc_sem_unlock(&event->sem);

		if (tmp.callback)
			(tmp.callback)(tmp.arg1, tmp.arg2, tmp.arg3);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	gsc_event_config
*
*	Purpose:
*
*		Configure the callback portion of the referenced event structure.
*
*	Arguments:
*
*		event		The event notification structure to configure.
*
*		id			A private event identifier.
*
*		callback	A call back function to call when the interrupt occurs.
*					Pass in NULL to cancal call back notification.
*
*		arg1		The first argument passed to the helper funtion.
*
*		arg2		The second argument passed to the helper funtion.
*
*		arg3		The third argument passed to the helper funtion.
*
*		dest		A copy of the source structure is put her so the caller
*					can safely look at the results of the call.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_event_config(
	gsc_event_t*	event,
	void*			id,
	void			(*callback)(void* arg1, UL arg2, UL arg3),
	void*			arg1,
	UL				arg2,
	UL				arg3,
	gsc_event_t*	dest)
{
	U32	status;

	if (event)
		status	= gsc_sem_lock(&event->sem);
	else
		status	= (U32) GSC_NULL_PARAM;

	if (status == (U32) GSC_SUCCESS)
	{
		/*lint -save -e613 */
		if (id != (void*) GSC_NO_CHANGE)
			event->id		= id;		// id: index

		if ((UL) callback != GSC_NO_CHANGE)
			event->callback	= callback;	// callback function

		if (arg1 != (void*) GSC_NO_CHANGE)	// arg1: device handle
			event->arg1		= arg1;

		if (arg2 != GSC_NO_CHANGE)		// arg2: I/O status or irq "which" bit
			event->arg2		= arg2;

		if (arg3 != GSC_NO_CHANGE)		// arg3: user argument
			event->arg3		= arg3;

		if (dest)
			dest[0]	= event[0];

		gsc_sem_unlock(&event->sem);
		/*lint -restore */
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_event_create
*
*	Purpose:
*
*		Initialize an event structure in preparation for use.
*
*	Arguments:
*
*		event		The allocated and initialize structure is recorded here.
*
*		name		A name to associate with the event.
*
*		device		The data used to access the device.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_event_create(
	gsc_event_t**	event,
	const char*		name,
	gsc_device_t*	device)
{
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		if ((event == NULL) || (name == NULL) || (device == NULL))
		{
			status	= (U32) GSC_NULL_PARAM;
			break;
		}

		event[0]	= malloc(sizeof(gsc_event_t));

		if (event[0] == NULL)
		{
			status	= (U32) GSC_INSUFFICIENT_RESOURCES;
			break;
		}

		memset(event[0], 0, sizeof(gsc_event_t));
		event[0]->device	= device;
		status				= os_event_create(	&event[0]->event,
												name,
												&device->device);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= gsc_sem_create(&event[0]->sem, name);

		if (status != (U32) GSC_SUCCESS)
		{
			os_event_destroy(&event[0]->event);
			break;
		}

		status	= os_thread_create(	&event[0]->thread,
									name,
									_event_callback_thread,
									event[0]);

		if (status != (U32) GSC_SUCCESS)
		{
			os_event_destroy(&event[0]->event);
			gsc_sem_destroy(&event[0]->sem);
			break;
		}

		// Everything did work.
		status	= (U32) GSC_SUCCESS;
		break;
	}

	if ((status != (U32) GSC_SUCCESS) && (event[0]))	/*lint !e613 */
	{
		free(event[0]);			/*lint !e613 */
		event[0]	= NULL;		/*lint !e613 */
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_event_destroy
*
*	Purpose:
*
*		Reset the referenced structure as it is no longer in use.
*
*	Arguments:
*
*		event		The structure to reset.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_event_destroy(gsc_event_t** event)
{
	gsc_event_t*	ptr;

	if ((event) && (event[0]))
	{
		ptr			= event[0];
		event[0]	= NULL;
		gsc_sem_lock(&ptr->sem);		// This may be invalid.
		gsc_sem_unlock(&ptr->sem);		// This may be invalid.
		ptr->terminate	= 1;
		os_event_trigger(&ptr->event);
		os_thread_destroy(&ptr->thread);
		os_event_destroy(&ptr->event);
		gsc_sem_destroy(&ptr->sem);		// This may be invalid.
		memset(ptr, 0, sizeof(gsc_event_t));
		free(ptr);
	}
}



/*****************************************************************************
*
*	Function:	gsc_event_trigger
*
*	Purpose:
*
*		Trigger the referenced event.
*
*	Arguments:
*
*		event		The event notification structure to trigger.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_event_trigger(gsc_event_t* event)
{
	if (event)
		os_event_trigger(&event->event);
}



/*****************************************************************************
*
*	Function:	gsc_event_wait
*
*	Purpose:
*
*		Wait for a single event to occur.
*
*	Arguments:
*
*		event				The event notification structure to trigger.
*
*		timeout_ms			The timeout limit in milliseconds.
*
*	Returned:
*
*		GSC_SUCCESS			The event occurred.
*		GSC_WAIT_TIMEOUT	The timeout period lapsed first.
*		GSC_FAILED			The request failed.
*
*****************************************************************************/

U32 gsc_event_wait(gsc_event_t* event, U32 timeout_ms)
{
	U32	status;

	if (event)
		status	= os_event_wait(&event->event, timeout_ms);
	else
		status	= (U32) GSC_NULL_PARAM;

	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_event_wait_2
*
*	Purpose:
*
*		Wait for either of two events to occur.
*
*	Arguments:
*
*		e1					The event notification structure to trigger.
*
*		e2				The event notification structure to trigger.
*
*		timeout_ms			The timeout limit in milliseconds.
*
*	Returned:
*
*		GSC_SUCCESS			An event occurred.
*		GSC_WAIT_TIMEOUT	The timeout period lapsed first.
*		GSC_FAILED			The request failed.
*
*****************************************************************************/

U32 gsc_event_wait_2(gsc_event_t* e1, gsc_event_t* e2, U32 timeout_ms)
{
	U32	status;

	if ((e1) && (e2))
		status	= os_event_wait_2(&e1->event, &e2->event, timeout_ms);
	else
		status	= (U32) GSC_NULL_PARAM;

	return(status);
}


