/* API - Any Device - Any OS */
/* $Workfile: gsc_io_dma.c $ */
/* $Revision: 5 $ */
/* $Date: 4/02/07 4:51p $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/api/gsc_io_dma.c $
 *
 * 5     4/02/07 4:51p Dnewbold
 * Ported to Linux.
 *
 * 4     3/01/07 5:18p Dnewbold
 * Linted.
 *
 * 3     5/09/06 1:31p Dnewbold
 * Increased code modularity.
 *
 * 2     5/04/06 3:41p Dnewbold
 * Removed key validation code and increased modularity.
 *
 * 1     8/22/05 11:49a Dnewbold
*/

#include "gsc_io.h"

#include "gsc_drv.h"
#include "gsc_sleep.h"

#include "gsc_os_time.h"



/******************************************************************************
*
*	Function:	gsc_io_dma_wait
*
*	Purpose:
*
*		Wait for a DMA to go to completion, or timeout.
*
*	Arguments:
*
*		io			The common I/O parameters for the request.
*
*		timeout_ms	The timeout period remaining.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_dma_wait(gsc_io_t* io, U32 timeout_ms)
{
	long	after;
	long	before;
	long	elapsed;
	U32		last_status		= 0;
	int		retries			= 0;
	U32		status;

	if ((io->timeout_s == 0) || (timeout_ms < 5))
	{
		// Wait at least this long.
		timeout_ms	= 5;
	}

	for (;;)
	{
		status	= gsc_drv_dma_status(io->device, io->dma_channel->index);

		if (status == (U32) GSC_DMA_DONE)
		{
			if (last_status != status)
			{
				// There have been cases where the first status returned was
				// incorrect. Therefore, we check it twice before accepting it.
				last_status	= status;
				continue;
			}

			io->xfered	= io->xfer;
			status		= (U32) GSC_SUCCESS;
			break;
		}

		if (timeout_ms == 0)
		{
			// We can't wait any longer.
			status	= (U32) GSC_WAIT_TIMEOUT;
			break;
		}

		if (status == (U32) GSC_DMA_IN_PROGRESS)
		{
			if (last_status != status)
			{
				// There have been cases where the first status returned was
				// incorrect. Therefore, we check it twice before accepting it.
				last_status	= status;
				continue;
			}

			// The DMA request is still active.
			before		= (long) os_time_delta_ms();
			status		= gsc_event_wait(io->dma_channel->irq, 1);
			after		= (long) os_time_delta_ms();
			elapsed		= after - before;

			if (timeout_ms > (U32) elapsed)
				timeout_ms	-= (U32) elapsed;
			else
				timeout_ms	= 0;

			continue;
		}

		// There was a problem.

		if (status == (U32) GSC_SUCCESS)
		{
			/*
			*	There have been cases where this status is returned repeatedly
			*	over a period of time. Don
			*/

			if ((retries % 10) == 9)
			{
				// Every 10 calls do a wait.
				gsc_sleep_ms(1);
				retries++;
				continue;
			}
			else if (retries <= 100)
			{
				retries++;
				continue;
			}
			else
			{
				// Things just are not working out. :(
			}
		}

		// There is a problem.
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	gsc_io_dma_read
*
*	Purpose:
*
*		Perform a DMA based read operation (read data from the board).
*
*	Arguments:
*
*		io			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_dma_read(gsc_io_t* io)
{
	U32	status;

	status	= gsc_dma_transfer(io, 1);
	return(status);
}



/******************************************************************************
*
*	Function:	gsc_io_dma_write
*
*	Purpose:
*
*		Perform a DMA based write operation (write data to the board).
*
*	Arguments:
*
*		io			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_dma_write(gsc_io_t* io)
{
	U32	status;

	status	= gsc_dma_transfer(io, 0);
	return(status);
}


