/* API - Any Device - Any OS */
/* $Workfile: gsc_irq.c $ */
/* $Revision: 7 $ */
/* $Date: 10/27/07 10:11a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/api/gsc_irq.c $
 *
 * 7     10/27/07 10:11a Dnewbold
 * Ported to Linux.
 *
 * 6     4/02/07 4:51p Dnewbold
 * Ported to Linux.
 *
 * 5     3/01/07 5:18p Dnewbold
 * Linted.
 *
 * 4     5/09/06 1:31p Dnewbold
 * Increased code modularity.
 *
 * 3     5/05/06 10:35a Dnewbold
 *
 * 2     5/04/06 3:43p Dnewbold
 * Removed key validation code and increased modularity.
 *
 * 1     8/22/05 11:49a Dnewbold
*/

#include <string.h>

#include "gsc_irq.h"

#include "gsc_drv.h"
#include "gsc_version.h"



/*****************************************************************************
*
*	Function:	_irq_local_destroy
*
*	Purpose:
*
*		Remove the specified local interrupt from the list for which
*		notification is desired.
*
*	Arguments:
*
*		irq			The main interrupt notification structure.
*
*		int_id		The interrupt to add.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _irq_local_destroy(gsc_irq_t* irq, U32 int_id)
{
	if (irq->local[int_id])
	{
		gsc_event_destroy(&irq->local[int_id]);
		irq->local[int_id]	= NULL;
	}
}



/*****************************************************************************
*
*	Function:	_irq_main_callback
*
*	Purpose:
*
*		This is the callback routine for PLX detected interrupts. Here, we
*		check to see which PLX interrupt occurred, then we trigger the
*		corresponding event handle.
*
*	Arguments:
*
*		arg1	A pointer to the main IRQ structure.
*
*		arg2	Unused.
*
*		arg3	Unused.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _irq_main_callback(void* arg1, UL arg2, UL arg3)
{
	gsc_device_t*	device;
	gsc_event_t		event;
	gsc_irq_t*		irq;
	gsc_irq_src_t	src;
	U32				status;

	irq		= (gsc_irq_t*) arg1;
	device	= irq->device;

	for (;;)
	{
		status	= gsc_sem_lock(&irq->pci->sem);

		if (status != (U32) GSC_SUCCESS)
			break;						// Abort;

		status	= gsc_drv_irq_status(device, &src);
		// Re-enable interrupts and notification.
		gsc_drv_irq_attach(device, &irq->src, &irq->pci->event);

		if (status != (U32) GSC_SUCCESS)
		{
			gsc_sem_unlock(&irq->pci->sem);
		}
		else if (src.dma0)
		{
			if (irq->plx[GSC_PLX_INT_DMA_0])
			{
				event	= irq->plx[GSC_PLX_INT_DMA_0][0];
				gsc_sem_unlock(&irq->pci->sem);
				gsc_event_trigger(&event);
			}
			else
			{
				gsc_sem_unlock(&irq->pci->sem);
			}
		}
		else if (src.dma1)
		{
			if (irq->plx[GSC_PLX_INT_DMA_1])
			{
				event	= irq->plx[GSC_PLX_INT_DMA_1][0];
				gsc_sem_unlock(&irq->pci->sem);
				gsc_event_trigger(&event);
			}
			else
			{
				gsc_sem_unlock(&irq->pci->sem);
			}
		}
		else if (src.local)
		{
			if (irq->plx[GSC_PLX_INT_LOCAL])
			{
				event	= irq->plx[GSC_PLX_INT_LOCAL][0];
				gsc_sem_unlock(&irq->pci->sem);
				gsc_irq_enable(irq);
				gsc_event_trigger(&event);
			}
			else
			{
				gsc_irq_enable(irq);
				gsc_sem_unlock(&irq->pci->sem);
			}
		}
		else
		{
			gsc_sem_unlock(&irq->pci->sem);
		}

		break;
	}
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	_irq_translate_int_id
*
*	Purpose:
*
*		Translate the given PLX interrupt identifier and either add or remove
*		it from the given structure.
*
*	Arguments:
*
*		int_id		The interrupt id to translate.
*
*		enable		Add the interrupt?
*
*		irq			The structure to fill in.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _irq_translate_int_id(U32 int_id, int enable, gsc_irq_src_t* irq)
{
	U32	status;

	switch (int_id)
	{
		default:				status		= (U32) GSC_INVALID_DATA;
								break;

		case GSC_PLX_INT_DMA_0:	irq->dma0	= enable ? 1 : 0;
								status		= (U32) GSC_SUCCESS;
								break;

		case GSC_PLX_INT_DMA_1:	irq->dma1	= enable ? 1 : 0;
								status		= (U32) GSC_SUCCESS;
								break;

		case GSC_PLX_INT_LOCAL:	irq->local	= enable ? 1 : 0;
								status		= (U32) GSC_SUCCESS;
								break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_irq_plx_destroy
*
*	Purpose:
*
*		Cleanup the given PLX interrupt as the device is no longer in use.
*
*	Arguments:
*
*		irq			The main interrupt notification structure.
*
*		int_id		The interrupt to remove.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _irq_plx_destroy(gsc_irq_t* irq, U32 int_id)
{
	if (irq->plx[int_id])
	{
		_irq_translate_int_id(int_id, 0, &irq->src);
		gsc_drv_irq_attach(irq->device, &irq->src, &irq->pci->event);
		gsc_event_destroy(&irq->plx[int_id]);
		irq->plx[int_id]	= NULL;
	}
}



/*****************************************************************************
*
*	Function:	gsc_irq_create
*
*	Purpose:
*
*		Prepare the given structure for initial use.
*
*	Arguments:
*
*		irq			The main interrupt notification structure.
*
*		device		The data used to access the device.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_irq_create(gsc_irq_t* irq, gsc_device_t* device)
{
	U32	stat;
	U32	status	= (U32) GSC_SUCCESS;

	memset(irq, 0, sizeof(irq[0]));
	irq->device	= device;

	stat	= gsc_event_create(&irq->pci, "IRQ PCI", device);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_drv_irq_master_event_create(irq, device);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	// Install the main callback routine.
	stat	= gsc_event_config(	irq->pci,
								0,					// id: index
								_irq_main_callback,	// callback function
								irq,				// arg1: gsc_irq_t*
								0,					// arg2:
								0,					// arg3:
								NULL);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;
	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_irq_destroy
*
*	Purpose:
*
*		Perform a complete shut down of an open product's interrupt
*		notification feature.
*
*	Arguments:
*
*		irq			The main interrupt notification structure.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_irq_destroy(gsc_irq_t* irq)
{
	int	i;

	for (i = 0; i < (int) ARRAY_ELEMENTS(irq->local); i++)
		_irq_local_destroy(irq, (U32) i);

	// Cleanup the PLX feature set events.
	_irq_plx_destroy(irq, GSC_PLX_INT_DMA_0);
	_irq_plx_destroy(irq, GSC_PLX_INT_DMA_1);
	_irq_plx_destroy(irq, GSC_PLX_INT_LOCAL);

	// Cleanup the main IRQ event.
	gsc_drv_irq_master_event_destroy(irq);
}



/*****************************************************************************
*
*	Function:	gsc_irq_disable
*
*	Purpose:
*
*		Disable all interrupts for this device.
*
*	Arguments:
*
*		irq			The main interrupt notification structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_irq_disable(gsc_irq_t* irq)
{
	U32	status;

	status	= gsc_drv_irq_disable(irq->device);
	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_irq_enable
*
*	Purpose:
*
*		Enable all interrupts for this device (as appropriate for GSC
*		products).
*
*	Arguments:
*
*		irq			The main interrupt notification structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_irq_enable(gsc_irq_t* irq)
{
	U32	status;

	status	= gsc_drv_irq_enable(irq->device);
	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_irq_local_create
*
*	Purpose:
*
*		Prepare the specified local interrupt for initial use.
*
*	Arguments:
*
*		irq			The main interrupt notification structure.
*
*		name		A name to associate with the given interrupt source.
*
*		int_id		The interrupt to add.
*
*		local		The needed data is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_irq_local_create(gsc_irq_t* irq, const char* name, U32 int_id, gsc_event_t** local)
{
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= gsc_sem_lock(&irq->pci->sem);

		if (status != (U32) GSC_SUCCESS)
			break;								// Can't get the semaphore.

		if (int_id >= ARRAY_ELEMENTS(irq->local))
		{
			gsc_sem_unlock(&irq->pci->sem);
			break;								// Invalid id.
		}

		if (irq->local[int_id])
		{
			gsc_sem_unlock(&irq->pci->sem);
			break;								// Duplicate request.
		}

		status	= gsc_event_create(local, name, irq->device);

		if (status == (U32) GSC_SUCCESS)
		{
			local[0]->id		= (void*) (UL) int_id;
			irq->local[int_id]	= local[0];
		}

		gsc_sem_unlock(&irq->pci->sem);
		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_irq_plx_create
*
*	Purpose:
*
*		Prepare a PLX interrupt handling event structure for initial use.
*
*	Arguments:
*
*		irq			The main interrupt notification structure.
*
*		name		A name to associate with the given interrupt source.
*
*		int_id		The interrupt to add.
*
*		plx			The needed data is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_irq_plx_create(	gsc_irq_t*		irq,
						const char*		name,
						U32				int_id,
						gsc_event_t**	plx)
{
	U32	status	= (U32) GSC_SUCCESS;

	status	= gsc_sem_lock(&irq->pci->sem);

	if ((status == (U32) GSC_SUCCESS) && (int_id >= ARRAY_ELEMENTS(irq->plx)))
		status	= (U32) GSC_INVALID_DATA;

	if ((status == (U32) GSC_SUCCESS) &&
		(irq->plx[int_id]))		/*lint !e661 */
		status	= (U32) GSC_OBJECT_ALREADY_ALLOCATED;

	if (status == (U32) GSC_SUCCESS)
		status	= _irq_translate_int_id(int_id, 1, &irq->src);

	if (status == (U32) GSC_SUCCESS)
		status	= gsc_drv_irq_attach(irq->device, &irq->src, &irq->pci->event);

	if (status == (U32) GSC_SUCCESS)
		status	= gsc_event_create(plx, name, irq->device);

	if (status == (U32) GSC_SUCCESS)
	{
		plx[0]->id			= (void*) (UL) int_id;
		irq->plx[int_id]	= plx[0];	/*lint !e661 */
	}

	gsc_sem_unlock(&irq->pci->sem);
	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_irq_local_wait
*
*	Purpose:
*
*		Block until one of the local interrupts referenced in the given
*		bit mask occurs.
*
*	Arguments:
*
*		irq			The main interrupt notification structure.
*
*		mask		The set of interrupts of interest.
*
*		timeout_ms	Return if an interrupt doesn't occur within this
*					number of milliseconds.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_irq_local_wait(gsc_irq_t* irq, U32 mask, U32 timeout_ms)
{
	U32				bit;
	U8				count	= 0;
	os_event_t		events[GSC_LOCAL_INT_QTY_MAX];
	int				i;
	U32				status;

	for (;;)	/* A convenience loop. */
	{
		status	= gsc_sem_lock(&irq->pci->sem);
		if (status != (U32) GSC_SUCCESS)	break;		// Can't get the semaphore.
		memset(events, 0, sizeof(events));

		for (	i = 0, bit = 0x1;
				i < (int) ARRAY_ELEMENTS(irq->local);
				i++, bit <<= 1)
		{
			if (bit & mask)
			{
				events[count]	= irq->local[i]->event;
				count++;
			}
		}

		gsc_sem_unlock(&irq->pci->sem);

		if (count == 0)
			status	= (U32) GSC_INVALID_DATA;
		else
			status	= os_event_wait_multi(events, count, timeout_ms);

		break;
	}

	return(status);
}
