/* API - Any Device - Any OS */
/* $Workfile: gsc_mem.c $ */
/* $Revision: 6 $ */
/* $Date: 10/27/07 10:12a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/api/gsc_mem.c $
 *
 * 6     10/27/07 10:12a Dnewbold
 * Ported to Linux.
 *
 * 5     4/02/07 4:52p Dnewbold
 * Ported to Linux.
 *
 * 4     3/01/07 5:19p Dnewbold
 * Linted.
 *
 * 3     5/09/06 1:31p Dnewbold
 * Increased code modularity.
 *
 * 2     5/04/06 3:44p Dnewbold
 * Removed key validation code and increased modularity.
 *
 * 1     8/22/05 11:49a Dnewbold
*/

#include <string.h>

#include "gsc_mem.h"

#include "gsc_drv.h"



/*****************************************************************************
*
*	Function:	_mem_free
*
*	Purpose:
*
*		Free the memory referenced by the given structure.
*
*	Arguments:
*
*		mem			The memory structure to alter.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _mem_free(gsc_mem_t* mem)
{
	U32	status;

	if (mem->size)
	{
		status	= gsc_drv_mem_unmap(mem->device,
									mem->size,
									mem->phy_adrs,
									mem->usr_adrs);

		if (status == (U32) GSC_SUCCESS)
		{
			mem->usr_adrs	= 0;
			status			= gsc_drv_mem_free(	mem->device,
												mem->size,
												mem->phy_adrs);
		}

		if (status == (U32) GSC_SUCCESS)
		{
			mem->size		= 0;
			mem->phy_adrs	= 0;
		}
	}
	else
	{
		status	= (U32) GSC_SUCCESS;
		mem->size		= 0;
		mem->phy_adrs	= 0;
		mem->usr_adrs	= 0;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_mem_alloc
*
*	Purpose:
*
*		Change the size of the structures allocated memory.
*
*	Arguments:
*
*		mem			The memory structure to alter.
*
*		size		The desired size of the new allocation. The actual
*					results may be smaller than requested.
*
*		dest		A copy of the structure is recorded here for direct access
*					by the called.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_mem_alloc(gsc_mem_t* mem, U32 size, gsc_mem_t* dest)
{
	U32	status;

	status	= gsc_sem_lock(&mem->sem);

	if (status == (U32) GSC_SUCCESS)
	{
		for (;;)	// A convenience loop.
		{
			if ((size == GSC_NO_CHANGE) || (size == mem->size))
			{
				// Don't change anything.
				break;
			}

			// Free the current memory.
			status	= _mem_free(mem);

			if ((status != (U32) GSC_SUCCESS) || (size == 0))
				break;

			// Allocate a block.
			mem->size	= size;
			status		= gsc_drv_mem_alloc(mem->device,
											&mem->size,
											&mem->phy_adrs);

			if (status != (U32) GSC_SUCCESS)
			{
				// The memory allocation failed.
				mem->size		= 0;
				mem->phy_adrs	= 0;
				break;
			}

			// Map the block to user space.
			status	= gsc_drv_mem_map(	mem->device,
										mem->size,
										mem->phy_adrs,
										&mem->usr_adrs);

			if (status != (U32) GSC_SUCCESS)
			{
				// We couldn't map the memory to user space.
				mem->usr_adrs	= 0;
				_mem_free(mem);
				break;
			}

			// Success.
			break;
		}

		if (dest)
			dest[0]	= mem[0];

		gsc_sem_unlock(&mem->sem);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_mem_free
*
*	Purpose:
*
*		Free the structures allocated memory.
*
*	Arguments:
*
*		mem			The memory structure to alter.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_mem_free(gsc_mem_t* mem)
{
	U32	status;

	status	= gsc_sem_lock(&mem->sem);

	if (status == (U32) GSC_SUCCESS)
	{
		status	= _mem_free(mem);
		gsc_sem_unlock(&mem->sem);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_mem_create
*
*	Purpose:
*
*		Initialize a memory structure as the device is going into use.
*
*	Arguments:
*
*		mem			The memory structure to initlaize.
*
*		name		A name to associate with the memory resource.
*
*		device		The data used to access the device.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_mem_create(gsc_mem_t* mem, const char* name, gsc_device_t* device)
{
	U32	status;

	memset(mem, 0, sizeof(gsc_mem_t));
	mem->device	= device;
	status		= gsc_sem_create(&mem->sem, name);
	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_mem_destroy
*
*	Purpose:
*
*		Cleanup the memory structure as the device is no longer in use.
*
*	Arguments:
*
*		mem			The memory structure to alter.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_mem_destroy(gsc_mem_t* mem)
{
	gsc_sem_lock(&mem->sem);
	_mem_free(mem);
	gsc_sem_destroy(&mem->sem);
	mem->size		= 0;
	mem->phy_adrs	= 0;
	mem->usr_adrs	= 0;
}



/*****************************************************************************
*
*	Function:	gsc_mem_test
*
*	Purpose:
*
*		See if the given pointer access memory in the referenced structure.
*
*	Arguments:
*
*		mem			The memory structure to alter.
*
*		ptr			The memory pointer to test.
*
*		size		The size of the memory referenced by the above pointer.
*
*		dest		The results are recorded here. The size is zero if the
*					memories are unique. Otherwise all fields are adjusted
*					to reference the overlap.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_mem_test(gsc_mem_t* mem, void* ptr, U32 size, gsc_mem_t* tmp)
{
	UL	bm;		// Begin address of "mem".
	UL	bp;		// Begin address of "ptr".
	UL	em;		// End address of "mem".
	UL	ep;		// End address of "ptr".
	U32	status;

	status	= gsc_sem_lock(&mem->sem);

	if (status == (U32) GSC_SUCCESS)
	{
		bm	= (UL) mem->usr_adrs;
		em	= bm + mem->size - 1;
		bp	= (UL) ptr;
		ep	= bp + size - 1;

		if ((mem->size == 0) ||
			(size == 0) ||
			(bp < bm) ||
			(bp > em))
		{
			memset(tmp, 0, sizeof(tmp[0]));
		}
		else
		{
			tmp->sem		= mem->sem;
			tmp->device		= mem->device;
			tmp->usr_adrs	= bp;
			tmp->phy_adrs	= mem->phy_adrs + (bp - bm);

			if (ep <= em)
				tmp->size	= size;
			else
				tmp->size	= mem->size - (bp - bm);
		}

		gsc_sem_unlock(&mem->sem);
	}

	return(status);
}


