/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* Any Device - Driver - Linux */
/* $Workfile: irq.c $ */
/* $Revision: 3 $ */
/* $Date: 10/27/07 10:33a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/driver/linux/irq.c $
 *
 * 3     10/27/07 10:33a Dnewbold
 * Additional driver porting updates.
 *
 * 2     3/29/07 2:44p Dnewbold
 *
 * 1     5/10/06 1:02p Dnewbold
*/

#include "main.h"



/******************************************************************************
*
*	Function:	_irq_isr_notify
*
*	Purpose:
*
*		Perform the work needed to notify applications of interrupts.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*		pid		The process specific data structure for this device.
*
*		mask	The bits for the interrupts that we detected.
*
*	Returned:
*
*		None.
*
******************************************************************************/

static void _irq_isr_notify(dev_data_t* dev, dev_pid_t* pid, u32 mask)
{
	mask	&= pid->irq_notify;

	if (mask)
	{
		pid->irq_triggered	|= mask;

		if (pid->queued)
		{
			pid->queued		= 0;
			pid->timeout	= 0;
			EVENT_RESUME_IRQ(&pid->queue, pid->condition);
		}
	}
}



/******************************************************************************
*
*	Function:	_irq_access_lock
*
*	Purpose:
*
*		Apply a locking mechanism to prevent simultaneous access to the
*		device's IRQ substructure.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*		isr		Is this an ISR based request?
*
*	Returned:
*
*		0		The request was satisfied.
*		<0		An error code for the problem seen.
*
******************************************************************************/

static int _irq_access_lock(dev_data_t* dev, int isr)
{
	int	status;

	if (isr)
	{
		status	= 0;
		spin_lock(&dev->irq.spinlock);
	}
	else
	{
		status	= down_interruptible(&dev->irq.sem);

		if (status == 0)
			disable_irq(dev->pci->irq);
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_irq_access_unlock
*
*	Purpose:
*
*		Remove the locking mechanism that prevented simultaneous access to the
*		device's IRQ substructure.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*		isr		Is this an ISR based request?
*
*	Returned:
*
*		None.
*
******************************************************************************/

static void _irq_access_unlock(dev_data_t* dev, int isr)
{
	if (isr)
	{
		spin_unlock(&dev->irq.spinlock);
	}
	else
	{
		enable_irq(dev->pci->irq);
		up(&dev->irq.sem);
	}
}



/******************************************************************************
*
*	Function:	irq_isr_common
*
*	Purpose:
*
*		Service an interrupt. For safety sake, each detected interrupt
*		is cleared then disabled.
*
*	Arguments:
*
*		irq		The interrupt number.
*
*		dev_id	The private data we've associated with the IRQ.
*
*		regs	Unused.
*
*	Returned:
*
*		0		The interrupt was NOT ours.
*		1		The interrupt was ours.
*
******************************************************************************/

int irq_isr_common(int irq, void* dev_id)
{
	dev_data_t*	dev		= dev_id;
	int			handled	= 0;
	int			i;
	u32			intcsr;			/* PLX Interrupt Control/Status */
	u32			mask	= 0;

	_irq_access_lock(dev, 1);

	for (;;)
	{
		intcsr	= readl(dev->vaddr.plx_intcsr_32);

		if (intcsr & INTCSR_DMA_0_IRQ_ACTIVE)
		{
			// A DMA 0 Done interrupt occurred.
			handled	= 1;
			mask	|= GSC_IRQ_PCI | GSC_IRQ_DMA0;

			// Clear the interrupt.
			writeb(DMA_CSR_CLEAR, dev->vaddr.plx_dmacsr0_8);

			// Disable the interrupt.
			intcsr	&= ~INTCSR_DMA_0_IRQ_ENABLE;
			writel(intcsr, dev->vaddr.plx_intcsr_32);
readl((void*) (0UL + (unsigned long) dev->bars.bar[2].vaddr));
		}

		if (intcsr & INTCSR_DMA_1_IRQ_ACTIVE)
		{
			// A DMA 1 Done interrupt occurred.
			handled	= 1;
			mask	|= GSC_IRQ_PCI | GSC_IRQ_DMA1;

			// Clear the interrupt.
			writeb(DMA_CSR_CLEAR, dev->vaddr.plx_dmacsr1_8);

			// Disable the interrupt.
			intcsr	&= ~INTCSR_DMA_1_IRQ_ENABLE;
			writel(intcsr, dev->vaddr.plx_intcsr_32);
readl((void*) (4UL + (unsigned long) dev->bars.bar[2].vaddr));
		}

		if (intcsr & INTCSR_LOCAL_IRQ_ACTIVE)
		{
			// A local side interrupt occurred.
			handled	= 1;
			mask	|= GSC_IRQ_PCI | GSC_IRQ_LOCAL;

			// The interrupt source must be cleared by application code.

			// Disable the interrupt.
			intcsr	&= ~INTCSR_LOCAL_IRQ_ENABLE;
			writel(intcsr, dev->vaddr.plx_intcsr_32);
readl((void*) (8UL + (unsigned long) dev->bars.bar[2].vaddr));
		}

		if (intcsr & INTCSR_PCI_DOORBELL_IRQ_ACTIVE)
		{
			// A PCI Doorbell interrupt occurred.
			handled	= 1;
			mask	|= GSC_IRQ_PCI;

			// Clear the interrupt.
			writel(0xFFFFFFFF, dev->vaddr.plx_l2pdbell_32);

			// Disable the interrupt.
			intcsr	&= ~INTCSR_PCI_DOORBELL_IRQ_ENABLE;
			writel(intcsr, dev->vaddr.plx_intcsr_32);
readl((void*) (12UL + (unsigned long) dev->bars.bar[2].vaddr));
		}

		if (intcsr & INTCSR_PCI_ABORT_IRQ_ACTIVE)
		{
			// A PCI Abort interrupt has occurred.
			handled	= 1;
			mask	|= GSC_IRQ_PCI;

			// Clear the interrupt.
			pci_write_config_word(dev->pci, 0x06, 0x3800);

			// Disable the interrupt.
			intcsr	&= ~INTCSR_PCI_ABORT_IRQ_ENABLE;
			writel(intcsr, dev->vaddr.plx_intcsr_32);
readl((void*) (16UL + (unsigned long) dev->bars.bar[2].vaddr));
		}

		if (intcsr & INTCSR_BIST_IRQ_ACTIVE)
		{
			// A BIST interrupt has occurred.
			handled	= 1;
			mask	|= GSC_IRQ_PCI;

			// Disable and clear the interrupt.
			pci_write_config_byte(dev->pci, 0x0F, 0);
readl((void*) (20UL + (unsigned long) dev->bars.bar[2].vaddr));
		}

		if (handled)
		{
			for (i = 0; i < ARRAY_ELEMENTS(dev->pid); i++)
			{
				if (dev->pid[i])
					_irq_isr_notify(dev, dev->pid[i], mask);
			}

			break;
		}

		handled	= 0;
		break;
	}

	_irq_access_unlock(dev, 1);
	return(handled);
}



/******************************************************************************
*
*	Function:	irq_release
*
*	Purpose:
*
*		Release the IRQ, if it has been acquired.
*
*	Arguments:
*
*		dev	The device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void irq_release(dev_data_t* dev)
{
	if (dev->irq.acquired)
	{
		dev->irq.acquired	= 0;
		free_irq(dev->pci->irq, dev);
	}
}



/******************************************************************************
*
*	Function:	irq_request
*
*	Purpose:
*
*		Request that this deivce be allotted its assigned IRQ.
*
*	Arguments:
*
*		dev	The device of interest.
*
*	Returned:
*
*		0	All went well.
*		<0	An error code for the problem seen.
*
******************************************************************************/

int irq_request(dev_data_t* dev)
{
	int	status;

	status	= IRQ_REQUEST(dev, irq_isr);

	if (status)
		dev->irq.acquired	= 0;
	else
		dev->irq.acquired	= 1;

	return(status);
}



/******************************************************************************
*
*	Function:	irq_disable_ioctl
*
*	Purpose:
*
*		Disable the specified set of interrupts.
*
*	Arguments:
*
*		dev	The data for the device of interest.
*
*		arg	The IOCTL service's required argument.
*
*	Returned:
*
*		0	All went well.
*		<0	An error code for the problem seen.
*
******************************************************************************/

int irq_disable_ioctl(dev_data_t* dev, gsc_drv_irq_t* irq)
{
	u32			dword;
	dev_pid_t*	pid;
	int			status;
	u32			value;

	for (;;)	// A convenience loop.
	{
		if (irq->enable & ~GSC_IRQ_MASK)
		{
			status		= -EINVAL;
			irq->status	= GSC_INVALID_DATA;
			break;
		}

		pid	= pid_locate(dev);

		if (pid == NULL)
		{
			status		= -ENODEV;
			irq->status	= GSC_INVALID_DATA;
			break;
		}

		// Compute the needed changes.
		value	= 0;

		if (irq->enable & GSC_IRQ_PCI)
			value	|= INTCSR_PCI_INT_ENABLE;

		if (irq->enable & GSC_IRQ_DMA0)
			value	|= INTCSR_DMA_0_IRQ_ENABLE;

		if (irq->enable & GSC_IRQ_DMA1)
			value	|= INTCSR_DMA_1_IRQ_ENABLE;

		if (irq->enable & GSC_IRQ_LOCAL)
			value	|= INTCSR_LOCAL_IRQ_ENABLE;

		// Apply the computed changes.
		status	= _irq_access_lock(dev, 0);

		if (status)
		{
			irq->status	= GSC_ACCESS_DENIED;
			break;
		}

		dword	= readl(dev->vaddr.plx_intcsr_32);
		dword	= dword & ~value;
		writel(dword, dev->vaddr.plx_intcsr_32);

		// We're done.
		_irq_access_unlock(dev, 0);
		irq->status	= GSC_SUCCESS;
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	irq_enable_ioctl
*
*	Purpose:
*
*		Enable the specified set of interrupts.
*
*	Arguments:
*
*		dev	The data for the device of interest.
*
*		arg	The IOCTL service's required argument.
*
*	Returned:
*
*		0	All went well.
*		<0	An error code for the problem seen.
*
******************************************************************************/

int irq_enable_ioctl(dev_data_t* dev, gsc_drv_irq_t* irq)
{
	u32			dword;
	dev_pid_t*	pid;
	int			status;
	u32			value;

	for (;;)	// A convenience loop.
	{
		if (irq->enable & ~GSC_IRQ_MASK)
		{
			status		= -EINVAL;
			irq->status	= GSC_INVALID_DATA;
			break;
		}

		pid	= pid_locate(dev);

		if (pid == NULL)
		{
			status		= -ENODEV;
			irq->status	= GSC_INVALID_DATA;
			break;
		}

		// Compute the needed changes.
		value	= 0;

		if (irq->enable & GSC_IRQ_PCI)
			value	|= INTCSR_PCI_INT_ENABLE;

		if (irq->enable & GSC_IRQ_DMA0)
			value	|= INTCSR_DMA_0_IRQ_ENABLE;

		if (irq->enable & GSC_IRQ_DMA1)
			value	|= INTCSR_DMA_1_IRQ_ENABLE;

		if (irq->enable & GSC_IRQ_LOCAL)
			value	|= INTCSR_LOCAL_IRQ_ENABLE;

		// Apply the computed changes.
		status	= _irq_access_lock(dev, 0);

		if (status)
		{
			irq->status	= GSC_ACCESS_DENIED;
			break;
		}

		dword	= readl(dev->vaddr.plx_intcsr_32);
		dword	= (dword & ~value) | value;
		writel(dword, dev->vaddr.plx_intcsr_32);

		// We're done.
		_irq_access_unlock(dev, 0);
		irq->status	= GSC_SUCCESS;
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	irq_notify_ioctl
*
*	Purpose:
*
*		Identify those interrupts for which notification is desired.
*
*	Arguments:
*
*		dev	The data for the device of interest.
*
*		arg	The IOCTL service's required argument.
*
*	Returned:
*
*		0	All went well.
*		<0	An error code for the problem seen.
*
******************************************************************************/

int irq_notify_ioctl(dev_data_t* dev, gsc_drv_irq_t* irq)
{
	dev_pid_t*	pid;
	int			status;

	for (;;)	// A convenience loop.
	{
		if (irq->enable & ~GSC_IRQ_MASK)
		{
			status		= -EINVAL;
			irq->status	= GSC_INVALID_DATA;
			break;
		}

		pid	= pid_locate(dev);

		if (pid == NULL)
		{
			status		= -ENODEV;
			irq->status	= GSC_INVALID_DATA;
			break;
		}

		status	= _irq_access_lock(dev, 0);

		if (status)
		{
			irq->status	= GSC_ACCESS_DENIED;
			break;
		}

		// We've got exclusive access now.
		pid->irq_notify	= irq->notify;
		_irq_access_unlock(dev, 0);
		irq->status	= GSC_SUCCESS;
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	irq_status_ioctl
*
*	Purpose:
*
*		Report the set of interrupts which have occurrred.
*
*	Arguments:
*
*		dev	The data for the device of interest.
*
*		arg	The IOCTL service's required argument.
*
*	Returned:
*
*		0	All went well.
*		<0	An error code for the problem seen.
*
******************************************************************************/

int irq_status_ioctl(dev_data_t* dev, gsc_drv_irq_t* irq)
{
	u32			dword;
	dev_pid_t*	pid;
	int			status;

	memset(irq, 0, sizeof(irq[0]));

	for (;;)	// A convenience loop.
	{
		pid	= pid_locate(dev);

		if (pid == NULL)
		{
			status		= -ENODEV;
			irq->status	= GSC_INVALID_DATA;
			break;
		}

		status	= _irq_access_lock(dev, 0);

		if (status)
		{
			irq->status	= GSC_ACCESS_DENIED;
			break;
		}

		// We've got exclusive access now.
		irq->notify			= pid->irq_notify;
		irq->triggered		= pid->irq_triggered;
		irq->wait_cancel	= pid->irq_cancel;
		dword				= readl(dev->vaddr.plx_intcsr_32);
		pid->irq_triggered	= 0;
		_irq_access_unlock(dev, 0);

		// Report which interrupts are enabled.

		if (dword & INTCSR_PCI_INT_ENABLE)
			irq->enable	|= GSC_IRQ_PCI;

		if (dword & INTCSR_DMA_0_IRQ_ENABLE)
			irq->enable	|= GSC_IRQ_DMA0;

		if (dword & INTCSR_DMA_1_IRQ_ENABLE)
			irq->enable	|= GSC_IRQ_DMA1;

		if (dword & INTCSR_LOCAL_IRQ_ENABLE)
			irq->enable	|= GSC_IRQ_LOCAL;

		irq->status	= GSC_SUCCESS;
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	irq_wait_cancel_ioctl
*
*	Purpose:
*
*		Cancal the current or next IRQ wait request.
*
*	Arguments:
*
*		dev	The data for the device of interest.
*
*		arg	The IOCTL service's required argument.
*
*	Returned:
*
*		0	All went well.
*		<0	An error code for the problem seen.
*
******************************************************************************/

int irq_wait_cancel_ioctl(dev_data_t* dev, gsc_drv_irq_t* irq)
{
	dev_pid_t*	pid;
	int			status;

	for (;;)	// A convenience loop.
	{
		pid	= pid_locate(dev);

		if (pid == NULL)
		{
			status		= -ENODEV;
			irq->status	= GSC_INVALID_DATA;
			break;
		}

		if (irq->wait_cancel == 0)
		{
			status			= 0;
			pid->irq_cancel	= 0;
			irq->status		= GSC_SUCCESS;
			break;
		}

		pid->irq_cancel	= 1;
		status			= _irq_access_lock(dev, 0);

		if (status)
		{
			irq->status	= GSC_ACCESS_DENIED;
			break;
		}

		if (pid->queued)
		{
			pid->queued		= 0;
			pid->timeout	= 0;
			EVENT_RESUME_IRQ(&pid->queue, pid->condition);
		}

		_irq_access_unlock(dev, 0);
		status		= 0;
		irq->status	= GSC_SUCCESS;
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	irq_wait_ioctl
*
*	Purpose:
*
*		Suspend the calling thread for upto one second until a configured
*		interrupt occurs.
*
*	Arguments:
*
*		dev	The data for the device of interest.
*
*		arg	The IOCTL service's required argument.
*
*	Returned:
*
*		0	All went well.
*		<0	An error code for the problem seen.
*
******************************************************************************/

int irq_wait_ioctl(dev_data_t* dev, gsc_drv_irq_t* irq)
{
	dev_pid_t*			pid;
	int					status;
	int					test;
	int					timeout;
	WAIT_QUEUE_ENTRY_T	wait;

	for (;;)	// A convenience loop.
	{
		pid	= pid_locate(dev);

		if (pid == NULL)
		{
			status		= -ENODEV;
			irq->status	= GSC_INVALID_DATA;
			break;
		}

		if (pid->queued)
		{
			status		= -EBUSY;
			irq->status	= GSC_ACCESS_DENIED;
			break;
		}

		if (pid->irq_cancel)
		{
			status			= 0;
			pid->irq_cancel	= 0;
			irq->status		= GSC_WAIT_CANCELED;
			break;
		}

		if (irq->wait_seconds > 60)
			irq->wait_seconds	= 60;
		else if (irq->wait_seconds < 1)
			irq->wait_seconds	= 1;

		/*
		*	Do the magic!
		*/

		status	= _irq_access_lock(dev, 0);

		if (status)
		{
			irq->status	= GSC_ACCESS_DENIED;
			break;
		}

		memset(&wait, 0, sizeof(wait));
		WAIT_QUEUE_ENTRY_INIT(&wait, current);
		SET_CURRENT_STATE(TASK_INTERRUPTIBLE);
		add_wait_queue(&pid->queue, &wait);
		pid->queued		= 1;
		pid->timeout	= 1;
		up(&dev->sem);		// So others can do things.
		_irq_access_unlock(dev, 0);
		EVENT_WAIT_IRQ_TO(&pid->queue, pid->condition, irq->wait_seconds * HZ);
		pid->queued		= 0;

		// We must be able to regain control.
		do
		{
			test	= down_interruptible(&dev->sem);
		} while (test);

		timeout			= pid->timeout;
		pid->timeout	= 0;
		remove_wait_queue(&pid->queue, &wait);
		SET_CURRENT_STATE(TASK_RUNNING);

		if (pid->irq_cancel)
			irq->status	= GSC_WAIT_CANCELED;
		else if (timeout)
			irq->status	= GSC_WAIT_TIMEOUT;
		else
			irq->status	= GSC_SUCCESS;

		pid->irq_cancel	= 0;
		status			= 0;
		break;
	}

	return(status);
}



