/* API - DMI32 - Any OS */
/* $Workfile: dev_access.c $ */
/* $Revision: 7 $ */
/* $Date: 4/05/07 3:41p $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/api/dev_access.c $
 *
 * 7     4/05/07 3:41p Dnewbold
 * Corrected BAR mapping during initialization.
 *
 * 6     4/02/07 5:27p Dnewbold
 * Added #include <string.h>
 *
 * 5     3/01/07 8:19a Dnewbold
 * Linted and updated.
 *
 * 4     5/09/06 1:37p Dnewbold
 *
 * 3     5/05/06 10:39a Dnewbold
 * Worked to increase modularity.
 *
 * 2     5/04/06 3:58p Dnewbold
 * Removed key validation code and increased modularity.
 *
 * 1     8/22/05 3:40p Dnewbold
*/

#include <string.h>

#include "dev_access.h"

#include "gsc_version.h"
#include "gsc_drv.h"
#include "gsc_sleep.h"

#include "dev_config.h"
#include "dev_irq.h"
#include "dev_misc.h"
#include "dev_reg.h"
#include "dev_version.h"



/* typedefs	*****************************************************************/

#define _REMAP_DRAM		(0x00000000 | BAR2_REMAP_ENABLE)



/* variables	*************************************************************/

static	dev_data_t	_dev_list[GSC_PROCESS_OPEN_MAX];
static	gsc_sem_t	_dev_sem;



/*****************************************************************************
*
*	Function:	_app_handle_validate
*
*	Purpose:
*
*		Validate the handle given by the application and provide the device
*		data structure pointer, if applicable.
*
*	Arguments:
*
*		handle		The handle to validate.
*
*		dev			The structure pointer is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _app_handle_validate(void* handle, dev_data_t** dev)
{
	U8	lst_index;
	U32	status;

	status	= gsc_handle_validate(	handle,
									ARRAY_ELEMENTS(_dev_list),
									sizeof(_dev_list[0]),
									&_dev_list[0].device,
									&lst_index);

	if (status == (U32) GSC_SUCCESS)
		dev[0]	= &_dev_list[lst_index];
	else
		dev[0]	= NULL;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dev_access_lock
*
*	Purpose:
*
*		Get exclusive access to the given device.
*
*	Arguments:
*
*		dev			The device to lock.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dev_access_lock(dev_data_t* dev)
{
	U32	status;

	status	= gsc_sem_lock(&dev->sem);
	return(status);
}



/*****************************************************************************
*
*	Function:	_dev_data_create
*
*	Purpose:
*
*		Prepare the given structure for initial use with the respective
*		device. This function is primarily for items which do not change
*		while the device is open.
*
*	Arguments:
*
*		dev			The data structure to initialize.
*					We retain the device substructure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dev_data_create(dev_data_t* dev)
{
	gsc_device_t	device;
	U32				stat;
	U32				status	= (U32) GSC_SUCCESS;

	device		= dev->device;
	memset(dev, 0, sizeof(dev[0]));
	dev->device	= device;

	stat	= gsc_sem_create(&dev->sem, "DEV");
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_sem_create(&dev->dram.sem, "DRAM");
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_bar_create(&dev->plx_regs, &dev->device, 0, GSC_PCI_9080_BAR0);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;
	stat	= gsc_bar_create(&dev->gsc_regs, &dev->device, 2, GSC_PCI_9080_BAR2);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_dma_create(&dev->dma, &dev->device);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_io_create(&dev->rx.gio,
							"I/O Rx",
							&dev->device,
							dev,
							&dev->rx,
							&dev->gsc_regs,
							&dev->dma,
							0);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_io_create(&dev->tx.gio,
							"I/O Tx",
							&dev->device,
							dev,
							&dev->tx,
							&dev->gsc_regs,
							&dev->dma,
							1);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_mem_create(&dev->rx.mem, "I/O Rx mem", &dev->device);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;
	stat	= gsc_mem_create(&dev->tx.mem, "I/O Tx mem", &dev->device);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	/* Interrupt Notification */
	stat	= gsc_irq_create(&dev->irq.girq, &dev->device);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_irq_plx_create(	&dev->irq.girq,
									"IRQ DMA 0",
									GSC_PLX_INT_DMA_0,
									&dev->dma.ch0.irq);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_irq_plx_create(	&dev->irq.girq,
									"IRQ DMA 1",
									GSC_PLX_INT_DMA_1,
									&dev->dma.ch1.irq);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_irq_plx_create(	&dev->irq.girq,
									"IRQ Local",
									GSC_PLX_INT_LOCAL,
									&dev->irq.local);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	// Buffer Start
	stat	= gsc_irq_local_create(	&dev->irq.girq,
									"IRQ Local Buffer Start",
									0,
									&dev->irq.locals[0]);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	// Buffer End
	stat	= gsc_irq_local_create(	&dev->irq.girq,
									"IRQ Local Buffer End",
									1,
									&dev->irq.locals[1]);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	// User I/O A
	stat	= gsc_irq_local_create(	&dev->irq.girq,
									"IRQ Local User I/O A",
									2,
									&dev->irq.locals[2]);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	// User I/O B
	stat	= gsc_irq_local_create(	&dev->irq.girq,
									"IRQ Local User I/O B",
									3,
									&dev->irq.locals[3]);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	// Multi-Buffer Under Run
	stat	= gsc_irq_local_create(	&dev->irq.girq,
									"IRQ Local Multi-Buffer Under Run",
									4,
									&dev->irq.locals[4]);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dev_data_destroy
*
*	Purpose:
*
*		Close the given data structure as the referenced device in no longer
*		being used.
*
*	Arguments:
*
*		dev			The data structure to initialize.
*					We retain the device substructure.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _dev_data_destroy(dev_data_t* dev)
{
	gsc_device_t	device;

	gsc_sem_destroy(&dev->sem);

	gsc_sem_destroy(&dev->dram.sem);

	gsc_bar_destroy(&dev->plx_regs);
	gsc_bar_destroy(&dev->gsc_regs);

	gsc_dma_destroy(&dev->dma);

	gsc_io_destroy(&dev->rx.gio);
	gsc_io_destroy(&dev->tx.gio);

	gsc_mem_destroy(&dev->tx.mem);
	gsc_mem_destroy(&dev->rx.mem);

	gsc_irq_destroy(&dev->irq.girq);

	device		= dev->device;
	memset(dev, 0, sizeof(dev[0]));
	dev->device	= device;
}



/*****************************************************************************
*
*	Function:	_dev_reset
*
*	Purpose:
*
*		Reset the DMI32.
*
*	Arguments:
*
*		dev			The device to reset.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dev_reset(dev_data_t* dev)
{
	U32	stat;
	U32	status	= (U32) GSC_SUCCESS;

	stat	= io_abort(dev, DMI32_WHICH_RX, DMI32_IO_ABORT_YES, NULL);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;
	stat	= io_abort(dev, DMI32_WHICH_TX, DMI32_IO_ABORT_YES, NULL);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_sem_lock(&dev->dram.sem);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	if (stat == (U32) GSC_SUCCESS)
	{
		// Access the GSC registers.
		dev->dram.pci_bus	= 0;
		dev->dram.users		= 0;
		stat	= PLX_REG_WRITE(dev, GSC_PLX_9080_LAS0BA, BAR2_REMAP_FW_REGS);
		status	= (status == (U32) GSC_SUCCESS) ? stat : status;
		gsc_sem_unlock(&dev->dram.sem);

		// Put the board in the default state we need it in.
		stat	= GSC_REG_WRITE(dev, DMI32_BCR, DMI32_BCR_BOARD_RESET);
		status	= (status == (U32) GSC_SUCCESS) ? stat : status;
		gsc_sleep_ms(1);	// Sleep for 1ms though a reset takes only 1us.
		stat	= GSC_REG_WRITE(dev, DMI32_ICR, DMI32_ICR_DEFAULT);
		status	= (status == (U32) GSC_SUCCESS) ? stat : status;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_dev_init
*
*	Purpose:
*
*		Initialize the given device to its initial open() state. This is for
*		items which may change while the device is open.
*
*	Arguments:
*
*		dev			The device to initialize.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dev_init(dev_data_t* dev)
{
	U32	misc;
	U32	stat;
	U32	status	= (U32) GSC_SUCCESS;
	U32	rx_sem;
	U32	tx_sem;

	stat	= _dev_reset(dev);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_sem_lock(&dev->tx.gio.sem);
	tx_sem	= stat == (U32) GSC_SUCCESS;
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_sem_lock(&dev->rx.gio.sem);
	rx_sem	= stat == (U32) GSC_SUCCESS;
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	if (dev->plx_regs.ptr == NULL)
	{
		stat	= gsc_bar_map(&dev->plx_regs);
		status	= (status == (U32) GSC_SUCCESS) ? stat : status;
	}

	if (dev->gsc_regs.ptr == NULL)
	{
		stat	= gsc_bar_map(&dev->gsc_regs);
		status	= (status == (U32) GSC_SUCCESS) ? stat : status;
	}

	// Cache some information.
	stat	= GSC_REG_READ(dev, DMI32_FRR, &dev->cache.frr);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;
	stat	= GSC_REG_READ(dev, DMI32_FR, &dev->cache.fr);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	misc	= DMI32_FRR_MISC_DECODE(dev->cache.frr);

	if (misc & DMI32_FRR_MISC_FR)
	{
		dev->features.features_register	= 1;
	}
	else
	{
		dev->cache.fr					= 0;
		dev->features.features_register	= 0;
	}

	if (dev->cache.fr & DMI32_FR_4GB)
	{
		dev->dram_capacity_mb			= 4L * 1024;
		dev->features.irq_mbur			= 1;
		dev->features.lmb_register		= 1;
		dev->features.start_word_dir	= 1;
	}
	else
	{
		dev->dram_capacity_mb			= 2L * 1024;
		dev->features.irq_mbur			= 0;
		dev->features.lmb_register		= 0;
		dev->features.start_word_dir	= 0;
	}

	// Other items.
	dev->strict_args	= DMI32_MISC_STRICT_ARGUMENTS_DEFAULT;
	dev->strict_config	= DMI32_MISC_STRICT_CONFIG_DEFAULT;
	dev->strict_dram	= DMI32_MISC_STRICT_DRAM_LIMITS_DEFAULT;

	stat	= io_dma_init(dev);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= io_init(dev, &dev->rx);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;
	stat	= io_init(dev, &dev->tx);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= irq_init(dev);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_irq_enable(&dev->irq.girq);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	if (rx_sem)
		gsc_sem_unlock(&dev->rx.gio.sem);

	if (tx_sem)
		gsc_sem_unlock(&dev->tx.gio.sem);

	return(status);
}



/*****************************************************************************
*
*	Function:	_dev_list_access_lock
*
*	Purpose:
*
*		Get exclusive access to our device list.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dev_list_access_lock(void)
{
	U32	status;

	status	= gsc_sem_lock(&_dev_sem);
	return(status);
}



/*****************************************************************************
*
*	Function:	_dev_list_access_unlock
*
*	Purpose:
*
*		Release exclusive access to our device list.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _dev_list_access_unlock(void)
{
	gsc_sem_unlock(&_dev_sem);
}



/*****************************************************************************
*
*	Function:	_dev_list_index_acquire
*
*	Purpose:
*
*		Locate an unused device list entry.
*
*	Arguments:
*
*		index		The index obtained is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dev_list_index_acquire(int* index)
{
	int	i;
	U32	status	= (U32) GSC_TOO_MANY_OPEN_HANDLES;

	for (i = 0; i < (int) ARRAY_ELEMENTS(_dev_list); i++)
	{
		if (GSC_DEVICE_IS_OPEN(&_dev_list[i].device))
			continue;

		status		= (U32) GSC_SUCCESS;
		index[0]	= i;
		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dev_access_library_begin
*
*	Purpose:
*
*		Initialize the library prior to being used.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void dev_access_library_begin(void)
{
	api_initialized	= TRUE;
	memset(_dev_list, 0, sizeof(_dev_list));

	if (api_status == (U32) GSC_SUCCESS)
		api_status	= gsc_sem_create(&_dev_sem, "DEV LIST");

	if (api_status == (U32) GSC_SUCCESS)
		gsc_compute_base_index();

	if (api_status == (U32) GSC_SUCCESS)
		api_status	= dev_version_verify();

	if (api_status == (U32) GSC_SUCCESS)
		api_status	= config_init();

	if (api_status == (U32) GSC_SUCCESS)
		api_status	= gsc_drv_init();
}



/*****************************************************************************
*
*	Function:	dev_access_library_end
*
*	Purpose:
*
*		Release all resources as the library is being unloaded.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void dev_access_library_end(void)
{
	int	i;

	if (api_initialized)
	{
		// Close all devices and unmap PLX and GSC registers.

		for (i = 0; i < (int) ARRAY_ELEMENTS(_dev_list); i++)
			dmi32_close(GSC_GET_HANDLE(&_dev_list[i].device));

		// End library usage.
		_dev_list_access_lock();
		gsc_sem_destroy(&_dev_sem);
		api_status		= (U32) GSC_SUCCESS;
		api_initialized	= 0;
	}
}



/*****************************************************************************
*
*	Function:	dev_access_lock
*
*	Purpose:
*
*		Validate the application handle, and if valid, acquire the semaphore
*		for the device and provide the caller with a pointer to the device
*		structure.
*
*	Arguments:
*
*		handle		The handle to validate.
*
*		dev			The structure pointer is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 dev_access_lock(void* handle, dev_data_t** dev)
{
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		dev[0]	= NULL;

		if (api_status != (U32) GSC_SUCCESS)
		{
			status	= (U32) GSC_INIT_FAILURE;
			break;
		}

		status	= _dev_list_access_lock();

		if (status != (U32) GSC_SUCCESS)
			break;

		status	= _app_handle_validate(handle, dev);

		if (status != (U32) GSC_SUCCESS)
		{
			_dev_list_access_unlock();
			break;
		}

		status	= _dev_access_lock(dev[0]);
		_dev_list_access_unlock();
		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dev_access_unlock
*
*	Purpose:
*
*		Release exclusive access to the given device.
*
*	Arguments:
*
*		dev		The device to unlock.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void dev_access_unlock(dev_data_t* dev)
{
	gsc_sem_unlock(&dev->sem);
}



/*****************************************************************************
*
*	Function:	dmi32_board_count
*
*	Purpose:
*
*		Count the number of installed boards.
*
*	Arguments:
*
*		count		The count is recorded here upon success.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_board_count(U8* count)
{
	U32	status;

	if (api_status != (U32) GSC_SUCCESS)
		status	= (U32) GSC_INIT_FAILURE;
	else if (count == NULL)
		status	= (U32) GSC_NULL_PARAM;
	else
		status	= gsc_count_boards(count);

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_close
*
*	Purpose:
*
*		Close an open communications channel.
*
*	Arguments:
*
*		handle		The handle of the channel to close.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_close(void* handle)
{
	dev_data_t*	dev;
	U32			status;

	for (;;)	/* A convenience loop. */
	{
		if (api_status != (U32) GSC_SUCCESS)
		{
			status	= (U32) GSC_INIT_FAILURE;
			break;
		}

		status	= _dev_list_access_lock();

		if (status != (U32) GSC_SUCCESS)
			break;

		status	= _app_handle_validate(handle, &dev);

		if (status != (U32) GSC_SUCCESS)
		{
			_dev_list_access_unlock();
			break;
		}

		status	= _dev_access_lock(dev);

		if (status != (U32) GSC_SUCCESS)
		{
			_dev_list_access_unlock();
			break;
		}

		// We ignore status values from here as we ARE closing the device.
		_dev_reset(dev);
		gsc_irq_disable(&dev->irq.girq);
		_dev_data_destroy(dev);
		status	= gsc_close(&dev->device);
		_dev_list_access_unlock();
		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_init
*
*	Purpose:
*
*		Initialize the referenced device and our recorded settings to the
*		initial open() state.
*
*	Arguments:
*
*		handle		The handle of the device to access.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_init(void* handle)
{
	dev_data_t*	dev;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		status	= _dev_init(dev);
		dev_access_unlock(dev);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_open
*
*	Purpose:
*
*		Open a communications channel to an DMI32.
*
*	Arguments:
*
*		index		The index of the board to access.
*
*		handle		The resulting handle is recorded here. It is NULL if
*					open fails.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_open(U8 index, void** handle)
{
	int	lst_index;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		if (api_status != (U32) GSC_SUCCESS)
		{
			status	= (U32) GSC_INIT_FAILURE;
			break;
		}

		if (handle == NULL)
		{
			status	= (U32) GSC_NULL_PARAM;
			break;
		}

		handle[0]	= NULL;
		status		= _dev_list_access_lock();

		if (status != (U32) GSC_SUCCESS)
			break;

		status	= _dev_list_index_acquire(&lst_index);

		if (status != (U32) GSC_SUCCESS)
		{
			_dev_list_access_unlock();
			break;
		}

		status	= gsc_open(	(U8) index,
							(U8) lst_index,
							&_dev_list[lst_index].device,
							handle);

		if (status == (U32) GSC_SUCCESS)
			status	= _dev_data_create(&_dev_list[lst_index]);

		if (status == (U32) GSC_SUCCESS)
			status	= _dev_init(&_dev_list[lst_index]);

		if (status == (U32) GSC_SUCCESS)
			status	= misc_dram_compute_size(&_dev_list[lst_index]);

		if (status != (U32) GSC_SUCCESS)
		{
			handle[0]	= NULL;
			_dev_data_destroy(&_dev_list[lst_index]);
			gsc_close(&_dev_list[lst_index].device);
		}

		_dev_list_access_unlock();
		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_reset
*
*	Purpose:
*
*		Reset the DMI32.
*
*	Arguments:
*
*		handle		The handle of the device to access.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_reset(void* handle)
{
	dev_data_t*	dev;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		status	= _dev_reset(dev);
		dev_access_unlock(dev);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_api_status
*
*	Purpose:
*
*		Provide the caller with the library's recorded status.
*
*	Arguments:
*
*		stat		The recorded status is put here.
*
*		arg			Any associated data value is recorded here.
*
*		api_ver		This is the API version number the application was for.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_api_status(U32* stat, U32* arg, U32 api_ver)
{
	U32	status;

	if ((stat == NULL) || (arg == NULL))
	{
		status	= (U32) GSC_NULL_PARAM;
	}
	else
	{
		status	= (U32) GSC_SUCCESS;
		stat[0]	= api_status;
		arg[0]	= api_status_arg;

		if ((api_status == (U32) GSC_SUCCESS) && (api_ver != DMI32_API_VERSION))
			stat[0]	= (U32) GSC_INVALID_VERSION_API;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dev_access_dram
*
*	Purpose:
*
*		Configure the board to access DRAM as given.
*
*	Arguments:
*
*		dev			The device to access.
*
*		acquire		Acquire access or release it.
*
*		pci_bus		0:		PCI bus accesses registers, cable accessses DRAM
*					else:	PCI bus accesses DRAM
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 dev_access_dram(dev_data_t* dev, int acquire, int pci_bus)
{
	U32	bsr;
	U32	status;

	status	= gsc_sem_lock(&dev->dram.sem);

	if (status != (U32) GSC_SUCCESS)
	{
	}
	else if (acquire == 0)
	{
		// Release access.

		if (dev->dram.users)
			dev->dram.users--;

		if ((dev->dram.users == 0) && (dev->dram.pci_bus))
		{
			// The device is not being accessed any longer but is
			// presently configured for DRAM access via the PCI bus.
			// Make it so the registers are access from the PCI bus.
			status	= PLX_REG_WRITE(dev, GSC_PLX_9080_LAS0BA, BAR2_REMAP_FW_REGS);
			dev->dram.pci_bus	= 0;
		}

		gsc_sem_unlock(&dev->dram.sem);
	}
	else
	{
		// Acquire the given access.
		if (dev->dram.users == 0)
		{
			/*
			*	No access is currently registered. Base our action on what we
			*	want and what we've got right now.
			*/

			if ((pci_bus ? 1 : 0) == (dev->dram.pci_bus ? 1 : 0))
			{
				// Things are already exactly what we're looking for.
			}
			else if (dev->dram.pci_bus)
			{
				/*
				*	DRAM access is now at the PCI bus, but we need to switch
				*	it to the cable so we can get PCI bus access to the
				*	register. When we're done, we'll have ...
				*
				*		Cable access to DRAM is permitted.
				*		PCI bus access is to the registers is permitted.
				*/

				status	= PLX_REG_WRITE(dev, GSC_PLX_9080_LAS0BA, BAR2_REMAP_FW_REGS);
				dev->dram.pci_bus	= 0;
			}
			else
			{
				/*
				*	DRAM access is now at the cable interface, but we need to
				*	switch it to the PCI bus so we can perform I/O to DRAM.
				*	We can't switch though, if data transfer is enabled.
				*	When we're done, we'll have ...
				*
				*		PCI bus access to DRAM is permitted.
				*		PCI bus access to the registers is NOT permitted.
				*/

				status	= gsc_bar_reg_read(&dev->gsc_regs, DMI32_BSR, &bsr);

				if ((status == (U32) GSC_SUCCESS) && (bsr & DMI32_BCR_XFER_ENABLE))
				{
					// Data transfer is in progress so we can't switch.
					status	= (U32) GSC_ACCESS_DENIED;
				}

				if (status == (U32) GSC_SUCCESS)
				{
					/*
					*	Switch DRAM access to the cable so the PCI bus sees
					*	the registers.
					*/

					status	= PLX_REG_WRITE(dev, GSC_PLX_9080_LAS0BA, _REMAP_DRAM);
					dev->dram.pci_bus	= 1;
				}
			}

			if (status == (U32) GSC_SUCCESS)
				dev->dram.users	= 1;
		}
		else if ((pci_bus ? 1 : 0) == (dev->dram.pci_bus ? 1 : 0))
		{
			// Things are already locked exactly like we want it.
			dev->dram.users++;
		}
		else
		{
			// Can't gain access right now.
			status	= (U32) GSC_ACCESS_DENIED;
		}

		gsc_sem_unlock(&dev->dram.sem);
	}

	return(status);
}


