/* API - DMI32 - Any OS */
/* $Workfile: dev_io.c $ */
/* $Revision: 5 $ */
/* $Date: 10/27/07 10:44a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/api/dev_io.c $
 *
 * 5     10/27/07 10:44a Dnewbold
 * Additional driver porting updates.
 *
 * 4     3/01/07 8:23a Dnewbold
 * Linted.
 *
 * 3     5/09/06 1:37p Dnewbold
 *
 * 2     5/04/06 3:58p Dnewbold
 * Removed key validation code and increased modularity.
 *
 * 1     8/22/05 3:40p Dnewbold
*/

#include "dev_io.h"

#include "dev_access.h"
#include "dev_config.h"
#include "dev_reg.h"



/*****************************************************************************
*
*	Function:	_io_common_parameter_which
*
*	Purpose:
*
*		Provide common code for processing I/O parameters based on the WHICH
*		argument.
*
*	Arguments:
*
*		dev			The device to access.
*
*		func		The function to call for parameter specific processing.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_common_parameter_which(	dev_data_t* dev,
										U32			(*func)(
														dev_data_t* dev,
														dev_io_t*	io,
														UL			set,
														UL*			get),
										U32			which,
										UL			set,
										UL*			get)
{
	U32	stat;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, DMI32_WHICH_TX_RX);
		if (status != (U32) GSC_SUCCESS)	break;

		if (which & DMI32_WHICH_TX)
		{
			stat	= (func)(dev, &dev->tx, set, get);
			status	= (status == (U32) GSC_SUCCESS) ? stat : status;
		}

		if (which & DMI32_WHICH_RX)
		{
			stat	= (func)(dev, &dev->rx, set, get);
			status	= (status == (U32) GSC_SUCCESS) ? stat : status;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_io_abort_side
*
*	Purpose:
*
*		Implement the I/O Abort parameter service for a single Tx/Rx side.
*		Validate and apply the setting, as appropriate, then retrieve the
*		current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		io			The I/O structure for the Tx/Rx side to access.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_abort_side(	dev_data_t*	dev,
							dev_io_t*	io,
							UL			set,
							UL*			get)
{
	U32	status	= (U32) GSC_SUCCESS;

	for (;;)	/* A convenience loop. */
	{
		/*
		*	Ordinarily we would get exclusive access. We don't do that
		*	here since an in progress I/O already has exclusive access.
		*/

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_IO_ABORT_DEFAULT;
		}
		else if (	(set == DMI32_IO_ABORT_NO) ||
					(set == DMI32_IO_ABORT_YES))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_IO_ABORT_YES;
		}
		else
		{
			set	= DMI32_IO_ABORT_NO;
		}

		if (set == DMI32_IO_ABORT_YES)
			status	= gsc_io_abort(&io->gio);
		else if (set != GSC_NO_CHANGE)
			io->gio.abort	= set;

		if (get)
			get[0]	= io->gio.abort;

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	io_abort
*
*	Purpose:
*
*		Implement the I/O Abort parameter service. Validate and apply the
*		setting(s), as appropriate, then retrieve the current setting(s) for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 io_abort(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	status	= _io_common_parameter_which(	dev,
											_io_abort_side,
											which,
											set,
											get);
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_aborted_side
*
*	Purpose:
*
*		Implement the I/O Aborted parameter service for a single Tx/Rx side.
*		Validate and apply the setting, as appropriate, then retrieve the
*		current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		io			The I/O structure for the Tx/Rx side to access.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_aborted_side(dev_data_t*	dev,
							dev_io_t*	io,
							UL			set,
							UL*			get)
{
	U32	status	= (U32) GSC_SUCCESS;

	for (;;)	/* A convenience loop. */
	{
		/*
		*	Ordinarily we would get exclusive access. We don't do that
		*	here since an in progress I/O already has exclusive access.
		*/

		// SET is not supported by this parameter.

		if (get)
			get[0]	= io->gio.aborted;

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	io_aborted
*
*	Purpose:
*
*		Implement the I/O Aborted parameter service. Validate and apply the
*		setting(s), as appropriate, then retrieve the current setting(s) for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 io_aborted(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	status	= _io_common_parameter_which(	dev,
											_io_aborted_side,
											which,
											set,
											get);
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_callback_arg_side
*
*	Purpose:
*
*		Implement the I/O Callback Argument parameter service for a single
*		Tx/Rx side. Validate and apply the setting, as appropriate, then
*		retrieve the current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		io			The I/O structure for the Tx/Rx side to access.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_callback_arg_side(	dev_data_t*		dev,
									dev_io_t*		io,
									unsigned long	set,
									unsigned long*	get)
{
	U32			status	= (U32) GSC_SUCCESS;
	gsc_event_t	tmp;

	for (;;)	/* A convenience loop. */
	{
		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_IO_CALLBACK_ARG_DEFAULT;
		}

		if (set != GSC_NO_CHANGE)
		{
			status	= gsc_io_callback_config(
						&io->gio,
						(void*) GSC_NO_CHANGE,	// id
						(void*) GSC_NO_CHANGE,	// function
						(void*) GSC_NO_CHANGE,	// arg1
						set,
						GSC_NO_CHANGE,
						NULL);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= gsc_io_callback_config(
						&io->gio,
						(void*) GSC_NO_CHANGE,	// id
						(void*) GSC_NO_CHANGE,	// function
						(void*) GSC_NO_CHANGE,	// arg1
						GSC_NO_CHANGE,
						GSC_NO_CHANGE,
						&tmp);
			if (status != (U32) GSC_SUCCESS)	break;
			get[0]	= tmp.arg2;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	io_callback_arg
*
*	Purpose:
*
*		Implement the I/O Callback Argument parameter service. Validate and
*		apply the setting(s), as appropriate, then retrieve the current
*		setting(s) for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 io_callback_arg(dev_data_t* dev, U32 which, unsigned long set, unsigned long* get)
{
	U32	status;

	status	= _io_common_parameter_which(	dev,
											_io_callback_arg_side,
											which,
											set,
											get);
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_callback_func_side
*
*	Purpose:
*
*		Implement the I/O Callback Function parameter service for a single
*		Tx/Rx side. Validate and apply the setting, as appropriate, then
*		retrieve the current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		io			The I/O structure for the Tx/Rx side to access.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_callback_func_side(	dev_data_t*		dev,
									dev_io_t*		io,
									unsigned long	set,
									unsigned long*	get)
{
	U32			status	= (U32) GSC_SUCCESS;
	gsc_event_t	tmp;

	for (;;)	/* A convenience loop. */
	{
		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_IO_CALLBACK_FUNC_DEFAULT;
		}

		if (set != GSC_NO_CHANGE)
		{
			status	= gsc_io_callback_config(
						&io->gio,
						(void*) GSC_NO_CHANGE,	// id
						(void*) set,			// function
						(void*) GSC_NO_CHANGE,	// arg1
						GSC_NO_CHANGE,
						GSC_NO_CHANGE,
						NULL);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= gsc_io_callback_config(
						&io->gio,
						(void*) GSC_NO_CHANGE,	// id
						(void*) GSC_NO_CHANGE,	// function
						(void*) GSC_NO_CHANGE,	// arg1
						GSC_NO_CHANGE,
						GSC_NO_CHANGE,
						&tmp);
			if (status != (U32) GSC_SUCCESS)	break;
			get[0]	= (unsigned long) tmp.callback;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	io_callback_func
*
*	Purpose:
*
*		Implement the I/O Callback Function parameter service. Validate and
*		apply the setting(s), as appropriate, then retrieve the current
*		setting(s) for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 io_callback_func(dev_data_t* dev, U32 which, unsigned long set, unsigned long* get)
{
	U32	status;

	status	= _io_common_parameter_which(	dev,
											_io_callback_func_side,
											which,
											set,
											get);
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_buffer_pointer_side
*
*	Purpose:
*
*		Implement the I/O Buffer Pointer parameter service for a single Tx/Rx
*		side. Validate and apply the setting, as appropriate, then retrieve
*		the current setting for the caller, also as appropriate. This is a
*		GET only service.
*
*	Arguments:
*
*		dev			The device to access.
*
*		io			The I/O structure for the Tx/Rx side to access.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_buffer_pointer_side(	dev_data_t*	dev,
									dev_io_t*	io,
									UL			set,
									UL*			get)
{
	U32			status;
	gsc_mem_t	tmp;
	U32			unlock;

	for (;;)	/* A convenience loop. */
	{
		status	= gsc_sem_lock(&io->gio.sem);
		unlock	= status == (U32) GSC_SUCCESS;
		if (status != (U32) GSC_SUCCESS)	break;

		// SET not supported for this parameter.

		if (get)
		{
			status	= gsc_mem_alloc(&io->mem, GSC_NO_CHANGE, &tmp);
			if (status != (U32) GSC_SUCCESS)	break;
			get[0]	= tmp.usr_adrs;
		}

		break;
	}

	if (unlock)
		gsc_sem_unlock(&io->gio.sem);

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	io_buffer_pointer
*
*	Purpose:
*
*		Implement the I/O Buffer Pointer parameter service. Validate and
*		apply the setting(s), as appropriate, then retrieve the current
*		setting(s) for the caller, also as appropriate. This is a GET only
*		service.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 io_buffer_pointer(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	status	= _io_common_parameter_which(	dev,
											_io_buffer_pointer_side,
											which,
											set,
											get);
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_buffer_size_side
*
*	Purpose:
*
*		Implement the I/O Buffer Size parameter service for a single Tx/Rx
*		side. Validate and apply the setting, as appropriate, then retrieve
*		the current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		io			The I/O structure for the Tx/Rx side to access.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_buffer_size_side(dev_data_t*	dev,
								dev_io_t*	io,
								UL			set,
								UL*			get)
{
	U32			status		= (U32) GSC_SUCCESS;
	gsc_mem_t	tmp;
	U32			unlock_rx	= 0;
	U32			unlock_tx	= 0;

	for (;;)	/* A convenience loop. */
	{
		// Lock both as either might be in use.
		status		= gsc_sem_lock(&io->gio.dev->rx.gio.sem);
		unlock_rx	= status == (U32) GSC_SUCCESS;
		if (status != (U32) GSC_SUCCESS)	break;

		status		= gsc_sem_lock(&io->gio.dev->tx.gio.sem);
		unlock_tx	= status == (U32) GSC_SUCCESS;
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_IO_BUFFER_SIZE_DEFAULT;
		}
		else if ((set & DMI32_IO_BUFFER_SIZE_MASK) == set)
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else
		{
			set	&= DMI32_IO_BUFFER_SIZE_MASK;
		}

		if (set != GSC_NO_CHANGE)
		{
			status	= gsc_mem_alloc(&io->mem, set, NULL);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= gsc_mem_alloc(&io->mem, GSC_NO_CHANGE, &tmp);
			if (status != (U32) GSC_SUCCESS)	break;
			get[0]	= tmp.usr_adrs;
		}

		break;
	}

	if (unlock_rx)
		gsc_sem_unlock(&io->gio.dev->rx.gio.sem);

	if (unlock_tx)
		gsc_sem_unlock(&io->gio.dev->tx.gio.sem);

	return(status);
}



/*****************************************************************************
*
*	Function:	io_buffer_size
*
*	Purpose:
*
*		Implement the I/O Driver Buffer Size parameter service. Validate and
*		apply the setting(s), as appropriate, then retrieve the current
*		setting(s) for the caller, also as appropriate. This is a GET only
*		service.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 io_buffer_size(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	status	= _io_common_parameter_which(	dev,
											_io_buffer_size_side,
											which,
											set,
											get);
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_mode_side
*
*	Purpose:
*
*		Implement the I/O Mode parameter service for a single Tx/Rx side.
*		Validate and apply the setting, as appropriate, then retrieve the
*		current setting for the caller, also as appropriate. Since this
*		parameter affect the DMA Channel Selection process, the DMA Channel
*		Selection selection is evaluated.
*
*	Arguments:
*
*		dev			The device to access.
*
*		io			The I/O structure for the Tx/Rx side to access.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_mode_side(	dev_data_t*	dev,
							dev_io_t*	io,
							UL			set,
							UL*			get)
{
	U32	status;
	U32	unlock;

	for (;;)	/* A convenience loop. */
	{
		status	= gsc_sem_lock(&io->gio.sem);
		unlock	= status == (U32) GSC_SUCCESS;
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_IO_MODE_DEFAULT;
		}
		else if (	(set == DMI32_IO_MODE_PIO) ||
					(set == DMI32_IO_MODE_DMA))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else
		{
			set	= DMI32_IO_MODE_DEFAULT;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == DMI32_IO_MODE_DMA)
			{
				io->gio.dma_demand	= 0;
				io->gio.dma_enable	= 1;
			}
			else // DMI32_IO_MODE_PIO
			{
				io->gio.dma_demand	= 0;
				io->gio.dma_enable	= 0;
				gsc_dma_channel_close(&dev->dma, &io->gio.dma_channel);
			}
		}

		if (get)
		{
			if (io->gio.dma_enable)
				get[0]	= DMI32_IO_MODE_DMA;
			else
				get[0]	= DMI32_IO_MODE_PIO;
		}

		break;
	}

	if (unlock)
		gsc_sem_unlock(&io->gio.sem);

	return(status);
}



/*****************************************************************************
*
*	Function:	io_mode
*
*	Purpose:
*
*		Implement the I/O Mode parameter service. Validate and apply the
*		setting(s), as appropriate, then retrieve the current setting(s) for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 io_mode(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	status	= _io_common_parameter_which(	dev,
											_io_mode_side,
											which,
											set,
											get);
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_offset_side
*
*	Purpose:
*
*		Implement the I/O Offset parameter service for a single Tx/Rx side.
*		Validate and apply the setting, as appropriate, then retrieve the
*		current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		io			The I/O structure for the Tx/Rx side to access.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_offset_side(	dev_data_t*	dev,
							dev_io_t*	io,
							UL			set,
							UL*			get)
{
	U32	status;
	U32	unlock;

	for (;;)	/* A convenience loop. */
	{
		status	= gsc_sem_lock(&io->gio.sem);
		unlock	= status == (U32) GSC_SUCCESS;
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_IO_OFFSET_DEFAULT;
		}
		else if (	(set >= DMI32_IO_OFFSET_MIN) &&
					(set <= DMI32_IO_OFFSET_MAX) &&
					((set & DMI32_IO_OFFSET_MASK) == set))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set > DMI32_IO_OFFSET_MAX)
		{
			set	= DMI32_IO_OFFSET_MAX;
		}
		else if (set < DMI32_IO_OFFSET_MIN)
		{
			set	= DMI32_IO_OFFSET_MIN;
		}
		else
		{
			set	&= DMI32_IO_OFFSET_MASK;
		}

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (dev->strict_dram == 0)
		{
		}
		else if (dev->dram_size_mb >= 4096)
		{
		}
		else if (set <= (dev->dram_size_mb * 1024 * 1024 - 4))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else
		{
			set	=	(dev->dram_size_mb * 1024 * 1024) - 4;
		}

		if (set != GSC_NO_CHANGE)
			io->offset	= set;

		if (get)
			get[0]	= io->offset;

		break;
	}

	if (unlock)
		gsc_sem_unlock(&io->gio.sem);

	return(status);
}



/*****************************************************************************
*
*	Function:	io_offset
*
*	Purpose:
*
*		Implement the I/O Offset parameter service. Validate and apply the
*		setting(s), as appropriate, then retrieve the current setting(s) for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 io_offset(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	status	= _io_common_parameter_which(	dev,
											_io_offset_side,
											which,
											set,
											get);
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_overlap_enable_side
*
*	Purpose:
*
*		Implement the I/O Overlap Enable parameter service for a single Tx/Rx
*		side. Validate and apply the setting, as appropriate, then retrieve
*		the current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		io			The I/O structure for the Tx/Rx side to access.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_overlap_enable_side(	dev_data_t*	dev,
									dev_io_t*	io,
									UL			set,
									UL*			get)
{
	U32	status;
	U32	unlock;

	for (;;)	/* A convenience loop. */
	{
		status	= gsc_sem_lock(&io->gio.sem);
		unlock	= status == (U32) GSC_SUCCESS;
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_IO_OVERLAP_ENABLE_DEFAULT;
		}
		else if (	(set == DMI32_IO_OVERLAP_ENABLE_NO) ||
					(set == DMI32_IO_OVERLAP_ENABLE_YES))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_IO_OVERLAP_ENABLE_YES;
		}
		else
		{
			set	= DMI32_IO_OVERLAP_ENABLE_NO;
		}

		if (set != GSC_NO_CHANGE)
		{
			status	= gsc_io_overlap_enable(&io->gio.overlap, set, NULL);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= gsc_io_overlap_enable(&io->gio.overlap, GSC_NO_CHANGE, get);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		break;
	}

	if (unlock)
		gsc_sem_unlock(&io->gio.sem);

	return(status);
}



/*****************************************************************************
*
*	Function:	io_overlap_enable
*
*	Purpose:
*
*		Implement the I/O Overlap Enable parameter service. Validate and
*		apply the setting(s), as appropriate, then retrieve the current
*		setting(s) for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 io_overlap_enable(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	status	= _io_common_parameter_which(	dev,
											_io_overlap_enable_side,
											which,
											set,
											get);
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_status_side
*
*	Purpose:
*
*		Implement the I/O Status parameter service for a single Tx/Rx side.
*		Validate and apply the setting, as appropriate, then retrieve the
*		current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		io			The I/O structure for the Tx/Rx side to access.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_status_side(	dev_data_t*	dev,
							dev_io_t*	io,
							UL			set,
							UL*			get)
{
	U32	status	= (U32) GSC_SUCCESS;

	for (;;)	/* A convenience loop. */
	{
		/*
		*	We don't lock access as we want status now, not when any active
		*	I/O is done.
		*/

		// SET not supported for this parameter.

		if (get)
		{
			status	= gsc_io_status(&io->gio, get);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	io_status
*
*	Purpose:
*
*		Implement the I/O Status parameter service. Validate and apply the
*		setting(s), as appropriate, then retrieve the current setting(s) for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 io_status(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	status	= _io_common_parameter_which(	dev,
											_io_status_side,
											which,
											set,
											get);
	return(status);
}



/*****************************************************************************
*
*	Function:	_io_timeout_side
*
*	Purpose:
*
*		Implement the I/O Timeout parameter service for a single Tx/Rx side.
*		Validate and apply the setting, as appropriate, then retrieve the
*		current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		io			The I/O structure for the Tx/Rx side to access.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_timeout_side(dev_data_t*	dev,
							dev_io_t*	io,
							UL			set,
							UL*			get)
{
	U32	status;
	U32	unlock;

	for (;;)	/* A convenience loop. */
	{
		status	= gsc_sem_lock(&io->gio.sem);
		unlock	= status == (U32) GSC_SUCCESS;
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_IO_TIMEOUT_DEFAULT;
		}
		else if (set > DMI32_IO_TIMEOUT_MAX)
		{
			if (dev->strict_args)
			{
				status	= (U32) GSC_INVALID_DATA;
				break;
			}
			else
			{
				set	= DMI32_IO_TIMEOUT_MAX;
			}
		}

		if (set != GSC_NO_CHANGE)
			io->gio.timeout_s	= set;

		if (get)
			get[0]	= io->gio.timeout_s;

		break;
	}

	if (unlock)
		gsc_sem_unlock(&io->gio.sem);

	return(status);
}



/*****************************************************************************
*
*	Function:	io_timeout
*
*	Purpose:
*
*		Implement the I/O Timeout parameter service. Validate and apply the
*		setting(s), as appropriate, then retrieve the current setting(s) for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 io_timeout(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	status	= _io_common_parameter_which(	dev,
											_io_timeout_side,
											which,
											set,
											get);
	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_io_wait
*
*	Purpose:
*
*		Wait for any one of the referenced I/O transfers to complete.
*
*	Arguments:
*
*		handle		The device to access.
*
*		which		The I/O transfer to wait on.
*
*		timeout_ms	Return if a completion doesn't occur within this number
*					of milliseconds.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_io_wait(void* handle, U32 which, U32 timeout_ms)
{
	dev_data_t*	dev;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		status	= config_which_validate(dev, which, DMI32_WHICH_TX_RX);

		if (status != (U32) GSC_SUCCESS)
		{
			dev_access_unlock(dev);
		}
		else
		{
			status	= gsc_io_wait(
							&dev->sem,
							(which & DMI32_WHICH_TX) ? &dev->tx.gio : NULL,
							(which & DMI32_WHICH_RX) ? &dev->rx.gio : NULL,
							timeout_ms);
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_io_read
*
*	Purpose:
*
*		Perform a data read operation. This is the common routine called for
*		both blocking and overlapped I/O.
*
*	Arguments:
*
*		dev			The device to access.
*
*		buffer		The data read is put here.
*
*		bytes		The number of data bytes to read.
*
*		count		The actual number of bytes read is put here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_read(dev_data_t* dev, void* buffer, U32 bytes, U32* count)
{
	U32	status;

	// The I/O semaphore is already locked.
	dev->rx.pio_active		= 0;
	dev->rx.gio.abort		= 0;
	dev->rx.gio.aborted		= 0;
	dev->rx.gio.buffer		= buffer;
	dev->rx.gio.xfer_buf	= buffer;
	dev->rx.gio.xfer_tot	= bytes;
	dev->rx.gio.xfered_tot	= 0;
	dev->rx.gio.fifo_like	= 0;
	dev->rx.gio.loc_adrs	= dev->rx.offset;

	if (dev->rx.gio.dma_enable == 0)
	{
		// PIO via BAR2 mapping into DRAM.
		dev->rx.pio_active	= 1;

		if (dev->tx.pio_active)
			status	= (U32) GSC_ACCESS_DENIED;	// Only one at a time.
		else
			status	= dev_access_dram(dev, 1, 1);

		if (status == (U32) GSC_SUCCESS)
		{
			// We must have DRAM access from the PCI bus.
			dev->rx.gio.func	= &io_pio_read_funcs;
			status				= gsc_io_transfer_work(&dev->rx.gio);
			dev_access_dram(dev, 0, 1);
		}

		dev->rx.pio_active	= 0;
	}
	else
	{
		// Blind DMA.
		dev->rx.gio.dma_need	= GSC_DMA_FLAG_BLDMA_RX;
		dev->rx.gio.func		= &io_dma_read_funcs;
		status					= gsc_io_transfer_work(&dev->rx.gio);
	}

	if (count)
		count[0]	= dev->rx.gio.xfered_tot;

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_read
*
*	Purpose:
*
*		Perform a data read operation.
*
*	Arguments:
*
*		handle		The device to access.
*
*		buffer		The data read is put here.
*
*		bytes		The number of data bytes to read.
*
*		count		The actual number of bytes read is put here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_read(void* handle, void* buffer, U32 bytes, U32* count)
{
	dev_data_t*	dev;
	U32			limit;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		if (dev->strict_dram)
		{
			if ((dev->rx.offset == 0) && (dev->dram_size_mb >= 4096))
				limit	= 0xFFFFFFFC;	// The limit is 4GB - 4.
			else
				limit	= dev->dram_size_mb * 1024 * 1024 - dev->rx.offset;

			if (bytes > limit)
				bytes	= limit;
		}

		status	= gsc_io_start_transfer(&dev->sem,
										&dev->rx.gio,
										_io_read,
										buffer,
										bytes,
										count);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_io_write
*
*	Purpose:
*
*		Perform a data write operation. This is the common routine called for
*		both blocking and overlapped I/O.
*
*	Arguments:
*
*		dev			The device to access.
*
*		buffer		The data is written from here.
*
*		bytes		The number of data bytes to write.
*
*		count		The actual number of bytes written is put here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _io_write(dev_data_t* dev, void* buffer, U32 bytes, U32* count)
{
	U32	status;

	// The I/O semaphore is already locked.
	dev->tx.pio_active		= 0;
	dev->tx.gio.abort		= 0;
	dev->tx.gio.aborted		= 0;
	dev->tx.gio.buffer		= buffer;
	dev->tx.gio.xfer_buf	= buffer;
	dev->tx.gio.xfer_tot	= bytes;
	dev->tx.gio.xfered_tot	= 0;
	dev->tx.gio.fifo_like	= 0;
	dev->tx.gio.loc_adrs	= dev->tx.offset;

	if (dev->tx.gio.dma_enable == 0)
	{
		// PIO via BAR2 mapping into DRAM.
		dev->tx.pio_active	= 1;

		if (dev->rx.pio_active)
			status	= (U32) GSC_ACCESS_DENIED;	// Only one at a time.
		else
			status	= dev_access_dram(dev, 1, 1);

		if (status == (U32) GSC_SUCCESS)
		{
			// We must have DRAM access from the PCI bus.
			dev->tx.gio.func	= &io_pio_write_funcs;
			status				= gsc_io_transfer_work(&dev->tx.gio);
			dev_access_dram(dev, 0, 1);
		}

		dev->tx.pio_active	= 0;
	}
	else
	{
		// Blind DMA.
		dev->tx.gio.dma_need	= GSC_DMA_FLAG_BLDMA_TX;
		dev->tx.gio.func		= &io_dma_write_funcs;
		status					= gsc_io_transfer_work(&dev->tx.gio);
	}

	if (count)
		count[0]	= dev->tx.gio.xfered_tot;

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_write
*
*	Purpose:
*
*		Perform a data write operation.
*
*	Arguments:
*
*		handle		The device to access.
*
*		buffer		The source for the data to write.
*
*		bytes		The number of data bytes to write.
*
*		count		The actual number of bytes written is put here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_write(void* handle, const void* buffer, U32 bytes, U32* count)
{
	dev_data_t*	dev;
	U32			limit;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		if (dev->strict_dram)
		{
			if ((dev->tx.offset == 0) && (dev->dram_size_mb >= 4096))
				limit	= 0xFFFFFFFC;	// The limit is 4GB - 4.
			else
				limit	= dev->dram_size_mb * 1024 * 1024 - dev->tx.offset;

			if (bytes > limit)
				bytes	= limit;
		}

		status	= gsc_io_start_transfer(&dev->sem,
										&dev->tx.gio,
										_io_write,
										(void*) buffer,
										bytes,
										count);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	io_init
*
*	Purpose:
*
*		Initialize the given I/O structure to the devices initial open()
*		state. The semaphore is assumed to already be in use when we are
*		called, so we'll work without trying to gain exclusive access.
*
*	Arguments:
*
*		dev			The device being accessed.
*
*		dio			The device I/O structure to initialize.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 io_init(dev_data_t* dev, dev_io_t* dio)
{
#define	_DMA_DEFAULT_	((DMI32_IO_MODE_DEFAULT == DMI32_IO_MODE_PIO) ? 0 : 1)
	U32	stat;
	U32	status	= (U32) GSC_SUCCESS;

	dio->offset		= DMI32_IO_OFFSET_DEFAULT;
	dio->pio_active	= 0;

	stat	= gsc_io_init(&dio->gio);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_mem_alloc(&dio->mem, DMI32_IO_BUFFER_SIZE_DEFAULT, NULL);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	dio->gio.auto_start			= 0;
	dio->gio.data_lane			= 0;
	dio->gio.data_size			= 32;
	dio->gio.dma_control_mode	= 0;
	dio->gio.dma_demand			= 0;	// DMDMA not supported
	dio->gio.dma_dynamic		= 0;	// Static
	dio->gio.dma_enable			= _DMA_DEFAULT_;
	dio->gio.fifo_like			= 0;	// DRAM access is not via a FIFO.
	dio->gio.loc_adrs			= 0;
	dio->gio.pio_threshold		= 0;	// We always use DMA.
	dio->gio.reg				= 0;	// There is no FIFO type register.
	dio->gio.timeout_s			= DMI32_IO_TIMEOUT_DEFAULT;

	stat	= gsc_dma_channel_close(&dev->dma, &dio->gio.dma_channel);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_io_overlap_enable(&dio->gio.overlap,
									DMI32_IO_OVERLAP_ENABLE_DEFAULT,
									NULL);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	// Initialize the I/O notification stuff.
	stat	= gsc_io_callback_config(
				&dio->gio,
				0,
				NULL,
				(void*) GSC_NO_CHANGE,	// arg1
				0,
				0,
				NULL);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	return(status);
}



/******************************************************************************
*
*	Function:	io_dram_bank_select_force
*
*	Purpose:
*
*		Adjust the 2GB bank via the Local Memory Bank Register so that PIO
*		acess views the proper memory region. This is done by temporarity
*		adjusting the BAR mapping, then putting it back.
*
*		The suffix FORCE is added as this routine ignores the appropriate view
*		locking as those who call us have it locked in a way that prevents our
*		normal access. That is why we have to do it secretly.
*
*	Arguments:
*
*		gio			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 io_dram_bank_select_force(gsc_io_t* gio)
{
	U32	bank;
	U32	las0ba;
	U32	status;

	status	= PLX_REG_READ(gio->dev, GSC_PLX_9080_LAS0BA, &las0ba);

	if (status == (U32) GSC_SUCCESS)
	{
		bank	= gio->dio->offset >> 31;
		status	= PLX_REG_WRITE(gio->dev, GSC_PLX_9080_LAS0BA, BAR2_REMAP_FW_REGS);

		if (status == (U32) GSC_SUCCESS)
			status	= gsc_bar_reg_write(&gio->dev->gsc_regs, DMI32_LMBR, bank);

		if (status == (U32) GSC_SUCCESS)
			status	= PLX_REG_WRITE(gio->dev, GSC_PLX_9080_LAS0BA, las0ba);
		else
			PLX_REG_WRITE(gio->dev, GSC_PLX_9080_LAS0BA, las0ba);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_seek
*
*	Purpose:
*
*		Advance the I/O offset in a manner similar to fseek().
*
*	Arguments:
*
*		handle		The device to access.
*
*		which		The object to appy the change to, either Tx or Rx.
*
*		offset		The desired offset from zero.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_seek(void* handle, U32 which, U32 offset)
{
	dev_data_t*	dev;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		if (which & DMI32_WHICH_TX)
			status	= _io_offset_side(dev, &dev->tx, offset, NULL);

		if (which & DMI32_WHICH_RX)
			status	= _io_offset_side(dev, &dev->rx, offset, NULL);

		dev_access_unlock(dev);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_tell
*
*	Purpose:
*
*		Report the I/O offset in a manner similar to ftell().
*
*	Arguments:
*
*		handle		The device to access.
*
*		which		The object to appy the change to, either Tx or Rx.
*
*		offset		The offset from zero is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_tell(void* handle, U32 which, UL* offset)
{
	dev_data_t*	dev;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		if (which & DMI32_WHICH_TX)
			status	= _io_offset_side(dev, &dev->tx, GSC_NO_CHANGE, offset);

		if (which & DMI32_WHICH_RX)
			status	= _io_offset_side(dev, &dev->rx, GSC_NO_CHANGE, offset);

		dev_access_unlock(dev);
	}

	return(status);
}


