/* API - DMI32 - Any OS */
/* $Workfile: dev_io_dma.c $ */
/* $Revision: 5 $ */
/* $Date: 10/27/07 10:44a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/api/dev_io_dma.c $
 *
 * 5     10/27/07 10:44a Dnewbold
 * Additional driver porting updates.
 *
 * 4     4/02/07 5:28p Dnewbold
 * Added #include <string.h>
 *
 * 3     3/01/07 8:25a Dnewbold
 *
 * 2     5/09/06 1:37p Dnewbold
 *
 * 1     8/22/05 3:40p Dnewbold
*/

#include <string.h>

#include "dev_io.h"

#include "dev_access.h"
#include "dev_reg.h"



/* prototypes	*************************************************************/

static	U32 _dma_available(gsc_io_t* gio);
static	U32	_dma_desc_init(gsc_io_t* io, gsc_dma_info_t* info);
static	U32	_dma_read(gsc_io_t* gio);
static	U32	_dma_setup(gsc_io_t* gio);
static	U32	_dma_wait(gsc_io_t* gio, U32 timeout_ms);
static	U32	_dma_write(gsc_io_t* gio);



/* variables	*************************************************************/

// The application is responsible for making sure chunck sizes are OK.
const gsc_io_func_t	io_dma_read_funcs	=
{
	{
	/* _8.setup			*/	NULL,
	/* _8.available		*/	NULL,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	NULL,
	/* _8.transfer_reg	*/	NULL,
	/* _8.wait			*/	NULL,
	/* _8.done			*/	NULL
	},
	{
	/* _16.setup		*/	NULL,
	/* _16.available	*/	NULL,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	NULL,
	/* _16.transfer_reg	*/	NULL,
	/* _16.wait			*/	NULL,
	/* _16.done			*/	NULL
	},
	{
	/* _32.setup		*/	_dma_setup,
	/* _32.available	*/	_dma_available,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	_dma_read,
	/* _32.transfer_reg	*/	_dma_read,
	/* _32.wait			*/	_dma_wait,
	/* _32.done			*/	gsc_dma_done
	}
};

// The application is responsible for making sure chunck sizes are OK.
const gsc_io_func_t	io_dma_write_funcs	=
{
	{
	/* _8.setup			*/	NULL,
	/* _8.available		*/	NULL,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	NULL,
	/* _8.transfer_reg	*/	NULL,
	/* _8.wait			*/	NULL,
	/* _8.done			*/	NULL
	},
	{
	/* _16.setup		*/	NULL,
	/* _16.available	*/	NULL,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	NULL,
	/* _16.transfer_reg	*/	NULL,
	/* _16.wait			*/	NULL,
	/* _16.done			*/	NULL
	},
	{
	/* _32.setup		*/	_dma_setup,
	/* _32.available	*/	_dma_available,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	_dma_write,
	/* _32.transfer_reg	*/	_dma_write,
	/* _32.wait			*/	_dma_wait,
	/* _32.done			*/	gsc_dma_done
	}
};



/******************************************************************************
*
*	Function:	_dma_available
*
*	Purpose:
*
*		Determine the number of available bytes, assuming 32-bit samples.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dma_available(gsc_io_t* gio)
{
	#define	_2GB	(2UL * 1024 * 1024 * 1024)

	U32	base;
	U32	offset;

	// The size of the transfer is limited to the bank size of 2BG.
	base		= gio->dio->offset & ~(_2GB - 1);
	offset		= gio->dio->offset - base;
	gio->xfer	= _2GB - offset;
	return((U32) GSC_SUCCESS);
}



/******************************************************************************
*
*	Function:	_dma_desc_init
*
*	Purpose:
*
*		Initialize the referenced description structure in preparation for a
*		Non-Demand Mode DMA transfer.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*		info		The structure to initialize.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dma_desc_init(gsc_io_t* gio, gsc_dma_info_t* info)
{
	/*
	*	These settings are specific to the DMI32 and may be different for
	*	other boards.
	*/

	memset(info, 0, sizeof(info[0]));
	info->ready_input		= 1;
	info->bterm_input		= 0;
	info->local_bursting	= 0;	// HW Manual said yes, at the time.
	info->wr_inval_mode		= 1;
	info->dma_eot_pin		= 0;
	info->demand_mode		= 0;
	info->stop_xfer_mode	= 0;
	info->fifo_mode			= 0;	// Memory block.
	info->wait_states		= 0;
	info->bus_width			= 32;
	info->priority			= GSC_DMA_PRI_SHARE;
	return((U32) GSC_SUCCESS);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	_dma_setup
*
*	Purpose:
*
*		Perform any needed setup in preparation for a DMA I/O operation which
*		is about to begin.
*
*	Arguments:
*
*		gio			The common I/O data structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dma_setup(gsc_io_t* gio)
{
	gsc_mem_t	mem;
	U32			status;

	status	= gsc_mem_test(&gio->dev->tx.mem, gio->buffer, gio->xfer_tot, &mem);

	if ((status == (U32) GSC_SUCCESS) && (mem.size == 0))
		status	= gsc_mem_test(&gio->dev->rx.mem, gio->buffer, gio->xfer_tot, &mem);

	if (status != (U32) GSC_SUCCESS)
	{
	}
	else if (mem.size)
	{
		// Block mode DMA
		gio->buffer		= (void*) (UL) mem.phy_adrs;
		gio->xfer_buf	= (void*) (UL) mem.phy_adrs;
		status			= gsc_dma_channel_open(	&gio->dev->dma,
												&gio->dma_channel,
												GSC_DMA_TYPE_BLOCK,
												gio->dma_need,
												_dma_desc_init,
												gio);
	}
	else
	{
		// Scatter-Gather mode DMA/DMDMA
		status	= gsc_dma_channel_open(	&gio->dev->dma,
										&gio->dma_channel,
										GSC_DMA_TYPE_CHAINED,
										gio->dma_need,
										_dma_desc_init,
										gio);
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_dma_read
*
*	Purpose:
*
*		Read the specified number of bytes.
*
*	Arguments:
*
*		gio			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dma_read(gsc_io_t* gio)
{
	U32	status;

	// Make sure we're looking at the correct 2GB bank.
	status	= io_dram_bank_select_force(gio);

	// Make sure the DMA target address is in the low 2GB range.
	gio->loc_adrs_adj	&= 0x7FFFFFFF;

	if (status == (U32) GSC_SUCCESS)
		status	= gsc_io_dma_read(gio);

	return(status);
}



/******************************************************************************
*
*	Function:	_dma_write
*
*	Purpose:
*
*		Write the specified number of bytes, assuming 32-bit samples.
*
*	Arguments:
*
*		gio			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dma_write(gsc_io_t* gio)
{
	U32	status;

	// Make sure we're looking at the correct 2GB bank.
	status	= io_dram_bank_select_force(gio);

	// Make sure the DMA target address is in the low 2GB range.
	gio->loc_adrs_adj	&= 0x7FFFFFFF;

	if (status == (U32) GSC_SUCCESS)
		status	= gsc_io_dma_write(gio);

	return(status);
}



/******************************************************************************
*
*	Function:	_dma_wait
*
*	Purpose:
*
*		Wait for DMA completion, with a timeout.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*		timeout_ms	The amount of time left to wait.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _dma_wait(gsc_io_t* gio, U32 timeout_ms)
{
	U32	status;

	// Wait for the DMA transfer to complete.
	// Wait for the DMA Done interrupt.
	status	= gsc_io_dma_wait(gio, timeout_ms);

	if (status == (U32) GSC_SUCCESS)
		gio->dio->offset	+= gio->xfered;

	return(status);
}



/*****************************************************************************
*
*	Function:	io_dma_init
*
*	Purpose:
*
*		Initialize the DMA data to its initial open() state.
*
*	Arguments:
*
*		dev			The device to initialize.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 io_dma_init(dev_data_t* dev)
{
	U32	ch0;
	U32	ch1;
	U32	stat;
	U32	status	= (U32) GSC_SUCCESS;

	stat	= gsc_dma_channel_close(&dev->dma, &dev->rx.gio.dma_channel);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	stat	= gsc_dma_channel_close(&dev->dma, &dev->tx.gio.dma_channel);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	// This is the easy way.
	ch0		= GSC_DMA_FLAG_BLDMA_RX;
	ch1		= GSC_DMA_FLAG_BLDMA_TX;

	stat	= gsc_dma_init(&dev->dma, ch0, ch1);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;
	return(status);
}


