/* API - DMI32 - Any OS */
/* $Workfile: dev_io_pio.c $ */
/* $Revision: 4 $ */
/* $Date: 10/27/07 10:45a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/api/dev_io_pio.c $
 *
 * 4     10/27/07 10:45a Dnewbold
 * Additional driver porting updates.
 *
 * 3     3/01/07 8:25a Dnewbold
 *
 * 2     5/05/06 10:39a Dnewbold
 * Worked to increase modularity.
 *
 * 1     8/22/05 3:40p Dnewbold
*/

#include "dev_io.h"

#include "dev_access.h"
#include "dev_reg.h"



/* prototypes	*************************************************************/

static	U32	_pio_available(gsc_io_t* gio);
static	U32	_pio_read_32_map(gsc_io_t* gio);
static	U32	_pio_read_32_reg(gsc_io_t* gio);
static	U32	_pio_wait(gsc_io_t* gio, U32 timeout_ms);
static	U32	_pio_write_32_map(gsc_io_t* gio);
static	U32	_pio_write_32_reg(gsc_io_t* gio);


/* variables	*************************************************************/

const gsc_io_func_t	io_pio_read_funcs	=
{
	{
	/* _8.setup			*/	NULL,
	/* _8.available		*/	NULL,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	NULL,
	/* _8.transfer_reg	*/	NULL,
	/* _8.wait			*/	NULL,
	/* _8.done			*/	NULL
	},
	{
	/* _16.setup		*/	NULL,
	/* _16.available	*/	NULL,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	NULL,
	/* _16.transfer_reg	*/	NULL,
	/* _16.wait			*/	NULL,
	/* _16.done			*/	NULL
	},
	{
	/* _32.setup		*/	NULL,
	/* _32.available	*/	_pio_available,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	_pio_read_32_map,
	/* _32.transfer_reg	*/	_pio_read_32_reg,
	/* _32.wait			*/	_pio_wait,
	/* _32.done			*/	NULL
	}
};

const gsc_io_func_t	io_pio_write_funcs	=
{
	{
	/* _8.setup			*/	NULL,
	/* _8.available		*/	NULL,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	NULL,
	/* _8.transfer_reg	*/	NULL,
	/* _8.wait			*/	NULL,
	/* _8.done			*/	NULL
	},
	{
	/* _16.setup		*/	NULL,
	/* _16.available	*/	NULL,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	NULL,
	/* _16.transfer_reg	*/	NULL,
	/* _16.wait			*/	NULL,
	/* _16.done			*/	NULL
	},
	{
	/* _32.setup		*/	NULL,
	/* _32.available	*/	_pio_available,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	_pio_write_32_map,
	/* _32.transfer_reg	*/	_pio_write_32_reg,
	/* _32.wait			*/	_pio_wait,
	/* _32.done			*/	NULL
	}
};



/******************************************************************************
*
*	Function:	_pio_available
*
*	Purpose:
*
*		Determine the number of available bytes, assuming 32-bit samples.
*		Since we access DRAM via the BAR2 memory region, we adjust the BAR2
*		mapping as needed here. Also, because we access DRAM via this window,
*		we limit the transfer quantity to what fits within visible windows.
*		Therefore, those who do the data moving don't have to worry about
*		boundary conditions. It also means that transfer sizes are small
*		enough that they can ignore the timeout limits.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _pio_available(gsc_io_t* gio)
{
	U32	base;
	U32	offset;

	// The size of the transfer is limited to the visible window.
	base		= gio->dio->offset & ~(BAR2_WINDOW_SIZE - 1);
	offset		= gio->dio->offset - base;
	gio->xfer	= BAR2_WINDOW_SIZE - offset;
	return((U32) GSC_SUCCESS);
}



/******************************************************************************
*
*	Function:	_pio_read_32_map
*
*	Purpose:
*
*		Read the specified number of bytes, assuming 32-bit samples. Perform
*		the read using memory mapped access to the BAR2 memory region.
*
*	Arguments:
*
*		gio			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _pio_read_32_map(gsc_io_t* gio)
{
	U32			base;
	U32*		dest	= gio->xfer_buf;
	int			i;
	U32			las0ba;
	U32			offset;
	U32			samples;
	const U32*	src;
	U32			status;

	base	= gio->dio->offset & ~(BAR2_WINDOW_SIZE - 1);
	offset	= gio->dio->offset - base;
	src		= (U32*) (offset + (UL) gio->dev->gsc_regs.ptr);
	samples	= gio->xfer / 4;

	// Adjust the window to the DRAM area of interest.
	status	= io_dram_bank_select_force(gio);

	if (status == (U32) GSC_SUCCESS)
	{
		las0ba	= (base & BAR2_REMAP_DRAM_ADRS_MASK) | BAR2_REMAP_ENABLE;
		status	= PLX_REG_WRITE(gio->dev, GSC_PLX_9080_LAS0BA, las0ba);
	}

	if (status == (U32) GSC_SUCCESS)
	{
		// Perform the transfer.

		for (i = 0; i < (int) samples; i++)
			dest[i]	= src[i];

		gio->dio->offset	+= gio->xfer;
		gio->xfered			= gio->xfer;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_pio_read_32_reg
*
*	Purpose:
*
*		Read the specified number of bytes, assuming 32-bit samples. Perform
*		the read using register access services to the BAR2 memory region.
*
*	Arguments:
*
*		gio			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _pio_read_32_reg(gsc_io_t* gio)
{
	const gsc_bar_t*	bar		= &gio->dev->gsc_regs;
	U32					base;
	U32*				dest	= gio->xfer_buf;
	int					i;
	U32					las0ba;
	U32					offset;
	U32					reg;
	U32					samples;
	U32					status;

	base	= gio->dio->offset & ~(BAR2_WINDOW_SIZE - 1);
	offset	= gio->dio->offset - base;
	samples	= gio->xfer / 4;

	// Adjust the window to the DRAM area of interest.
	status	= io_dram_bank_select_force(gio);

	if (status == (U32) GSC_SUCCESS)
	{
		las0ba	= (base & BAR2_REMAP_DRAM_ADRS_MASK) | BAR2_REMAP_ENABLE;
		status	= PLX_REG_WRITE(gio->dev, GSC_PLX_9080_LAS0BA, las0ba);
	}

	// Perform the transfer.

	for (i = 0; (status == (U32) GSC_SUCCESS) && (i < (int) samples); i++)
	{
		reg		= DMI32_REG_ENCODE(4, offset + (4 * i));
		status	= gsc_bar_reg_read(bar, reg, &dest[i]);

		if (status == (U32) GSC_SUCCESS)
		{
			gio->dio->offset	+= 4;
			gio->xfered			+= 4;
		}
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_pio_wait
*
*	Purpose:
*
*		This is a dummy wait service since we will never have to wait for data
*		or space to become available.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*		timeout_ms	The amount of time left to wait.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _pio_wait(gsc_io_t* gio, U32 timeout_ms)
{
	return((U32) GSC_SUCCESS);
}	/*lint !e715 */



/******************************************************************************
*
*	Function:	_pio_write_32_map
*
*	Purpose:
*
*		Write the specified number of bytes, assuming 32-bit samples. Perform
*		the write using memory mapped access to the BAR2 memory region.
*
*	Arguments:
*
*		gio			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _pio_write_32_map(gsc_io_t* gio)
{
	U32		base;
	U32*	dest;
	int		i;
	U32		las0ba;
	U32		offset;
	U32		samples;
	U32*	src	= gio->xfer_buf;
	U32		status;

	base	= gio->dio->offset & ~(BAR2_WINDOW_SIZE - 1);
	offset	= gio->dio->offset - base;
	dest	= (U32*) (offset + (UL) gio->dev->gsc_regs.ptr);
	samples	= gio->xfer / 4;

	// Adjust the window to the DRAM area of interest.
	status	= io_dram_bank_select_force(gio);

	if (status == (U32) GSC_SUCCESS)
	{
		las0ba	= (base & BAR2_REMAP_DRAM_ADRS_MASK) | BAR2_REMAP_ENABLE;
		status	= PLX_REG_WRITE(gio->dev, GSC_PLX_9080_LAS0BA, las0ba);
	}

	if (status == (U32) GSC_SUCCESS)
	{
		// Perform the transfer.

		for (i = 0; i < (int) samples; i++)
			dest[i]	= src[i];

		gio->dio->offset	+= gio->xfer;
		gio->xfered			= gio->xfer;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_pio_write_32_reg
*
*	Purpose:
*
*		Write the specified number of bytes, assuming 32-bit samples. Perform
*		the write using register access services to the BAR2 memory region.
*
*	Arguments:
*
*		gio			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _pio_write_32_reg(gsc_io_t* gio)
{
	const gsc_bar_t*	bar		= &gio->dev->gsc_regs;
	U32					base;
	int					i;
	U32					las0ba;
	U32					offset;
	U32					reg;
	U32					samples;
	U32*				src	= gio->xfer_buf;
	U32					status;

	base	= gio->dio->offset & ~(BAR2_WINDOW_SIZE - 1);
	offset	= gio->dio->offset - base;
	samples	= gio->xfer / 4;

	// Adjust the window to the DRAM area of interest.
	status	= io_dram_bank_select_force(gio);

	if (status == (U32) GSC_SUCCESS)
	{
		las0ba	= (base & BAR2_REMAP_DRAM_ADRS_MASK) | BAR2_REMAP_ENABLE;
		status	= PLX_REG_WRITE(gio->dev, GSC_PLX_9080_LAS0BA, las0ba);
	}

	// Perform the transfer.

	for (i = 0; (status == (U32) GSC_SUCCESS) && (i < (int) samples); i++)
	{
		reg		= DMI32_REG_ENCODE(4, offset + (4 * i));
		status	= gsc_bar_reg_write(bar, reg, src[i]);

		if (status == (U32) GSC_SUCCESS)
		{
			gio->dio->offset	+= 4;
			gio->xfered			+= 4;
		}
	}

	return(status);
}


