/* API - DMI32 - Any OS */
/* $Workfile: dev_irq.c $ */
/* $Revision: 4 $ */
/* $Date: 10/27/07 10:46a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/api/dev_irq.c $
 *
 * 4     10/27/07 10:46a Dnewbold
 * Linux porting updates.
 *
 * 3     3/01/07 8:25a Dnewbold
 *
 * 2     5/05/06 10:39a Dnewbold
 * Worked to increase modularity.
 *
 * 1     8/22/05 3:40p Dnewbold
*/

#include "dev_irq.h"

#include "gsc_version.h"

#include "dev_config.h"
#include "dev_reg.h"



/*****************************************************************************
*
*	Function:	_interrupt_disable
*
*	Purpose:
*
*		Disable the referenced GSC interrupts.
*
*	Arguments:
*
*		dev			The device to access.
*
*		mask		The mask of interrupts to disable. This is according to
*					Interrupt Control Register layout.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _interrupt_disable(dev_data_t* dev, U32 mask)
{
	U32	status;

	status	= GSC_REG_MOD(dev, DMI32_ICR, 0, mask);
	return(status);
}



/*****************************************************************************
*
*	Function:	_interrupt_enable
*
*	Purpose:
*
*		Enable the referenced GSC interrupts.
*
*	Arguments:
*
*		dev			The device to access.
*
*		mask		The mask of interrupts to enable. This is according to
*					Interrupt Control Register layout.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _interrupt_enable(dev_data_t* dev, U32 mask)
{
	U32	status;

	status	= GSC_REG_MOD(dev, DMI32_ICR, mask, mask);
	return(status);
}



/*****************************************************************************
*
*	Function:	_irq_callback_arg_irq
*
*	Purpose:
*
*		Implement the Interrupt Notification Callback Argument parameter
*		service for a single GSC interrupt. Validate and apply the setting(s),
*		as appropriate, then retrieve the current setting(s) for the caller,
*		also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		local		The structure for the GSC interrupt to access.
*
*		mask		The mask for the GSC interrupt to access. This is according
*					to Interrupt Control Register layout.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _irq_callback_arg_irq(	dev_data_t*		dev,
									gsc_event_t*	local,
									U32				mask,
									unsigned long	set,
									unsigned long*	get)
{
	U32			status;
	gsc_event_t	tmp;

	for (;;)	/* A convenience loop. */
	{
		status	= gsc_event_config(
					local,
					(void*) GSC_NO_CHANGE,	// id
					(void*) GSC_NO_CHANGE,	// function
					(void*) GSC_NO_CHANGE,	// arg1
					GSC_NO_CHANGE,
					GSC_NO_CHANGE,
					&tmp);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_IRQ_CALLBACK_ARG_DEFAULT;
		}

		if ((set != GSC_NO_CHANGE) && (set != tmp.arg3))
		{
			status	= gsc_event_config(
						local,
						(void*) GSC_NO_CHANGE,	// id
						(void*) GSC_NO_CHANGE,	// function
						(void*) GSC_NO_CHANGE,	// arg1
						GSC_NO_CHANGE,
						set,
						&tmp);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= gsc_event_config(
						local,
						(void*) GSC_NO_CHANGE,	// id
						(void*) GSC_NO_CHANGE,	// function
						(void*) GSC_NO_CHANGE,	// arg1
						GSC_NO_CHANGE,
						GSC_NO_CHANGE,
						&tmp);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= tmp.arg3;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	irq_callback_arg
*
*	Purpose:
*
*		Implement the Interrupt Notification Callback Argument parameter
*		service. Validate and apply the setting(s), as appropriate, then
*		retrieve the current setting(s) for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 irq_callback_arg(dev_data_t* dev, U32 which, unsigned long set, unsigned long* get)
{
	int	i;
	U32	mask;
	U32	stat;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, DMI32_WHICH_IRQ_ALL);
		if (status != (U32) GSC_SUCCESS)	break;
		which	&= DMI32_WHICH_IRQ_ALL;

		for (	i = 0, mask = 0x1;
				i < (int) ARRAY_ELEMENTS(dev->irq.locals);
				i++, mask <<= 1)
		{
			if ((mask & which) == 0)
				continue;

			stat	= _irq_callback_arg_irq(dev,
											dev->irq.locals[i],
											mask,
											set,
											get);
			status	= (status == (U32) GSC_SUCCESS) ? stat : status;
			if (stat != (U32) GSC_SUCCESS)	continue;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_irq_callback_func_irq
*
*	Purpose:
*
*		Implement the Interrupt Notification Callback Function parameter
*		service for a single GSC interrupt. Validate and apply the setting(s),
*		as appropriate, then retrieve the current setting(s) for the caller,
*		also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		local		The structure for the GSC interrupt to access.
*
*		mask		The mask for the GSC interrupt to access. This is according
*					to Interrupt Control Register layout.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _irq_callback_func_irq(	dev_data_t*		dev,
									gsc_event_t*	local,
									U32				mask,
									unsigned long	set,
									unsigned long*	get)
{
	U32			status;
	gsc_event_t	tmp;

	for (;;)	/* A convenience loop. */
	{
		status	= gsc_event_config(
					local,
					(void*) GSC_NO_CHANGE,	// id
					(void*) GSC_NO_CHANGE,	// function
					(void*) GSC_NO_CHANGE,	// arg1
					GSC_NO_CHANGE,
					GSC_NO_CHANGE,
					&tmp);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_IRQ_CALLBACK_ARG_DEFAULT;
		}

		if ((set != GSC_NO_CHANGE) && (set != (UL) tmp.callback))
		{
			status	= gsc_event_config(
						local,
						(void*) GSC_NO_CHANGE,	// id
						(void*) set,			// function
						(void*) GSC_NO_CHANGE,	// arg1
						GSC_NO_CHANGE,
						GSC_NO_CHANGE,
						&tmp);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= gsc_event_config(
						local,
						(void*) GSC_NO_CHANGE,	// id
						(void*) GSC_NO_CHANGE,	// function
						(void*) GSC_NO_CHANGE,	// arg1
						GSC_NO_CHANGE,
						GSC_NO_CHANGE,
						&tmp);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= (UL) tmp.callback;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	irq_callback_func
*
*	Purpose:
*
*		Implement the Interrupt Notification Callback Function parameter
*		service. Validate and apply the setting(s), as appropriate, then
*		retrieve the current setting(s) for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 irq_callback_func(dev_data_t* dev, U32 which, unsigned long set, unsigned long* get)
{
	int	i;
	U32	mask;
	U32	stat;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, DMI32_WHICH_IRQ_ALL);
		if (status != (U32) GSC_SUCCESS)	break;
		which	&= DMI32_WHICH_IRQ_ALL;

		for (	i = 0, mask = 0x1;
				i < (int) ARRAY_ELEMENTS(dev->irq.locals);
				i++, mask <<= 1)
		{
			if ((mask & which) == 0)
				continue;

			stat	= _irq_callback_func_irq(	dev,
												dev->irq.locals[i],
												mask,
												set,
												get);
			status	= (status == (U32) GSC_SUCCESS) ? stat : status;
			if (stat != (U32) GSC_SUCCESS)	continue;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_irq_enable_irq
*
*	Purpose:
*
*		Implement the Interrupt Notification Interrupt Enable parameter
*		service for a single GSC interrupt. Validate and apply the setting(s),
*		as appropriate, then retrieve the current setting(s) for the caller,
*		also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		mask		The mask for the GSC interrupt to access. This is according
*					to Interrupt Control Register layout.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _irq_enable_irq(	dev_data_t*			dev,
							U32					mask,
							UL					set,
							UL*					get)
{
	U32	icr;
	U32	status	= (U32) GSC_SUCCESS;

	for (;;)	/* A convenience loop. */
	{
		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_IRQ_ENABLE_DEFAULT;
		}
		else if (	(set == DMI32_IRQ_ENABLE_NO) ||
					(set == DMI32_IRQ_ENABLE_YES))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_IRQ_ENABLE_YES;
		}
		else
		{
			set	= DMI32_IRQ_ENABLE_NO;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set)
				status	= _interrupt_enable(dev, mask);
			else
				status	= _interrupt_disable(dev, mask);

			if (status != (U32) GSC_SUCCESS)	break;;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_ICR, &icr);
			if (status != (U32) GSC_SUCCESS)	break;;

			if (icr & mask)
				get[0]	= DMI32_IRQ_ENABLE_YES;
			else
				get[0]	= DMI32_IRQ_ENABLE_NO;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	irq_enable
*
*	Purpose:
*
*		Implement the Interrupt Notification Interrupt Enable parameter
*		service. Validate and apply the setting(s), as appropriate, then
*		retrieve the current setting(s) for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 irq_enable(dev_data_t* dev, U32 which, UL set, UL* get)
{
	int	i;
	U32	mask;
	U32	stat;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, DMI32_WHICH_IRQ_ALL);
		if (status != (U32) GSC_SUCCESS)	break;
		which	&= DMI32_WHICH_IRQ_ALL;

		for (	i = 0, mask = 0x1;
				i < (int) ARRAY_ELEMENTS(dev->irq.locals);
				i++, mask <<= 1)
		{
			if ((mask & which) == 0)
				continue;

			stat	= _irq_enable_irq(dev, mask, set, get);
			status	= (status == (U32) GSC_SUCCESS) ? stat : status;
			if (stat != (U32) GSC_SUCCESS)	continue;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	irq_state
*
*	Purpose:
*
*		Implement the Interrupt Notification Interrupt State parameter
*		service. Validate and apply the setting(s), as appropriate, then
*		retrieve the current setting(s) for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 irq_state(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	icr;
	U32	isr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, DMI32_WHICH_IRQ_ALL);
		if (status != (U32) GSC_SUCCESS)	break;
		which	&= DMI32_WHICH_IRQ_ALL;

		// SET is not supported for this parameter.

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_ICR, &icr);
			if (status != (U32) GSC_SUCCESS)	break;

			status	= GSC_REG_READ(dev, DMI32_ISR, &isr);
			if (status != (U32) GSC_SUCCESS)	break;

			if (isr & DMI32_WHICH_IRQ_ALL & which)
				get[0]	= DMI32_IRQ_STATE_ACTIVE;
			else
				get[0]	= DMI32_IRQ_STATE_INACTIVE;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	dmi32_irq_wait
*
*	Purpose:
*
*		Wait for any one of the referenced interrupts to occur. The
*		referenced interrupts are automatically enabled.
*
*	Arguments:
*
*		handle		The device to access.
*
*		which		The set of interrupts of interest.
*
*		timeout_ms	Return if an interrupt doesn't occur within this
*					number of milliseconds.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_irq_wait(void* handle, U32 which, U32 timeout_ms)
{
	dev_data_t*	dev;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		status	= config_which_validate(dev, which, DMI32_WHICH_IRQ_ALL);
		which	&= DMI32_WHICH_IRQ_ALL;

		if ((status == (U32) GSC_SUCCESS) && (which))
			status	= _interrupt_enable(dev, which);

		dev_access_unlock(dev);

		if ((status == (U32) GSC_SUCCESS) && (which))
			status	= gsc_irq_local_wait(&dev->irq.girq, which, timeout_ms);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_local_helper
*
*	Purpose:
*
*		This is the helper function called when there is a local interrupt.
*		Once here, the purpose is to check the source of the interrupt and
*		tell the common code which local interrupt occured.
*
*	Arguments:
*
*		arg1		We've set thus up to be a dev_data_t* for the device.
*
*		arg2		Unused.
*
*		arg3		Unused.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _local_helper(void* arg1, unsigned long arg2, unsigned long arg3)
{
	dev_data_t*	dev	=	(dev_data_t*) arg1;
	int			i;
	U32			icr;
	U32			isr	= 0;
	U32			mask;
	U32			status;

	status	= GSC_REG_READ(dev, DMI32_ICR, &icr);

	if (status == (U32) GSC_SUCCESS)
		status	= GSC_REG_READ(dev, DMI32_ISR, &isr);

	if (status == (U32) GSC_SUCCESS)
	{
		for (	i = 0, mask = 0x1;
				i < (int) ARRAY_ELEMENTS(dev->irq.locals);
				i++, mask <<= 1)
		{
			if ((mask & icr & isr) && (dev->irq.locals[i]))
			{
				GSC_REG_WRITE(dev, DMI32_ISR, mask);	// Clear the interrupt.
				GSC_REG_MOD(dev, DMI32_ICR, 0, mask);	// Disable the interrupt.
				gsc_event_trigger(dev->irq.locals[i]);
			}
		}
	}
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	irq_init
*
*	Purpose:
*
*		setup the IRQ data to the device's initial open() state.
*
*	Arguments:
*
*		dev			The device to access.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 irq_init(dev_data_t* dev)
{
	int	i;
	U32	stat;
	U32	status	= (U32) GSC_SUCCESS;

	stat	= gsc_event_config(
				dev->irq.local,
				(void*) GSC_PLX_INT_LOCAL,	// id
				_local_helper,
				dev,
				0,
				0,
				NULL);
	status	= (status == (U32) GSC_SUCCESS) ? stat : status;

	for (i = 0; i < (int) ARRAY_ELEMENTS(dev->irq.locals); i++)
	{
		if (dev->irq.locals[i] == NULL)
			continue;

		stat	= gsc_event_config(
					dev->irq.locals[i],
					(void*) GSC_NO_CHANGE,	// id
					NULL,
					dev,
					0x1UL << i,
					0,
					NULL);
		status	= (status == (U32) GSC_SUCCESS) ? stat : status;
	}

	return(status);
}


