/* API - DMI32 - Any OS */
/* $Workfile: dev_misc.c $ */
/* $Revision: 6 $ */
/* $Date: 10/27/07 10:47a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/api/dev_misc.c $
 *
 * 6     10/27/07 10:47a Dnewbold
 * Linux porting updates.
 *
 * 5     4/02/07 5:28p Dnewbold
 * Added #include <string.h>
 *
 * 4     3/01/07 8:30a Dnewbold
 * Linted.
 *
 * 3     5/05/06 10:39a Dnewbold
 * Worked to increase modularity.
 *
 * 2     5/04/06 3:58p Dnewbold
 * Removed key validation code and increased modularity.
 *
 * 1     8/22/05 3:40p Dnewbold
*/

#include <string.h>

#include "dev_misc.h"

#include "gsc_reg.h"

#include "dev_config.h"
#include "dev_reg.h"



/*****************************************************************************
*
*	Function:	misc_board_jumpers
*
*	Purpose:
*
*		Implement the miscelaneous Board Jumpers parameter service. Validate
*		and apply the setting, as appropriate, then retrieve the current
*		setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 misc_board_jumpers(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bsr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET is not supported by this parameter.

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_BSR, &bsr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= (bsr >> 4) & 0x3;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	misc_dram_capacity_mb
*
*	Purpose:
*
*		Implement the miscelaneous DRAM Capacity MB parameter service.
*		Validate and apply the setting, as appropriate, then retrieve the
*		current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 misc_dram_capacity_mb(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET is not supported by this parameter.

		if (get)
			get[0]	= dev->dram_capacity_mb;

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	misc_dram_size_mb
*
*	Purpose:
*
*		Implement the miscelaneous DRAM Size MB parameter service. Validate
*		and apply the setting, as appropriate, then retrieve the current
*		setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 misc_dram_size_mb(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET is not supported by this parameter.

		if (get)
			get[0]	= dev->dram_size_mb;

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	misc_features
*
*	Purpose:
*
*		Implement the miscallaneous Features parameter service. Validate and
*		apply the setting, as appropriate, then retrieve the current
*		setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			This gives the feature to check for.
*
*		get			Support for the given feature is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 misc_features(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status	= (U32) GSC_SUCCESS;
	U32	value	= DMI32_MISC_FEATURES_ABSENT;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET - set argument specifies the feature to GET

		// GET

		switch (set)
		{
			default:	if (dev->strict_args)
							status	= (U32) GSC_INVALID_DATA;

						break;


			case GSC_DEFAULT:
			case GSC_NO_CHANGE:

						break;

			case DMI32_MISC_FEATURES_FR:

						// Features Register
						value	= dev->features.features_register;
						break;

			case DMI32_MISC_FEATURES_IRQ_MBUR:

						// ICR bit for Multi-Buffer Under Run
						value	= dev->features.irq_mbur;
						break;

			case DMI32_MISC_FEATURES_LMBR:

						// Local Memory Bank Register
						value	= dev->features.lmb_register;
						break;

			case DMI32_MISC_FEATURES_SW_DIR:

						// The BCR Start Word Direction bit.
						value	= dev->features.start_word_dir;
						break;
		}

		if (status != (U32) GSC_SUCCESS)
			break;

		if (get)
		{
			get[0]	= value
					? DMI32_MISC_FEATURES_PRESENT
					: DMI32_MISC_FEATURES_ABSENT;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	misc_map_gsc_regs
*
*	Purpose:
*
*		Implement the miscallaneous service for mapping the GSC register
*		block to application space. Validate and apply the setting, as
*		appropriate, then retrieve the current setting for the caller, also
*		as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 misc_map_gsc_regs(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	rx_lock;
	U32	status;
	U32	tx_lock;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_MISC_MAP_GSC_REGS_DEFAULT;
		}
		else if (	(set == DMI32_MISC_MAP_GSC_REGS_ENABLE) ||
					(set == DMI32_MISC_MAP_GSC_REGS_DISABLE))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_MISC_MAP_GSC_REGS_ENABLE;
		}
		else
		{
			set	= DMI32_MISC_MAP_GSC_REGS_DISABLE;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == DMI32_MISC_MAP_GSC_REGS_ENABLE)
			{
				status	= gsc_bar_map(&dev->gsc_regs);
			}
			else
			{
				rx_lock	= gsc_sem_lock(&dev->rx.gio.sem);
				tx_lock	= gsc_sem_lock(&dev->tx.gio.sem);

				if ((rx_lock == (U32) GSC_SUCCESS) && (tx_lock == (U32) GSC_SUCCESS))
					status	= gsc_bar_unmap(&dev->gsc_regs);

				if (rx_lock == (U32) GSC_SUCCESS)
					gsc_sem_unlock(&dev->rx.gio.sem);

				if (tx_lock == (U32) GSC_SUCCESS)
					gsc_sem_unlock(&dev->tx.gio.sem);

			}

			if (status != (U32) GSC_SUCCESS)
				break;
		}

		if (get)
		{
			if (dev->gsc_regs.ptr)
				get[0]	= DMI32_MISC_MAP_GSC_REGS_ENABLE;
			else
				get[0]	= DMI32_MISC_MAP_GSC_REGS_DISABLE;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	misc_map_plx_regs
*
*	Purpose:
*
*		Implement the miscallaneous service for mapping the PLX feature set
*		register block to application space. Validate and apply the setting,
*		as appropriate, then retrieve the current setting for the caller,
*		also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 misc_map_plx_regs(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_MISC_MAP_PLX_REGS_DEFAULT;
		}
		else if (	(set == DMI32_MISC_MAP_PLX_REGS_ENABLE) ||
					(set == DMI32_MISC_MAP_PLX_REGS_DISABLE))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_MISC_MAP_PLX_REGS_ENABLE;
		}
		else
		{
			set	= DMI32_MISC_MAP_PLX_REGS_DISABLE;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == DMI32_MISC_MAP_PLX_REGS_ENABLE)
				status	= gsc_bar_map(&dev->plx_regs);
			else
				status	= gsc_bar_unmap(&dev->plx_regs);

			if (status != (U32) GSC_SUCCESS)
				break;
		}

		if (get)
		{
			if (dev->plx_regs.ptr)
				get[0]	= DMI32_MISC_MAP_PLX_REGS_ENABLE;
			else
				get[0]	= DMI32_MISC_MAP_PLX_REGS_DISABLE;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	misc_strict_arguments
*
*	Purpose:
*
*		Implement the miscallaneous Strict Argument parameter service.
*		Validate and apply the setting, as appropriate, then retrieve the
*		current setting(s) for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 misc_strict_arguments(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_MISC_STRICT_ARGUMENTS_DEFAULT;
		}
		else if (	(set == DMI32_MISC_STRICT_ARGUMENTS_ENABLE) ||
					(set == DMI32_MISC_STRICT_ARGUMENTS_DISABLE))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_MISC_STRICT_ARGUMENTS_ENABLE;
		}
		else
		{
			set	= DMI32_MISC_STRICT_ARGUMENTS_DISABLE;
		}

		if (set != GSC_NO_CHANGE)
			dev->strict_args	= set;

		if (get)
			get[0]	= dev->strict_args;

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	misc_strict_config
*
*	Purpose:
*
*		Implement the miscallaneous Strict Configuration parameter service.
*		Validate and apply the setting, as appropriate, then retrieve the
*		current setting(s) for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 misc_strict_config(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_MISC_STRICT_CONFIG_DEFAULT;
		}
		else if (	(set == DMI32_MISC_STRICT_CONFIG_ENABLE) ||
					(set == DMI32_MISC_STRICT_CONFIG_DISABLE))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_MISC_STRICT_CONFIG_ENABLE;
		}
		else
		{
			set	= DMI32_MISC_STRICT_CONFIG_DISABLE;
		}

		if (set != GSC_NO_CHANGE)
			dev->strict_config	= set;

		if (get)
			get[0]	= dev->strict_config;

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	misc_strict_dram_limits
*
*	Purpose:
*
*		Implement the Miscallaneous Strict DRAM Limits parameter service.
*		Validate and apply the setting, as appropriate, then retrieve the
*		current setting(s) for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 misc_strict_dram_limits(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_MISC_STRICT_DRAM_LIMITS_DEFAULT;
		}
		else if (	(set == DMI32_MISC_STRICT_DRAM_LIMITS_ENABLE) ||
					(set == DMI32_MISC_STRICT_DRAM_LIMITS_DISABLE))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_MISC_STRICT_DRAM_LIMITS_ENABLE;
		}
		else
		{
			set	= DMI32_MISC_STRICT_DRAM_LIMITS_DISABLE;
		}

		if (set != GSC_NO_CHANGE)
			dev->strict_dram	= set;

		if (get)
			get[0]	= dev->strict_dram;

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_dram_compute_bank_size
*
*	Purpose:
*
*		Compute the size of the DRAM in Megabyte units for the current 2GB
*		bank.
*
*	Arguments:
*
*		dev			The device to access.
*
*		size_mb		The size in the current bank.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _dram_compute_bank_size(dev_data_t* dev, U32* size_mb)
{
	#define	_64M	(64L * 1024 * 1024)

	U32	access;
	U32	base;
	U32	end;
	U32	get;
	int	i;
	U32	las0ba;
	U32	offset;
	U32	old[32];
	U32	reg;
	U32	set;
	U32	size	= 0;
	U32	status	= (U32) GSC_SUCCESS;

	memset(old, 0, sizeof(old));
	status		= dev_access_dram(dev, 1, 1);
	access		= status;
	size_mb[0]	= 0;

	// Place values at the end of power of two boundaries.

	for (i = 0, end = 3;; end = ((1UL + end) * 2) - 1, i++)
	{
		if (status != (U32) GSC_SUCCESS)	break;
		offset	= end - 3;
		set		= offset;
		base	= offset & ~(BAR2_WINDOW_SIZE - 1);

		// Adjust the window to the DRAM area of interest.
		las0ba	= (base & BAR2_REMAP_DRAM_ADRS_MASK) | BAR2_REMAP_ENABLE;
		status	= PLX_REG_WRITE(dev, GSC_PLX_9080_LAS0BA, las0ba);
		if (status != (U32) GSC_SUCCESS)	break;

		offset	-= base;
		reg		= DMI32_REG_ENCODE(4, offset);

		// Record what is there now.
		status	= gsc_bar_reg_read(&dev->gsc_regs, reg, &old[i]);
		if (status != (U32) GSC_SUCCESS)	break;

		// Apply a new value we can look for later.
		status	= gsc_bar_reg_write(&dev->gsc_regs, reg, set);
		if (status != (U32) GSC_SUCCESS)	break;
		if (end == 0x7FFFFFFF)		break;
	}

	// Look at the end of power of two boundaries for the prerecorded values.

	for (i = 0, end = 3;; end = ((1UL + end) * 2) - 1, i++)
	{
		if (status != (U32) GSC_SUCCESS)	break;
		offset	= end - 3;
		set		= offset;
		base	= offset & ~(BAR2_WINDOW_SIZE - 1);

		// Adjust the window to the DRAM area of interest.
		las0ba	= (base & BAR2_REMAP_DRAM_ADRS_MASK) | BAR2_REMAP_ENABLE;
		status	= PLX_REG_WRITE(dev, GSC_PLX_9080_LAS0BA, las0ba);
		if (status != (U32) GSC_SUCCESS)	break;

		offset	-= base;
		reg		= DMI32_REG_ENCODE(4, offset);

		// See what is recorded there now.
		status	= gsc_bar_reg_read(&dev->gsc_regs, reg, &get);
		if (status != (U32) GSC_SUCCESS)	break;

		if (get == set)
			size	= get + 4;

		// Put the original value back.
		status	= gsc_bar_reg_write(&dev->gsc_regs, reg, old[i]);
		if (status != (U32) GSC_SUCCESS)	break;
		if (end == 0x7FFFFFFF)		break;
	}

	if (size > _64M)
	{
		// 64M is the minimum increment we will check.

		// Place values at the end of 64MB boundaries.

		for (i = 1;; i++)
		{
			if (status != (U32) GSC_SUCCESS)	break;
			end		= (_64M * (U32) i - 1);
			offset	= end - 3;
			set		= offset;
			base	= offset & ~(BAR2_WINDOW_SIZE - 1);
			if (end >= 0x7FFFFFFF)		break;

			// Adjust the window to the DRAM area of interest.
			las0ba	= (base & BAR2_REMAP_DRAM_ADRS_MASK) | BAR2_REMAP_ENABLE;
			status	= PLX_REG_WRITE(dev, GSC_PLX_9080_LAS0BA, las0ba);
			if (status != (U32) GSC_SUCCESS)	break;

			offset	-= base;
			reg		= DMI32_REG_ENCODE(4, offset);

			// Record what is there now.
			status	= gsc_bar_reg_read(&dev->gsc_regs, reg, &old[i]);
			if (status != (U32) GSC_SUCCESS)	break;

			// Apply a new value we can look for later.
			status	= gsc_bar_reg_write(&dev->gsc_regs, reg, set);
			if (status != (U32) GSC_SUCCESS)	break;
			if (end == 0x7FFFFFFF)		break;
		}

		// Look at the end of 64MB boundaries for the prerecorded values.

		for (i = 1;; i++)
		{
			if (status != (U32) GSC_SUCCESS)	break;
			end		= (_64M * (U32) i - 1);
			offset	= end - 3;
			set		= offset;
			base	= offset & ~(BAR2_WINDOW_SIZE - 1);
			if (end >= 0x7FFFFFFF)		break;

			// Adjust the window to the DRAM area of interest.
			las0ba	= (base & BAR2_REMAP_DRAM_ADRS_MASK) | BAR2_REMAP_ENABLE;
			status	= PLX_REG_WRITE(dev, GSC_PLX_9080_LAS0BA, las0ba);
			if (status != (U32) GSC_SUCCESS)	break;

			offset	-= base;
			reg		= DMI32_REG_ENCODE(4, offset);

			// See what is recorded there now.
			status	= gsc_bar_reg_read(&dev->gsc_regs, reg, &get);
			if (status != (U32) GSC_SUCCESS)	break;

			if (get == set)
				size	= get + 4;

			// Put the original value back.
			status	= gsc_bar_reg_write(&dev->gsc_regs, reg, old[i]);
			if (status != (U32) GSC_SUCCESS)	break;
			if (end == 0x7FFFFFFF)		break;

		}
	}

	if (access == (U32) GSC_SUCCESS)
		dev_access_dram(dev, 0, 1);

	if (status == (U32) GSC_SUCCESS)
			size_mb[0]	= size / 1024 / 1024;

	return(status);
}



/*****************************************************************************
*
*	Function:	misc_dram_compute_size
*
*	Purpose:
*
*		Compute the size of the DRAM in Megabyte units.
*
*	Arguments:
*
*		dev			The device to access.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 misc_dram_compute_size(dev_data_t* dev)
{
	U32	bank;
	U32	bank_mb;
	U32	bank_qty	= dev->dram_capacity_mb / 2048;
	U32	status		= (U32) GSC_SUCCESS;
	U32	total_mb	= 0;

	for (bank = 0; bank < bank_qty; bank++)
	{
		if (status != (U32) GSC_SUCCESS)	break;

		// Select the bank to access.
		GSC_REG_WRITE(dev, DMI32_LMBR, bank);
		status	= _dram_compute_bank_size(dev, &bank_mb);

		if (bank_mb)
			total_mb	+= bank_mb;
	}

	GSC_REG_WRITE(dev, DMI32_LMBR, 0);

	if (status == (U32) GSC_SUCCESS)
		dev->dram_size_mb	= total_mb;
	else
		dev->dram_size_mb	= 0;

	return(status);
}


