/* API - DMI32 - Any OS */
/* $Workfile: dev_reg.c $ */
/* $Revision: 3 $ */
/* $Date: 3/01/07 8:30a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/api/dev_reg.c $
 *
 * 3     3/01/07 8:30a Dnewbold
 * Linted.
 *
 * 2     5/05/06 10:39a Dnewbold
 * Worked to increase modularity.
 *
 * 1     8/22/05 3:40p Dnewbold
*/

#include "dev_reg.h"

#include "gsc_reg.h"



/*****************************************************************************
*
*	Function:	dmi32_reg_mod
*
*	Purpose:
*
*		Perform a read-modify-write on a DMI32 register.
*
*	Arguments:
*
*		handle		The handle of the device to access.
*
*		reg			The register to read.
*
*		value		The value to write. Excess bits are ignored.
*
*		mask		The set of bits to modify. Excess bits are ignored.
*					If a bit is set here, then the register bits get
*					changed.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_reg_mod(void* handle, U32 reg, U32 value, U32 mask)
{
	dev_data_t*	dev;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		if (reg & GSC_REG_PCI)
			status	= (U32) GSC_ACCESS_DENIED;
		else if (reg & GSC_REG_PLX)
			status	= (U32) GSC_ACCESS_DENIED;
		else
			status	= GSC_REG_MOD(dev, reg, value, mask);

		dev_access_unlock(dev);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_reg_read
*
*	Purpose:
*
*		Read a DMI32 register.
*
*	Arguments:
*
*		handle		The handle of the device to access.
*
*		reg			The register to read.
*
*		value		The value read is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_reg_read(void* handle, U32 reg, U32* value)
{
	dev_data_t*	dev;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		if (value == NULL)
			status	= (U32) GSC_SUCCESS;
		else if (reg & GSC_REG_PCI)
			status	= PCI_REG_READ(dev, reg, value);
		else if (reg & GSC_REG_PLX)
			status	= PLX_REG_READ(dev, reg, value);
		else
			status	= GSC_REG_READ(dev, reg, value);

		dev_access_unlock(dev);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_reg_write
*
*	Purpose:
*
*		Write to a DMI32 register.
*
*	Arguments:
*
*		handle		The handle of the device to access.
*
*		reg			The register to read.
*
*		value		The value to write. Excess bits are ignored.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_reg_write(void* handle, U32 reg, U32 value)
{
	dev_data_t*	dev;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		if (reg & GSC_REG_PCI)
			status	= (U32) GSC_ACCESS_DENIED;
		else if (reg & GSC_REG_PLX)
			status	= (U32) GSC_ACCESS_DENIED;
		else
			status	= GSC_REG_WRITE(dev, reg, value);

		dev_access_unlock(dev);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	reg_firmware_mod
*
*	Purpose:
*
*		Perform a read-modify-write operation on a DMI32 firmware register.
*		This requires that the firmware registers be mapped into the BAR2
*		memory region.
*
*	Arguments:
*
*		dev			The device data structure.
*
*		reg			The register to read.
*
*		value		The value to write. Excess bits are ignored.
*
*		mask		The set of bits to modify. Excess bits are ignored.
*					If a bit is set here, then the register bits get
*					changed.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 reg_firmware_mod(dev_data_t* dev, U32 reg, U32 value, U32 mask)
{
	U32	status;

	status	= dev_access_dram(dev, 1, 0);

	if (status == (U32) GSC_SUCCESS)
	{
		status	= gsc_bar_reg_mod(&dev->gsc_regs, reg, value, mask);
		dev_access_dram(dev, 0, 0);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	reg_firmware_read
*
*	Purpose:
*
*		Read from a DMI32 firmware register. This requires that the firmware
*		registers be mapped into the BAR2 memory region.
*
*	Arguments:
*
*		dev			The device data structure.
*
*		reg			The register to read.
*
*		value		The value to read is put here. Excess bits are ignored.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 reg_firmware_read(dev_data_t* dev, U32 reg, U32* value)
{
	U32	status;

	status	= dev_access_dram(dev, 1, 0);

	if (status == (U32) GSC_SUCCESS)
	{
		status	= gsc_bar_reg_read(&dev->gsc_regs, reg, value);
		dev_access_dram(dev, 0, 0);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	reg_firmware_write
*
*	Purpose:
*
*		Write to a DMI32 firmware register. This requires that the firmware
*		registers be mapped into the BAR2 memory region.
*
*	Arguments:
*
*		dev			The device data structure.
*
*		reg			The register to read.
*
*		value		The value to write. Excess bits are ignored.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 reg_firmware_write(dev_data_t* dev, U32 reg, U32 value)
{
	U32	status;

	status	= dev_access_dram(dev, 1, 0);

	if (status == (U32) GSC_SUCCESS)
	{
		status	= gsc_bar_reg_write(&dev->gsc_regs, reg, value);
		dev_access_dram(dev, 0, 0);
	}

	return(status);
}


