/* API - DMI32 - Any OS */
/* $Workfile: dev_user_io.c $ */
/* $Revision: 4 $ */
/* $Date: 10/27/07 10:47a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/api/dev_user_io.c $
 *
 * 4     10/27/07 10:47a Dnewbold
 * Linux porting updates.
 *
 * 3     3/01/07 8:33a Dnewbold
 * Linted.
 *
 * 2     5/05/06 10:39a Dnewbold
 * Worked to increase modularity.
 *
 * 1     8/22/05 3:40p Dnewbold
*/

#include "dev_user_io.h"

#include "dev_access.h"
#include "dev_config.h"
#include "dev_reg.h"



/* variables	*************************************************************/

typedef enum
{
	_CURRENT_NONE,
	_CURRENT_INPUT,
	_CURRENT_OUTPUT
} current_t;

static const struct
{
	U32	bit;	// In BCR
	U32	shift;	// in BCR and BSR.
} access[2]	=
{
	{	// Port A
		/* bit		*/	DMI32_BCR_USER_IO_A_DIR,
		/* shift	*/	8
	},
	{	// Port B
		/* bit		*/	DMI32_BCR_USER_IO_B_DIR,
		/* shift	*/	15
	}
};



/*****************************************************************************
*
*	Function:	_apply_current_which
*
*	Purpose:
*
*		Update the "which" bits to use based upon the current parameter's
*		notion of CURRENT PORT.
*
*	Arguments:
*
*		dev			The device to access.
*
*		current		The parameter's notion of current.
*
*		mask		The qualification mask to modify.
*
*		which		The actual which parameter to modify.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _apply_current_which(dev_data_t* dev, U32 current, U32* mask, U32* which)
{
	U32	bcr		= 0;
	U32	status;

	mask[0]	= DMI32_WHICH_USER_IO_A_B;

	if (current != (U32) _CURRENT_NONE)
		mask[0]	= DMI32_WHICH_USER_IO_CURRENT;

	if (current != (U32) _CURRENT_NONE)
		status	= GSC_REG_READ(dev, DMI32_BCR, &bcr);
	else
		status	= (U32) GSC_SUCCESS;

	if (current == (U32) _CURRENT_OUTPUT)
	{
		if (bcr & DMI32_BCR_USER_IO_A_DIR)
		{
			// A is an output.
			// B may be also, but is disabled.
			which[0]	|= DMI32_WHICH_USER_IO_A;
		}
		else if (bcr & DMI32_BCR_USER_IO_B_DIR)
		{
			// A is an input.
			// B is an output.
			which[0]	|= DMI32_WHICH_USER_IO_B;
		}
		else
		{
			// A is an input.
			// B is an input, but is disabled.
		}
	}
	else if (current == (U32) _CURRENT_INPUT)
	{
		if ((bcr & DMI32_BCR_USER_IO_A_DIR) == 0)
		{
			// A is an input.
			// B may be also, but is disabled.
			which[0]	|= DMI32_WHICH_USER_IO_A;
		}
		else if ((bcr & DMI32_BCR_USER_IO_B_DIR) == 0)
		{
			// A is an output.
			// B is an input.
			which[0]	|= DMI32_WHICH_USER_IO_B;
		}
		else
		{
			// A is an output.
			// B is an output, but is disabled.
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_user_io_service
*
*	Purpose:
*
*		Perform processing common to all User I/O parameters, then call the
*		given service to perform the parameter specific processing.
*
*	Arguments:
*
*		dev			The device to access.
*
*		current		Which direction does the parameter apply to?
*
*		func		This is the parameter specific service.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _user_io_service(dev_data_t*	dev,
							current_t	current,
							U32			(*func)(dev_data_t*	dev,
												int			is_a,
												UL			set,
												UL*			get),
							U32			which,
							UL			set,
							UL*		get)
{
	U32	mask;
	U32	stat;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= _apply_current_which(dev, (U32) current, &mask, &which);
		if (status != (U32) GSC_SUCCESS)	break;
		status	= config_which_validate(dev, which, mask);
		if (status != (U32) GSC_SUCCESS)	break;

		if (which & DMI32_WHICH_USER_IO_A)
		{
			stat	= (func)(dev, 1, set, get);
			status	= (status == (U32) GSC_SUCCESS) ? stat : status;
		}

		if (which & DMI32_WHICH_USER_IO_B)
		{
			stat	= (func)(dev, 0, set, get);
			status	= (status == (U32) GSC_SUCCESS) ? stat : status;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_direction_side
*
*	Purpose:
*
*		Implement the User I/O Direction parameter service for a single A/B
*		side. Validate and apply the setting, as appropriate, then retrieve
*		the current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		is_a		Is this the A port?
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _direction_side(	dev_data_t*	dev,
							int			is_a,
							UL			set,
							UL*			get)
{
	U32	bcr;
	U32	bit;
	int	index	= is_a ? 0 : 1;
	U32	status	= (U32) GSC_SUCCESS;

	for (;;)	/* A convenience loop. */
	{
		bit	= access[index].bit;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_USER_IO_DIRECTION_DEFAULT;
		}
		else if (	(set == DMI32_USER_IO_DIRECTION_IN) ||
					(set == DMI32_USER_IO_DIRECTION_OUT))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_USER_IO_DIRECTION_OUT;
		}
		else
		{
			set	&= DMI32_USER_IO_DIRECTION_IN;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == DMI32_USER_IO_DIRECTION_OUT)
				set	= bit;
			else
				set	= 0;

			status	= GSC_REG_MOD(dev, DMI32_BCR, set, bit);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			if (bcr & bit)
				get[0]	= DMI32_USER_IO_DIRECTION_OUT;
			else
				get[0]	= DMI32_USER_IO_DIRECTION_IN;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	user_io_direction
*
*	Purpose:
*
*		Implement the User I/O Direction parameter service. Validate and
*		apply the setting, as appropriate, then retrieve the current setting
*		for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		The "which" bits to apply.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 user_io_direction(	dev_data_t*	dev,
						U32			which,
						UL			set,
						UL*			get)
{
	U32	status;

	status	= _user_io_service(	dev,
								_CURRENT_NONE,
								_direction_side,
								which,
								set,
								get);
	return(status);
}



/*****************************************************************************
*
*	Function:	_input_side
*
*	Purpose:
*
*		Implement the User I/O Input parameter service for a single A/B
*		side. Validate and apply the setting, as appropriate, then retrieve
*		the current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		is_a		Is this the A port?
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _input_side(	dev_data_t*	dev,
						int			is_a,
						UL			set,
						UL*			get)
{
	U32	bsr;
	int	index	= is_a ? 0 : 1;
	U32	status	= (U32) GSC_SUCCESS;

	for (;;)	/* A convenience loop. */
	{
		// SET is not supported by this parameter.

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_BSR, &bsr);
			if (status != (U32) GSC_SUCCESS)	break;
			get[0]	= (bsr >> access[index].shift)
					& DMI32_USER_IO_INPUT_MASK;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	user_io_input
*
*	Purpose:
*
*		Implement the User I/O Input parameter service. Validate and
*		apply the setting, as appropriate, then retrieve the current setting
*		for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		The "which" bits to apply.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 user_io_input(	dev_data_t*	dev,
					U32			which,
					UL			set,
					UL*			get)
{
	U32	status;

	status	= _user_io_service(	dev,
								_CURRENT_INPUT,
								_input_side,
								which,
								set,
								get);
	return(status);
}



/*****************************************************************************
*
*	Function:	_output_side
*
*	Purpose:
*
*		Implement the User I/O Output parameter service for a single A/B
*		side. Validate and apply the setting, as appropriate, then retrieve
*		the current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		is_a		Is this the A port?
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

static U32 _output_side(dev_data_t*	dev,
						int			is_a,
						UL			set,
						UL*			get)
{
	U32	bcr;
	int	index	= is_a ? 0 : 1;
	U32	mask;
	U32	status	= (U32) GSC_SUCCESS;

	for (;;)	/* A convenience loop. */
	{
		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_USER_IO_OUTPUT_DEFAULT;
		}
		else if ((set & DMI32_USER_IO_OUTPUT_MASK) == set)
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else
		{
			set	&= DMI32_USER_IO_OUTPUT_MASK;
		}

		if (set != GSC_NO_CHANGE)
		{
			mask	= DMI32_USER_IO_OUTPUT_MASK << access[index].shift;
			set		<<= access[index].shift;
			status	= GSC_REG_MOD(dev, DMI32_BCR, set, mask);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;
			get[0]	= (bcr >> access[index].shift)
					& DMI32_USER_IO_OUTPUT_MASK;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	user_io_output
*
*	Purpose:
*
*		Implement the User I/O Output parameter service. Validate and
*		apply the setting, as appropriate, then retrieve the current setting
*		for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		The "which" bits to apply.
*
*		set			The value to apply.
*
*		get			Retriev the current setting and record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 user_io_output(	dev_data_t*	dev,
					U32			which,
					UL			set,
					UL*			get)
{
	U32	status;

	status	= _user_io_service(	dev,
								_CURRENT_OUTPUT,
								_output_side,
								which,
								set,
								get);
	return(status);
}


