/* API - DMI32 - Any OS */
/* $Workfile: dev_xfer.c $ */
/* $Revision: 5 $ */
/* $Date: 10/27/07 10:48a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/api/dev_xfer.c $
 *
 * 5     10/27/07 10:48a Dnewbold
 * Linux porting updates.
 *
 * 4     3/01/07 8:33a Dnewbold
 * Linted.
 *
 * 3     5/05/06 10:39a Dnewbold
 * Worked to increase modularity.
 *
 * 2     5/04/06 3:58p Dnewbold
 * Removed key validation code and increased modularity.
 *
 * 1     8/22/05 3:40p Dnewbold
*/

#include "dev_xfer.h"

#include "gsc_sleep.h"

#include "dev_config.h"
#include "dev_reg.h"



/*****************************************************************************
*
*	Function:	xfer_clkin_test
*
*	Purpose:
*
*		Implement the Transfer CLKIN Test parameter service. Validate and
*		apply the setting, as appropriate, then retrieve the current setting
*		for the caller, also as appropriate. If asked to do so, we initiate
*		a test. The GET argument retrieves the results of the test.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 xfer_clkin_test(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bsr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_XFER_CLKIN_TEST_DEFAULT;
		}
		else if (	(set == DMI32_XFER_CLKIN_TEST_NO) ||
					(set == DMI32_XFER_CLKIN_TEST_YES))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_XFER_CLKIN_TEST_YES;
		}
		else
		{
			set	= DMI32_XFER_CLKIN_TEST_NO;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == DMI32_XFER_CLKIN_TEST_YES)
			{
				status	= GSC_REG_MOD(	dev,
										DMI32_BCR,
										DMI32_BCR_CLKIN_TEST,
										DMI32_BCR_CLKIN_TEST);
				gsc_sleep_ms(1);
				if (status != (U32) GSC_SUCCESS)	break;
			}
		}

		if (get)
		{
				status	= GSC_REG_READ(dev, DMI32_BSR, &bsr);
				if (status != (U32) GSC_SUCCESS)	break;
				set	= bsr & DMI32_BSR_CLKIN_STATUS;

				if (set == DMI32_BSR_CLKIN_STATUS_NONE)
				{
					// A clock was not detected.
					get[0]	= DMI32_XFER_CLKIN_TEST_NONE;
				}
				else if (set == DMI32_BSR_CLKIN_STATUS_OK)
				{
					// A clock was detected.
					get[0]	= bsr & DMI32_BSR_CLKIN_COUNT;
					get[0]	= get[0] >> DMI32_BSR_CLKIN_SHIFT;
					get[0]	= (get[0] + 3) / 5;
				}
				else
				{
					// The test is either still busy or not requested.
					get[0]	= DMI32_XFER_CLKIN_TEST_BUSY;
				}
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	xfer_direction
*
*	Purpose:
*
*		Implement the Transfer Direction parameter service. Validate and
*		apply the setting, as appropriate, then retrieve the current setting
*		for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 xfer_direction(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_XFER_DIRECTION_DEFAULT;
		}
		else if (	(set == DMI32_XFER_DIRECTION_RX) ||
					(set == DMI32_XFER_DIRECTION_TX))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_XFER_DIRECTION_TX;
		}
		else
		{
			set	&= DMI32_XFER_DIRECTION_RX;
		}

		if (set != GSC_NO_CHANGE)
		{
			status	= GSC_REG_READ(dev, DMI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			if (bcr & DMI32_BCR_XFER_ENABLE)
			{
				// Not permitted once transfer enabled.
				status	= (U32) GSC_FAILED;
				break;
			}

			if (set == DMI32_XFER_DIRECTION_TX)
				set	= DMI32_BCR_XFER_DIR;
			else
				set	= 0;

			status	= GSC_REG_MOD(dev, DMI32_BCR, set, DMI32_BCR_XFER_DIR);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			if (bcr & DMI32_BCR_XFER_DIR)
				get[0]	= DMI32_XFER_DIRECTION_TX;
			else
				get[0]	= DMI32_XFER_DIRECTION_RX;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	xfer_enable
*
*	Purpose:
*
*		Implement the Transfer Flow Control parameter service. Validate and
*		apply the setting, as appropriate, then retrieve the current setting
*		for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 xfer_enable(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_XFER_ENABLE_DEFAULT;
		}
		else if (	(set == DMI32_XFER_ENABLE_NO) ||
					(set == DMI32_XFER_ENABLE_YES))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_XFER_ENABLE_YES;
		}
		else
		{
			set	= DMI32_XFER_ENABLE_NO;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == DMI32_XFER_ENABLE_YES)
				set	= DMI32_BCR_XFER_ENABLE;
			else
				set	= 0;

			status	= GSC_REG_MOD(dev, DMI32_BCR, set, DMI32_BCR_XFER_ENABLE);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= (bcr & DMI32_BCR_XFER_ENABLE)
					? DMI32_XFER_ENABLE_YES
					: DMI32_XFER_ENABLE_NO;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	xfer_internal_clock_speed
*
*	Purpose:
*
*		Implement the Transfer Internal Clock Speed parameter service.
*		Validate and apply the setting, as appropriate, then retrieve the
*		current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 xfer_internal_clock_speed(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_XFER_INTERNAL_CLOCK_SPEED_DEFAULT;
		}
		else if (	(set == DMI32_XFER_INTERNAL_CLOCK_SPEED_HIGH) ||
					(set == DMI32_XFER_INTERNAL_CLOCK_SPEED_LOW))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_XFER_INTERNAL_CLOCK_SPEED_HIGH;
		}
		else
		{
			set	&= DMI32_XFER_INTERNAL_CLOCK_SPEED_LOW;
		}

		if (set != GSC_NO_CHANGE)
		{
			status	= GSC_REG_READ(dev, DMI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			if (bcr & DMI32_BCR_XFER_ENABLE)
			{
				// Not permitted once transfer enabled.
				status	= (U32) GSC_FAILED;
				break;
			}

			if (set == DMI32_XFER_INTERNAL_CLOCK_SPEED_LOW)
				set	= DMI32_BCR_INT_CLOCK_SPEED;
			else
				set	= 0;

			status	= GSC_REG_MOD(dev, DMI32_BCR, set, DMI32_BCR_INT_CLOCK_SPEED);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			if (bcr & DMI32_BCR_INT_CLOCK_SPEED)
				get[0]	= DMI32_XFER_INTERNAL_CLOCK_SPEED_LOW;
			else
				get[0]	= DMI32_XFER_INTERNAL_CLOCK_SPEED_HIGH;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	xfer_mode
*
*	Purpose:
*
*		Implement the Transfer Mode parameter service. Validate and apply the
*		setting, as appropriate, then retrieve the current setting for the
*		caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 xfer_mode(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_XFER_MODE_DEFAULT;
		}
		else if (	(set == DMI32_XFER_MODE_SINGLE) ||
					(set == DMI32_XFER_MODE_MULTI))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_XFER_MODE_MULTI;
		}
		else
		{
			set	&= DMI32_XFER_MODE_SINGLE;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == DMI32_XFER_MODE_MULTI)
				set	= DMI32_BCR_XFER_MODE;
			else
				set	= 0;

			status	= GSC_REG_MOD(dev, DMI32_BCR, set, DMI32_BCR_XFER_MODE);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			if (bcr & DMI32_BCR_XFER_MODE)
				get[0]	= DMI32_XFER_MODE_MULTI;
			else
				get[0]	= DMI32_XFER_MODE_SINGLE;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	xfer_offset
*
*	Purpose:
*
*		Implement the Transfer Offset parameter service. Validate and apply
*		the setting, as appropriate, then retrieve the current setting for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 xfer_offset(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;
	U32	value;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_XFER_OFFSET_DEFAULT;
		}
		else if (	(set >= DMI32_XFER_OFFSET_MIN) &&
					(set <= DMI32_XFER_OFFSET_MAX) &&
					((set & DMI32_XFER_OFFSET_MASK) == set))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set > DMI32_XFER_OFFSET_MAX)
		{
			set	= DMI32_XFER_OFFSET_MAX;
		}
		else if (set < DMI32_XFER_OFFSET_MIN)
		{
			set	= DMI32_XFER_OFFSET_MIN;
		}
		else
		{
			set	&= DMI32_XFER_OFFSET_MASK;
		}

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (dev->strict_dram == 0)
		{
		}
		else if (set <= (dev->dram_size_mb * 1024 * 1024 - 4))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else
		{
			set	=	(dev->dram_size_mb * 1024 * 1024) - 4;
		}

		if (set != GSC_NO_CHANGE)
		{
			status	= GSC_REG_WRITE(dev, DMI32_TBBAR, set);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_TBBAR, &value);
			get[0]	= value;
			if (status != (U32) GSC_SUCCESS)	break;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	xfer_size
*
*	Purpose:
*
*		Implement the Transfer Size parameter service. Validate and apply
*		the setting, as appropriate, then retrieve the current setting for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 xfer_size(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	limit;
	U32	status;
	U32	tbbar;
	U32	value;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_XFER_SIZE_DEFAULT;
		}
		else if (set == DMI32_XFER_SIZE_4GB)
		{
			/*
			*	This is a special case since a value of 4GB requires 33-bits.
			*	Since we're using 32-bit values, 4GB comes out at 0. This is
			*	a special case for the hardware, not the driver.
			*/
		}
		else if (	(set >= DMI32_XFER_SIZE_MIN) &&
					(set <= DMI32_XFER_SIZE_MAX) &&
					((set & DMI32_XFER_SIZE_MASK) == set))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set > DMI32_XFER_SIZE_MAX)
		{
			set	= DMI32_XFER_SIZE_MAX;
		}
		else if (set < DMI32_XFER_SIZE_MIN)
		{
			set	= DMI32_XFER_SIZE_MIN;
		}
		else
		{
			set	&= DMI32_XFER_SIZE_MASK;
		}

		status	= GSC_REG_READ(dev, DMI32_TBBAR, &tbbar);
		if (status != (U32) GSC_SUCCESS)	break;
		limit	= (dev->dram_size_mb * 1024 * 1024) - tbbar;

		if (set == GSC_NO_CHANGE)
		{
			// No change requested.
		}
		else if (dev->strict_dram == 0)
		{
			// They can do as they please.
		}
		else if ((dev->dram_size_mb >= 4096) && (tbbar == 0))
		{
			// Any size is valid.
		}
		else if ((dev->dram_size_mb < 4096) &&
				(tbbar >= (dev->dram_size_mb * 1024 * 1024)))
		{
			/*
			*	The application has manually set the transfer Buffer Base
			*	Address Register to an inappropriate value. We have to
			*	presume that the application is doing this intentionally.
			*	We will therfore let the application have its way.
			*/
		}
		else if ((set != DMI32_XFER_SIZE_4GB) && (set <= limit))
		{
			// The size is valid.
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else
		{
			set	= limit;
		}

		if (set != GSC_NO_CHANGE)
		{
			status	= GSC_REG_WRITE(dev, DMI32_TBSR, set);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_TBSR, &value);
			get[0]	= value;
			if (status != (U32) GSC_SUCCESS)	break;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	xfer_start_word
*
*	Purpose:
*
*		Implement the Transfer Start Word parameter service. Validate and
*		apply the setting, as appropriate, then retrieve the current setting
*		for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 xfer_start_word(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	status;
	U32	value;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_XFER_START_WORD_DEFAULT;
		}
		else if ((set & DMI32_XFER_START_WORD_MASK) == set)
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else
		{
			set	&= DMI32_XFER_START_WORD_MASK;
		}

		if (set != GSC_NO_CHANGE)
		{
			status	= GSC_REG_WRITE(dev, DMI32_TSWR, set);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_TSWR, &value);
			get[0]	= value;
			if (status != (U32) GSC_SUCCESS)	break;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	xfer_start_word_dir
*
*	Purpose:
*
*		Implement the Transfer Start Word Direction parameter service.
*		Validate and apply the setting, as appropriate, then retrieve the
*		current setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 xfer_start_word_dir(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_XFER_START_WORD_DIR_DEFAULT;
		}
		else if (	(set == DMI32_XFER_START_WORD_DIR_RX) ||
					(set == DMI32_XFER_START_WORD_DIR_TX))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_XFER_START_WORD_DIR_TX;
		}
		else
		{
			set	&= DMI32_XFER_START_WORD_DIR_RX;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (dev->features.start_word_dir == 0)
			{
			}
			else
			{
				if (set == DMI32_XFER_START_WORD_DIR_RX)
					set	= DMI32_BCR_START_WORD_DIR;
				else
					set	= 0;

				status	= GSC_REG_MOD(	dev,
										DMI32_BCR,
										set,
										DMI32_BCR_START_WORD_DIR);
				if (status != (U32) GSC_SUCCESS)	break;
			}
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			if (dev->features.start_word_dir)
			{
				// Consult the BCR Start Word Direction bit.

				if (bcr & DMI32_BCR_START_WORD_DIR)
					get[0]	= DMI32_XFER_START_WORD_DIR_RX;
				else
					get[0]	= DMI32_XFER_START_WORD_DIR_TX;
			}
			else
			{
				// Consult the BCR Test Mode bit.

				if (bcr & DMI32_BCR_TEST_MODE)
					get[0]	= DMI32_XFER_START_WORD_DIR_RX;
				else
					get[0]	= DMI32_XFER_START_WORD_DIR_TX;
			}
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	xfer_state
*
*	Purpose:
*
*		Implement the Transfer State parameter service. Validate and apply
*		the setting, as appropriate, then retrieve the current setting for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 xfer_state(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET not supported

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= (bcr & DMI32_BCR_XFER_ENABLE)
					? DMI32_XFER_STATE_ACTIVE
					: DMI32_XFER_STATE_INACTIVE;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	xfer_state_rx
*
*	Purpose:
*
*		Implement the Transfer State Rx parameter service. Validate and apply
*		the setting, as appropriate, then retrieve the current setting for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 xfer_state_rx(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bsr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET not supported

		if (get == NULL)
		{
		}
		else
		{
			// Check the BSR Transmit In Progress bit.
			status	= GSC_REG_READ(dev, DMI32_BSR, &bsr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= (bsr & DMI32_BSR_RX_IN_PROGRESS)
					? DMI32_XFER_STATE_RX_ACTIVE
					: DMI32_XFER_STATE_RX_INACTIVE;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	xfer_state_tx
*
*	Purpose:
*
*		Implement the Transfer State Tx parameter service. Validate and apply
*		the setting, as appropriate, then retrieve the current setting for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 xfer_state_tx(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bsr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET not supported

		if (get == NULL)
		{
		}
		else
		{
			// Check the BSR Transmit In Progress bit.
			status	= GSC_REG_READ(dev, DMI32_BSR, &bsr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= (bsr & DMI32_BSR_TX_IN_PROGRESS)
					? DMI32_XFER_STATE_TX_ACTIVE
					: DMI32_XFER_STATE_TX_INACTIVE;
		}

		break;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	xfer_test_mode
*
*	Purpose:
*
*		Implement the Transfer Test Mode parameter service. Validate and
*		apply the setting, as appropriate, then retrieve the current setting
*		for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 xfer_test_mode(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	/* A convenience loop. */
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= DMI32_XFER_TEST_MODE_DEFAULT;
		}
		else if (	(set == DMI32_XFER_TEST_MODE_NO) ||
					(set == DMI32_XFER_TEST_MODE_YES))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= DMI32_XFER_TEST_MODE_YES;
		}
		else
		{
			set	&= DMI32_XFER_TEST_MODE_NO;
		}

		if (set != GSC_NO_CHANGE)
		{
			status	= GSC_REG_READ(dev, DMI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			if (bcr & DMI32_BCR_XFER_ENABLE)
			{
				// Not permitted once transfer enabled.
				status	= (U32) GSC_FAILED;
				break;
			}

			if (set == DMI32_XFER_TEST_MODE_YES)
				set	= DMI32_BCR_TEST_MODE;
			else
				set	= 0;

			status	= GSC_REG_MOD(dev, DMI32_BCR, set, DMI32_BCR_TEST_MODE);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, DMI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			if (bcr & DMI32_BCR_TEST_MODE)
				get[0]	= DMI32_XFER_TEST_MODE_YES;
			else
				get[0]	= DMI32_XFER_TEST_MODE_NO;
		}

		break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_xfer_buf_get
*
*	Purpose:
*
*		Retrieve the transfer buffer offset and size.
*
*	Arguments:
*
*		handle		The device to access.
*
*		offset		The desired buffer offset.
*
*		size		The desired buffer size.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_xfer_buf_get(void* handle, U32* offset, U32* size)
{
	dev_data_t*		dev;
	U32				status;
	unsigned long	value;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		status	= xfer_offset(dev, 0, GSC_NO_CHANGE, &value);

		if (offset)
			offset[0]	= (U32) (value & 0xFFFFFFFF);

		if (status == (U32) GSC_SUCCESS)
		{
			status	= xfer_size(dev, 0, GSC_NO_CHANGE, &value);

			if (size)
				size[0]	= value;
		}

		dev_access_unlock(dev);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_xfer_buf_set
*
*	Purpose:
*
*		Set the transfer buffer offset and size.
*
*	Arguments:
*
*		handle		The device to access.
*
*		offset		The desired buffer offset.
*
*		size		The desired buffer size.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_xfer_buf_set(void* handle, U32 offset, U32 size)
{
	dev_data_t*	dev;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		status	= xfer_offset(dev, 0, offset, NULL);

		if (status == (U32) GSC_SUCCESS)
			status	= xfer_size(dev, 0, size, NULL);

		dev_access_unlock(dev);
	}

	return(status);
}


