/* Application - DMI32 - Any OS */
/* $Workfile: irq.c $ */
/* $Revision: 3 $ */
/* $Date: 10/27/07 10:54a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/bbtest/irq.c $
 *
 * 3     10/27/07 10:54a Dnewbold
 * Additional porting updates.
 *
 * 2     4/05/07 2:36p Dnewbold
 * Ported to Linux.
 *
 * 1     8/22/05 4:08p Dnewbold
*/

#include <stdio.h>

#include "main.h"
#include "util_label.h"
#include "util_sleep.h"



/* variables	*************************************************************/

static	U32		_callback_arg;



/*****************************************************************************
*
*	Function:	_irq_callback
*
*	Purpose:
*
*		This is an API callback function. Its purpose is to record ARG3.
*		Also as soon as the callback is made it uninstalls itself.
*
*	Arguments:
*
*		arg1	This is the device handle.
*
*		arg2	This is the user argument.
*
*		arg3	For interrupt callbacks this is the which bit for the
*				interrupt.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static void _irq_callback(void* arg1, U32 arg2, U32 arg3)
{
	void*	handle	= (void*) arg1;

	_callback_arg	= arg2;
	DMI32_IRQ_CALLBACK_FUNC__SET(handle, arg2, 0);
}



/*****************************************************************************
*
*	Function:	_irq_buffer_begin_test
*
*	Purpose:
*
*		Verify the Buffer Begin Interrupt.
*
*	Arguments:
*
*		h0		The handle to the first device under test.
*
*		h1		The handle to the second device under test.
*
*		clock	Do both boards have a clock.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_buffer_begin_test(void* h0, void* h1, U32 clock)
{
	int			err		= 0;
	int			errs	= 0;
	const char*	msg		= NULL;
	U32			status	= GSC_SUCCESS;

	for (;;)	// A convenience loop.
	{
		label_text("Buffer Begin");

		if (clock == 0)
			break;

		// Start data transfer.
		msg	= "initialization";

		err	= xfer_activity_config(h0, h1);
		if (err)	break;

		/*
		*	Since the interrupt status isn't latched when the interrupt is
		*	disabled, we can't verify that the condition has occurred. We'll
		*	have to enable and catch the interrupt to do that. That is what
		*	we're doing here.
		*/

		msg	= "setup";
		_callback_arg	= 0;

		status	= DMI32_IRQ_CALLBACK_FUNC__BS_SET(h0, (unsigned long) _irq_callback);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_IRQ_ENABLE__BS_YES(h0);
		if (status != GSC_SUCCESS)	break;

		sleep_ms(100);	// Wait for the interrupt and the callback.

		msg	= "shutdown";
		status	= DMI32_IRQ_CALLBACK_FUNC__BS_SET(h0, 0);	// Just in case.
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_IRQ_ENABLE__BS_NO(h0);
		if (status != GSC_SUCCESS)	break;

		// Did the interrupt occur?

		msg	= "evaluate";
		if (_callback_arg != DMI32_WHICH_IRQ_BS)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// We're done.
		break;
	}

	dmi32_init(h0);
	dmi32_init(h1);
	errs	= (status == GSC_SUCCESS) ? err : 1;

	if ((clock) && (verbose) && (status != GSC_SUCCESS) && (msg))
		printf("(%s 0x%lX) ", msg, (long) _callback_arg);

	if (clock == 0)
		printf("SKIPPED (clock unavailable)\n");
	else if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_buffer_end_test
*
*	Purpose:
*
*		Verify the Buffer End Interrupt.
*
*	Arguments:
*
*		h0		The handle to the first device under test.
*
*		h1		The handle to the second device under test.
*
*		clock	Do both boards have a clock.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_buffer_end_test(void* h0, void* h1, U32 clock)
{
	int	err		= 0;
	int	errs	= 0;
	U32	status	= GSC_SUCCESS;

	for (;;)	// A convenience loop.
	{
		label_text("Buffer End");

		if (clock == 0)
			break;

		// Start data transfer.
		err	= xfer_activity_config(h0, h1);
		if (err)	break;

		/*
		*	Since the interrupt status isn't latched when the interrupt is
		*	disabled, we can't verify that the condition has occurred. We'll
		*	have to enable and catch the interrupt to do that. That is what
		*	we're doing here.
		*/

		_callback_arg	= 0;

		status	= DMI32_IRQ_CALLBACK_FUNC__BE_SET(h0, (U32) _irq_callback);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_IRQ_ENABLE__BE_YES(h0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_MODE__SINGLE(h0);
		if (status != GSC_SUCCESS)	break;

		sleep_ms(100);	// Wait for the interrupt and the callback.

		status	= DMI32_IRQ_CALLBACK_FUNC__BE_SET(h0, 0);	// Just in case.
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_IRQ_ENABLE__BE_NO(h0);
		if (status != GSC_SUCCESS)	break;

		// Did the interrupt occur?

		if (_callback_arg != DMI32_WHICH_IRQ_BE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// We're done.
		break;
	}

	dmi32_init(h0);
	dmi32_init(h1);
	errs	= (status == GSC_SUCCESS) ? err : 1;

	if (clock == 0)
		printf("SKIPPED (clock unavailable)\n");
	else if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_user_io_a_test
*
*	Purpose:
*
*		Verify the User I/O A Input Interrupt.
*
*	Arguments:
*
*		h0		The handle to the first device under test.
*
*		h1		The handle to the second device under test.
*
*		clock	Do both boards have a clock.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_user_io_a_test(void* h0, void* h1, U32 clock)
{
	int	errs	= 0;
	int	i;
	U32	status	= GSC_SUCCESS;

	for (;;)	// A convenience loop.
	{
		label_text("User I/O A Input");

		if (clock == 0)
			break;

		// Configure the first board.
		status	= dmi32_init(h0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_USER_IO_DIRECTION__A_OUT(h0);
		if (status != GSC_SUCCESS)	break;

		// Configure the second board.
		status	= dmi32_init(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_TEST_MODE__YES(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_USER_IO_DIRECTION__A_IN(h1);
		if (status != GSC_SUCCESS)	break;

		/*
		*	Since the interrupt status isn't latched when the interrupt is
		*	disabled, we can't verify that the condition has occurred. We'll
		*	have to enable and catch the interrupt to do that. That is what
		*	we're doing here.
		*/

		_callback_arg	= 0;

		status	= DMI32_IRQ_CALLBACK_FUNC__UIOA_SET(h1, (U32) _irq_callback);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_IRQ_ENABLE__UIOA_YES(h1);
		if (status != GSC_SUCCESS)	break;

		for (i = 0; i <= 5; i++)
		{
			// Do this repeatedly to deal with cable quality issues.
			status	= DMI32_USER_IO_OUTPUT__CUR_SET(h0, 0x40);
			if (status != GSC_SUCCESS)	break;
		}

		if (status != GSC_SUCCESS)	break;
		sleep_ms(10);	// Wait for the callback.

		status	= DMI32_IRQ_CALLBACK_FUNC__UIOA_SET(h1, 0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_IRQ_ENABLE__UIOA_NO(h1);
		if (status != GSC_SUCCESS)	break;

		// Did the interrupt occur?

		if (_callback_arg != DMI32_WHICH_IRQ_UIOA)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// We're done.
		break;
	}

	dmi32_init(h0);
	dmi32_init(h1);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (clock == 0)
		printf("SKIPPED (clock unavailable)\n");
	else if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_user_io_b_test
*
*	Purpose:
*
*		Verify the User I/O B Input Interrupt.
*
*	Arguments:
*
*		h0		The handle to the first device under test.
*
*		h1		The handle to the second device under test.
*
*		clock	Do both boards have a clock.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_user_io_b_test(void* h0, void* h1, U32 clock)
{
	int	errs	= 0;
	int	i;
	U32	status	= GSC_SUCCESS;

	for (;;)	// A convenience loop.
	{
		label_text("User I/O A Input");

		if (clock == 0)
			break;

		// Configure the first board.
		status	= dmi32_init(h0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_USER_IO_DIRECTION__A_IN(h0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_USER_IO_DIRECTION__B_OUT(h0);
		if (status != GSC_SUCCESS)	break;

		// Configure the second board.
		status	= dmi32_init(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_TEST_MODE__YES(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_USER_IO_DIRECTION__A_OUT(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_USER_IO_DIRECTION__B_IN(h1);
		if (status != GSC_SUCCESS)	break;

		/*
		*	Since the interrupt status isn't latched when the interrupt is
		*	disabled, we can't verify that the condition has occurred. We'll
		*	have to enable and catch the interrupt to do that. That is what
		*	we're doing here.
		*/

		_callback_arg	= 0;

		status	= DMI32_IRQ_CALLBACK_FUNC__UIOB_SET(h1, (U32) _irq_callback);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_IRQ_ENABLE__UIOB_YES(h1);
		if (status != GSC_SUCCESS)	break;

		for (i = 0; i <= 5; i++)
		{
			// Do this repeatedly to deal with cable quality issues.
			status	= DMI32_USER_IO_OUTPUT__CUR_SET(h0, 0x40);
			if (status != GSC_SUCCESS)	break;
		}

		if (status != GSC_SUCCESS)	break;
		sleep_ms(10);	// Wait for the callback.

		status	= DMI32_IRQ_CALLBACK_FUNC__UIOB_SET(h1, 0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_IRQ_ENABLE__UIOB_NO(h1);
		if (status != GSC_SUCCESS)	break;

		// Did the interrupt occur?

		if (_callback_arg != DMI32_WHICH_IRQ_UIOB)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// We're done.
		break;
	}

	dmi32_init(h0);
	dmi32_init(h1);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (clock == 0)
		printf("SKIPPED (clock unavailable)\n");
	else if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_multi_buffer_under_run_test
*
*	Purpose:
*
*		Verify the Multi-Buffer Under Run Interrupt.
*
*	Arguments:
*
*		h0		The handle to the first device under test.
*
*		h1		The handle to the second device under test.
*
*		clock	Do both boards have a clock.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_multi_buffer_under_run_test(void* h0, void* h1, U32 clock)
{
	int				err		= 0;
	int				errs	= 0;
	unsigned long	feature;
	U32				status;

	for (;;)	// A convenience loop.
	{
		label_text("Multi-Buffer Under Run");

		status	= DMI32_MISC_FEATURES__IRQ_MBUR(h0, &feature);
		if (status != GSC_SUCCESS)					break;
		if (feature != DMI32_MISC_FEATURES_PRESENT)	break;

		if (clock == 0)
			break;

		// Start data transfer.
		err	= xfer_activity_config(h0, h1);
		if (err)	break;

		/*
		*	Since the interrupt status isn't latched when the interrupt is
		*	disabled, we can't verify that the condition has occurred. We'll
		*	have to enable and catch the interrupt to do that. That is what
		*	we're doing here.
		*/

		_callback_arg	= 0;

		status	= DMI32_IRQ_CALLBACK_FUNC__MBUR_SET(h0, (U32) _irq_callback);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_IRQ_ENABLE__MBUR_YES(h0);
		if (status != GSC_SUCCESS)	break;

		sleep_ms(100);	// Wait for the interrupt and the callback.

		status	= DMI32_IRQ_CALLBACK_FUNC__MBUR_SET(h0, 0);	// Just in case.
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_IRQ_ENABLE__MBUR_NO(h0);
		if (status != GSC_SUCCESS)	break;

		// Did the interrupt occur?

		if (_callback_arg != DMI32_WHICH_IRQ_MBUR)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// We're done.
		break;
	}

	dmi32_init(h0);
	dmi32_init(h1);
	errs	= (status == GSC_SUCCESS) ? err : 1;

	if (feature != DMI32_MISC_FEATURES_PRESENT)
		printf("SKIPPED (interrupt unavailable)\n");
	else if (clock == 0)
		printf("SKIPPED (clock unavailable)\n");
	else if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	irq_tests
*
*	Purpose:
*
*		Verify interrupt processing using the first board as the source and
*		the second board as the destination.
*
*	Arguments:
*
*		i0		The index of the first board being accessed.
*
*		h0		The handle to the first device under test.
*
*		i1		The index of the second board being accessed.
*
*		h1		The handle to the second device under test.
*
*		clock	Is there a clock on both boards?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int irq_tests(int i0, void* h0, int i1, void* h1, int clock)
{
	int		errs	= 0;
	char	label[128];

	sprintf(label, "Interrupts (%d -> %d)", i0, i1);
	label_text(label);
	printf("\n");

	label_level_inc();
	errs	+= _irq_buffer_begin_test(h0, h1, clock);
	errs	+= _irq_buffer_end_test(h0, h1, clock);
	errs	+= _irq_user_io_a_test(h0, h1, clock);
	errs	+= _irq_user_io_b_test(h0, h1, clock);
	errs	+= _irq_multi_buffer_under_run_test(h0, h1, clock);
	label_level_dec();

	return(errs);
}


