/* Application - DMI32 - Any OS */
/* $Workfile: dram.c $ */
/* $Revision: 3 $ */
/* $Date: 10/27/07 11:00a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/sbtest/dram.c $
 *
 * 3     10/27/07 11:00a Dnewbold
 * Additional porting updates.
 *
 * 2     4/05/07 2:56p Dnewbold
 * Ported to Linux.
 *
 * 1     8/22/05 4:10p Dnewbold
*/

#include <stdio.h>
#include <string.h>

#include "main.h"
#include "util_label.h"
#include "util_time.h"



/* variables	*************************************************************/

static	U32	mb_buffer[256L * 1024];



/*****************************************************************************
*
*	Function:	_format_value
*
*	Purpose:
*
*		Format a value into K, M or G units. One K is calculated at 1000
*		units.
*
*	Arguments:
*
*		value	The value to format.
*
*		units	Either 1000 or 1024.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _format_value(unsigned long value)
{
	static const struct
	{
		unsigned long	limit;
		const char*		format;
		long			div;
	} rates[]	=
	{
		/* limit	format		div		*/
		{     1000,	"%.0f ",	1		},
		{    99900,	"%.1f K",	1000	},
		{  1000000,	"%.0f K",	1000	},
		{ 99900000,	"%.1f M",	1000000	},
		{       -1,	"%.0f M",	1000000	}
	};

	int	i;

	for (i = 0; i < ARRAY_ELEMENTS(rates); i++)
	{
		if ((value < rates[i].limit) || (rates[i].limit == (unsigned long) -1))
		{
			printf(rates[i].format, (double) value / rates[i].div);
			break;
		}
	}
}



/*****************************************************************************
*
*	Function:	_buf_verify
*
*	Purpose:
*
*		Verify that the two buffers contain identical data.
*
*	Arguments:
*
*		size		The number of elements to to verify.
*
*		src			The source buffer.
*
*		dest		The buffer that should be the same as the first.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _buf_verify(U32 size, const U32* src, const U32* dest)
{
	int	i;
	U32	status	= GSC_SUCCESS;

	for (i = 0; i < (int) size; i++)
	{
		if (src[i] != dest[i])
		{
			status	= GSC_FAILED;
			printf(	"(src[%d] 0x%lX != dest[%d] 0x%lX) ",
					i,
					(long) src[i],
					i,
					(long) dest[i]);
			break;
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_dram_bit_pattern_test
*
*	Purpose:
*
*		Send a varying bit pattern to portions of DRAM and verify what is
*		read back.
*
*	Arguments:
*
*		handle		The handle used to access the device under test.
*
*		msg			Record any error message at this pointer.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An apporpriate error status.
*
*****************************************************************************/

static int _dram_bit_pattern_test(void* handle, const char** msg)
{
	#define	SAMPLES	(33 * 32 * 2)
	#define	BYTES	(SAMPLES * 4)

	int		bits;
	int		i;
	U32		mask;
	U32		pattern;
	int		pos;
	U32		rx[SAMPLES];
	U32		status	= GSC_SUCCESS;
	U32		tx[SAMPLES];
	U32		xfer;

	i		= 0;
	mask	= 0;

	for (bits = 0; bits <= 32; bits++)
	{
		for (pos = 0; pos <= 31; pos++)
		{
			pattern	= (mask << pos) | (mask >> (32 - pos));
			tx[i]	= pattern;
			i++;
			tx[i]	= ~pattern;
			i++;
		}

		mask	= (mask << 1) | 1;
	}

	status	= dmi32_init(handle);

	if (status == GSC_SUCCESS)
	{
		for (i = 0; (status == GSC_SUCCESS) && (i < 16); i++)
			status	= dmi32_write(handle, tx, sizeof(tx), &xfer);
	}

	if (status != GSC_SUCCESS)
		msg[0]	= "pattern write";

	if (status == GSC_SUCCESS)
	{
		msg[0]	= "pattern read";

		for (i = 0; (status == GSC_SUCCESS) && (i < 16); i++)
		{
			memset(rx, 0, sizeof(rx));
			status	= dmi32_read(handle, rx, sizeof(rx), &xfer);

			if (memcmp(tx, rx, sizeof(tx)))
			{
				printf("(RX %d) ", i);
				status	= GSC_FAILED;
				break;
			}
		}

		if (status != GSC_SUCCESS)
			msg[0]	= "pattern read/verify";
	}

	dmi32_init(handle);
	return(status);

	#undef	SAMPLES
	#undef	BYTES
}



/*****************************************************************************
*
*	Function:	_dram_xfer_verify_tests
*
*	Purpose:
*
*		Verify the operation of the specified I/O mode.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		name	The text describing the mode.
*
*		mode	The I/O mode to select.
*
*		size_mb	The size of installed memory.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _dram_xfer_verify_tests(void* handle, const char* name, U32 mode, U32 size_mb)
{
	#define	SAMPLES	2048
	#define	BYTES	(SAMPLES * 4)

	int			errs	= 0;
	int			i;
	const char*	msg	= NULL;
	U32			rx[SAMPLES];
	U32			status;
	U32			tx[SAMPLES];
	U32			xfer;

	label_text(name);
	status	= dmi32_init(handle);

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_MODE__TX_SET(handle, mode);

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_MODE__RX_SET(handle, mode);

	// Incrementing pattern in entire buffer.

	if (status == GSC_SUCCESS)
		msg	= "incrementing pattern fill";

	memset(rx, 0, sizeof(rx));

	for (i = 0; i < ARRAY_ELEMENTS(tx); i++)
		tx[i]	= i;

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_OFFSET__TX_SET(handle, 0);

	if (status == GSC_SUCCESS)
		status	= dmi32_write(handle, tx, BYTES, &xfer);

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_OFFSET__RX_SET(handle, 0);

	if (status == GSC_SUCCESS)
		status	= dmi32_read(handle, rx, BYTES, &xfer);

	if (status == GSC_SUCCESS)
		status	= _buf_verify(SAMPLES, tx, rx);

	if (status == GSC_SUCCESS)
		msg	= NULL;

	// DEADBEEF all by itself near the end of the buffer.
	// Make sure reads and writes don't transfer extra data.

	if (status == GSC_SUCCESS)
		msg	= "buffer ends";

	memset(rx, 0, sizeof(rx));
	tx[SAMPLES - 2]	= 0xDEADBEEF;
	tx[SAMPLES - 1]	= 0;

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_OFFSET__TX_SET(handle, BYTES - 8);

	if (status == GSC_SUCCESS)
		status	= dmi32_write(handle, tx + SAMPLES - 2, 4, &xfer);

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_OFFSET__RX_SET(handle, 0);

	if (status == GSC_SUCCESS)
		status	= dmi32_read(handle, rx, BYTES - 4, &xfer);

	if (status == GSC_SUCCESS)
		status	= _buf_verify(SAMPLES, tx, rx);

	if (status == GSC_SUCCESS)
		msg	= NULL;

	// Copy first half to second half, keeping the last two values.
	// Make sure the "file pointer" advances.

	if (status == GSC_SUCCESS)
		msg	= "pattern copy";

	memcpy(tx + SAMPLES / 2, tx, (BYTES / 2) - 4);
	memset(rx, 0, sizeof(rx));
	tx[SAMPLES - 2]	= 0;
	tx[SAMPLES - 1]	= 1;

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_OFFSET__TX_SET(handle, 0);

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_OFFSET__RX_SET(handle, 0);

	if (status == GSC_SUCCESS)
		status	= dmi32_write(handle, tx, BYTES / 2, &xfer);

	if (status == GSC_SUCCESS)
		status	= dmi32_write(handle, tx + SAMPLES / 2, (BYTES / 2) - 4, &xfer);

	if (status == GSC_SUCCESS)
		status	= dmi32_write(handle, tx + SAMPLES - 2, 4, &xfer);

	if (status == GSC_SUCCESS)
		status	= dmi32_write(handle, tx + SAMPLES - 1, 4, &xfer);

	if (status == GSC_SUCCESS)
		status	= dmi32_read(handle, rx, BYTES / 2, &xfer);

	if (status == GSC_SUCCESS)
		status	= dmi32_read(handle, rx + SAMPLES / 2, (BYTES / 2) - 4, &xfer);

	if (status == GSC_SUCCESS)
		status	= dmi32_read(handle, rx + SAMPLES - 2, 4, &xfer);

	if (status == GSC_SUCCESS)
		status	= dmi32_read(handle, rx + SAMPLES - 1, 4, &xfer);

	if (status == GSC_SUCCESS)
		status	= _buf_verify(SAMPLES, tx, rx);

	if (status == GSC_SUCCESS)
		msg	= NULL;

	// Verify I/O at device and driver boundary conditions.

	if (size_mb > 2048)
	{
		if (status == GSC_SUCCESS)
			msg	= "boundary conditions";

		// Put NULLs at zero.
		memset(tx, 0, sizeof(rx));

		if (status == GSC_SUCCESS)
			status	= DMI32_IO_OFFSET__TX_SET(handle, 0);

		if (status == GSC_SUCCESS)
			status	= dmi32_write(handle, tx, sizeof(tx), &xfer);

		// Put an incrementing pattern across the 2GB boundary.

		for (i = 0; i < ARRAY_ELEMENTS(tx); i++)
			tx[i]	= i;

		if (status == GSC_SUCCESS)
			status	= DMI32_IO_OFFSET__TX_SET(handle, _2GB - 64);

		if (status == GSC_SUCCESS)
			status	= dmi32_write(handle, tx, sizeof(tx), &xfer);

		// Verify that the pattern we just wrote is there.

		if (status == GSC_SUCCESS)
			status	= DMI32_IO_OFFSET__RX_SET(handle, _2GB - 64);

		if (status == GSC_SUCCESS)
			status	= dmi32_read(handle, rx, sizeof(rx), &xfer);

		if (status == GSC_SUCCESS)
			status	= _buf_verify(SAMPLES, tx, rx);

		// Verify that the NULLs are still there as well.
		memset(tx, 0, sizeof(rx));

		if (status == GSC_SUCCESS)
			status	= DMI32_IO_OFFSET__RX_SET(handle, 0);

		if (status == GSC_SUCCESS)
			status	= dmi32_read(handle, rx, sizeof(rx), &xfer);

		if (status == GSC_SUCCESS)
			status	= _buf_verify(SAMPLES, tx, rx);

		if (status == GSC_SUCCESS)
			msg	= NULL;
	}

	dmi32_init(handle);

	if (status == GSC_SUCCESS)
		status	= _dram_bit_pattern_test(handle, &msg);

	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (errs)
		printf("FAIL <---  (%s)\n", msg);
	else
		printf("PASS\n");

	return(errs);

	#undef	SAMPLES
	#undef	BYTES
}



/*****************************************************************************
*
*	Function:	_rate_test
*
*	Purpose:
*
*		Perform a throughput test.
*
*	Arguments:
*
*		handle		The handle of the board to access.
*
*		seconds		The number of seconds over which to run the test.
*
*		tx			Is this a write() test? Otherwise read().
*
*		rate		The transfer rate is recorded here.
*
*		buffer		The transfer buffer.
*
*		size		The size of the buffer, in bytes.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _rate_test(	void*	handle,
						U32		seconds,
						U32		tx,
						U32*	rate,
						void*	buffer,
						size_t	size)
{
	long	begin;
	U32		bytes;
	long	end;
	long	ms;
	U32		request;
	long	sec_ms	= 1000L * seconds;
	U32		xfer;
	U8*		src;
	U32		status	= GSC_SUCCESS;

	begin	= time_delta_ms();

	for (bytes = 0, request = 0; status == GSC_SUCCESS;)
	{
		if (request == 0)
		{
			request	= size;
			src		= (U8*) buffer;
		}

		if (tx)
		{
			status	= dmi32_write(handle, buffer, size, &xfer);

			if (status != GSC_SUCCESS)
			{
				printf(	"(dmi32_write(handle, buf, %lu, %lu) error = 0x%lX) ",
						(long) size,
						(long) xfer,
						(long) status);
			}
		}
		else
		{
			status	= dmi32_read(handle, buffer, size, &xfer);

			if (status != GSC_SUCCESS)
			{
				printf(	"(dmi32_read(handle, buf, %lu, %lu) error = 0x%lX) ",
						(long) size,
						(long) xfer,
						(long) status);
			}
		}

		request	-= xfer;
		bytes	+= xfer;
		src		+= xfer;
		end		= time_delta_ms();
		ms		= end - begin;

		if (ms >= sec_ms)
		{
			rate[0]	= (long) (1000.0 * bytes / ms);
			break;
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dram_access_rates
*
*	Purpose:
*
*		Perform a series of throughput tests for DRAM access from the PCI
*		bus.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors encounterred.
*
*****************************************************************************/

int dram_access_rates(void* handle)
{
	static const struct
	{
		const char*	name;
		U32			mode;
		U32			api_buf;
		U32			tx;
	} list[]	=
	{
		{
			/* name		*/	"PIO (Write, User Buf)",
			/* mode		*/	DMI32_IO_MODE_PIO,
			/* api_buf	*/	0,
			/* tx		*/	1
		},
		{
			/* name		*/	"PIO (Write, API Buf)",
			/* mode		*/	DMI32_IO_MODE_PIO,
			/* api_buf	*/	1,
			/* tx		*/	1
		},
		{
			/* name		*/	"PIO (Read, User Buf)",
			/* mode		*/	DMI32_IO_MODE_PIO,
			/* api_buf	*/	0,
			/* tx		*/	0
		},
		{
			/* name		*/	"PIO (Read, API Buf)",
			/* mode		*/	DMI32_IO_MODE_PIO,
			/* api_buf	*/	1,
			/* tx		*/	0
		},
		{
			/* name		*/	"DMA (Write, User Buf)",
			/* mode		*/	DMI32_IO_MODE_DMA,
			/* api_buf	*/	0,
			/* tx		*/	1
		},
		{
			/* name		*/	"DMA (Write, API Buf)",
			/* mode		*/	DMI32_IO_MODE_DMA,
			/* api_buf	*/	0,
			/* tx		*/	1
		},
		{
			/* name		*/	"DMA (Read, User Buf)",
			/* mode		*/	DMI32_IO_MODE_DMA,
			/* api_buf	*/	0,
			/* tx		*/	0
		},
		{
			/* name		*/	"DMA (Read, API Buf)",
			/* mode		*/	DMI32_IO_MODE_DMA,
			/* api_buf	*/	0,
			/* tx		*/	0
		}
	};

	unsigned long	api_buf_ptr;
	unsigned long	api_buf_size;
	int				errs		= 0;
	int				i;
	void*			ptr			= NULL;
	U32				rate		= 0;
	U32				seconds		= 3;
	U32				status;
	U32				which;

	label_text("DRAM Access Rates");
	printf("\n");
	label_level_inc();

	for (i = 0; i < ARRAY_ELEMENTS(list); i++)
	{
		label_text(list[i].name);
		which	= list[i].tx ? DMI32_WHICH_TX : DMI32_WHICH_RX;
		status	= dmi32_init(handle);

		if (status == GSC_SUCCESS)
			status	= DMI32_IO_MODE__SET(handle, which, list[i].mode);

		if (status == GSC_SUCCESS)
			status	= DMI32_IO_OFFSET__SET(handle, which, 0);

		if (status == GSC_SUCCESS)
			status	= DMI32_IO_BUFFER_SIZE__SET(handle, which, _1MB, &api_buf_size);

		if (status == GSC_SUCCESS)
			status	= DMI32_IO_BUFFER_POINTER__GET(handle, which, &api_buf_ptr);

		if (status != GSC_SUCCESS)
		{
		}
		else if (list[i].api_buf == 0)
		{
			ptr	= mb_buffer;
		}
		else if (api_buf_size < _1MB)
		{
			status	= GSC_FAILED;
			printf("(API buffer too small) ");
		}
		else if (api_buf_ptr == 0)
		{
			status	= GSC_FAILED;
			printf("(API buffer pointer is NULL) ");
		}
		else
		{
			ptr	= (void*) api_buf_ptr;
		}

		if (status == GSC_SUCCESS)
			status	= _rate_test(handle, seconds, list[i].tx, &rate, ptr, _1MB);

		if (status != GSC_SUCCESS)
		{
			errs++;
			printf("FAIL <---\n");
		}
		else
		{
			_format_value(rate);
			printf("B/S   (1K = 1000");
			printf(")\n");
		}

		dmi32_init(handle);
	}

	label_level_dec();
	return(errs);
}



/*****************************************************************************
*
*	Function:	dram_access_test
*
*	Purpose:
*
*		Perform DRAM memory tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		size_mb	The size of installed memory.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int dram_access_test(void* handle, U32 size_mb)
{
	int	errs	= 0;

	label_text("DRAM Access");
	printf("\n");

	label_level_inc();
	errs	+= _dram_xfer_verify_tests(handle, "PIO", DMI32_IO_MODE_PIO, size_mb);
	errs	+= _dram_xfer_verify_tests(handle, "DMA", DMI32_IO_MODE_DMA, size_mb);
	label_level_dec();

	return(errs);
}



/*****************************************************************************
*
*	Function:	_mb_buffer_index_fill
*
*	Purpose:
*
*		Fill the specified 1MB DRAM buffer with a pattern containing the
*		32-bit location's index.
*
*	Arguments:
*
*		handle		The handle used to access the device under test.
*
*		mb			The 1MB buffer to fill.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _mb_buffer_index_fill(void* handle, U32 mb)
{
	U32		index;
	long	l;
	U32		offset;
	U32		status;
	U32		xfer;

	index	= mb * 1024 * 1024 / 4;
	offset	= mb * 1024 * 1024;
	status	= DMI32_IO_OFFSET__TX_SET(handle, offset);

	for (l = 0; l < (int) ARRAY_ELEMENTS(mb_buffer); l++, index++)
		mb_buffer[l]	= index;

	if (status == GSC_SUCCESS)
		status	= dmi32_write(handle, mb_buffer, sizeof(mb_buffer), &xfer);

	if ((status == GSC_SUCCESS) && (xfer != sizeof(mb_buffer)))
	{
		status	= GSC_FAILED;
		printf(	"(xfer'd %ld rather than %ld bytes) ",
				(long) xfer,
				(long) sizeof(mb_buffer));
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_dram_pattern_index_fill
*
*	Purpose:
*
*		Perform a series of tests per the given data and list.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		size_mb	The size of DRAM in megabytes.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _dram_pattern_index_fill(void* handle, U32 size_mb)
{
	int	errs		= 0;
	U32	increment	= size_mb / 32;
	U32	mb;
	U32	status;

	label_text("Index Pattern Fill");
	printf("(x%ldMB) ", (long) increment);
	status	= dmi32_init(handle);

	for (mb = 0; (status == GSC_SUCCESS) && (mb < size_mb); mb++)
	{
		if ((mb % increment) == 0)
			printf(".h");

		status = _mb_buffer_index_fill(handle, mb);
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_mb_buffer_index_verify
*
*	Purpose:
*
*		Verify that the specified 1MB DRAM buffer has a pattern containing
*		the 32-bit location's index.
*
*	Arguments:
*
*		handle		The handle used to access the device under test.
*
*		mb			The 1MB buffer to fill.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _mb_buffer_index_verify(void* handle, U32 mb)
{
	U32		index;
	long	l;
	U32		offset;
	U32		status;
	U32		xfer;

	index	= mb * 1024 * 1024 / 4;
	offset	= mb * 1024 * 1024;
	status	= DMI32_IO_OFFSET__RX_SET(handle, offset);

	if (status == GSC_SUCCESS)
		status	= dmi32_read(handle, mb_buffer, sizeof(mb_buffer), &xfer);

	if ((status == GSC_SUCCESS) && (xfer != sizeof(mb_buffer)))
	{
		status	= GSC_FAILED;
		printf(	"(xfer'd %ld rather than %ld bytes) ",
				(long) xfer,
				(long) sizeof(mb_buffer));
	}

	if (status == GSC_SUCCESS)
	{
		for (l = 0; l < (int) ARRAY_ELEMENTS(mb_buffer); l++, index++)
		{
			if (mb_buffer[l] != index)
			{
				status	= GSC_FAILED;
				printf(	"(read 0x%lX rather than 0x%lX) ",
						(long) mb_buffer[l],
						(long) index);
				break;
			}
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_dram_pattern_index_verify
*
*	Purpose:
*
*		Verify that the DMI32's DRAM contains the sequential indexes from
*		zero to the end of memory.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		size_mb	The size of DRAM in megabytes.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _dram_pattern_index_verify(void* handle, U32 size_mb)
{
	int	errs		= 0;
	U32	increment	= size_mb / 32;
	U32	mb;
	U32	status;

	label_text("Index Pattern Verify");
	printf("(x%ldMB) ", (long) increment);
	status	= dmi32_init(handle);

	for (mb = 0; (status == GSC_SUCCESS) && (mb < size_mb); mb++)
	{
		if ((mb % increment) == 0)
			printf(".i");

		status = _mb_buffer_index_verify(handle, mb);
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	dram_memory_tests
*
*	Purpose:
*
*		Perform DRAM tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		size_mb	Report the size in megabyte units here.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int dram_memory_tests(void* handle, U32 size_mb)
{
	int	errs	= 0;

	label_text("DRAM Tests");
	printf("(%ldMB)\n", (long) size_mb);

	label_level_inc();
	errs	+= _dram_pattern_index_fill(handle, size_mb);
	errs	+= _dram_pattern_index_verify(handle, size_mb);
	label_level_dec();

	return(errs);
}



/*****************************************************************************
*
*	Function:	dram_compute_size_mb
*
*	Purpose:
*
*		Compute the size of installed DRAM.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		size_mb	Report the size in megabyte units here.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int dram_compute_size_mb(void* handle, U32* size_mb)
{
	#define	INC_MB	(128UL * 1024 * 1024)

	U32			end;
	int			errs;
	U32			get;
	int			i;
	U32			offset;
	U32			set;
	U32			size	= 0;
	U32			status;
	U32			xfer;

	// Use PIO since we're only doing single value transfers, which aren't
	// appropriate for DMA.
	status	= DMI32_IO_MODE__TX_PIO(handle);

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_MODE__RX_PIO(handle);

	// Place values at the end of blocks of the specified size.

	for (i = 1; status == GSC_SUCCESS; i++)
	{
		end		= (INC_MB * i - 1);
		offset	= end - 3;
		status	= DMI32_IO_OFFSET__TX_SET(handle, offset);
		if (status != GSC_SUCCESS)	break;

		set		= offset;
		status	= dmi32_write(handle, &set, 4, &xfer);
		if (status != GSC_SUCCESS)	break;

		if (end >= 0xFFFFFFFF)
			break;
	}

	// Look for the values we just wrote.

	for (i = 1; status == GSC_SUCCESS; i++)
	{
		end		= (INC_MB * i - 1);
		offset	= end - 3;
		status	= DMI32_IO_OFFSET__RX_SET(handle, offset);
		if (status != GSC_SUCCESS)	break;

		get		= ~offset;
		status	= dmi32_read(handle, &get, 4, &xfer);
		if (status != GSC_SUCCESS)	break;

		if (get == offset)
			size	+=  INC_MB / 1024 / 1024;

		if (end >= 0xFFFFFFFF)
			break;
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (errs)
		size_mb[0]	= 0;
	else
		size_mb[0]	= size;

	return(errs);
}


