/* Application - DMI32 - Any OS */
/* $Workfile: irq.c $ */
/* $Revision: 2 $ */
/* $Date: 10/27/07 11:01a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/sbtest/irq.c $
 *
 * 2     10/27/07 11:01a Dnewbold
 * Ported to Linux and updated API.
 *
 * 1     8/22/05 4:10p Dnewbold
*/

#include <stdio.h>

#include "main.h"
#include "util_label.h"



/* typedefs	*****************************************************************/

typedef struct
{
	unsigned long	set;
	unsigned long	get;
} set_get_t;

typedef struct
{
	const char*			name;
	U32					parm;
	const set_get_t*	list;
	size_t				size;
} set_get_list_t;



/*****************************************************************************
*
*	Function:	_get_irq_name
*
*	Purpose:
*
*		Return the name of the given IRQ.
*
*	Arguments:
*
*		which	The IRQ if interest.
*
*	Returned:
*
*		!NULL	The name of the IRQ.
*
*****************************************************************************/

static const char* _get_irq_name(U32 which)
{
	const char*	name	= "NONE";

	if (which & DMI32_WHICH_IRQ_MBUR)	name	= "MBUR";
	which	&= ~DMI32_WHICH_IRQ_MBUR;
	if (which & DMI32_WHICH_IRQ_UIOB)	name	= "UIOB";
	which	&= ~DMI32_WHICH_IRQ_UIOB;
	if (which & DMI32_WHICH_IRQ_UIOA)	name	= "UIOA";
	which	&= ~DMI32_WHICH_IRQ_UIOA;
	if (which & DMI32_WHICH_IRQ_BE)		name	= "BE";
	which	&= ~DMI32_WHICH_IRQ_BE;
	if (which & DMI32_WHICH_IRQ_BS)		name	= "BS";
	which	&= ~DMI32_WHICH_IRQ_BS;
	if (which)							name	= "INVALID";

	return(name);
}



/*****************************************************************************
*
*	Function:	_irq_parm_list_test
*
*	Purpose:
*
*		Perform a series of tests per the given data and list.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	Which interrupt are these tests for?
*
*		avail	Is the source available in firmware?
*
*		set		The data set to act on.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_parm_list_test(	void*					handle,
								U32						which,
								U32						avail,
								const set_get_list_t*	set,
								int						verbose)
{
	int				errs	= 0;
	unsigned long	get;
	int				i;
	const char*		irq_name;
	char			name[128];
	U32				status;

	irq_name	= _get_irq_name(which);
	sprintf(name, "%s (%s)", set->name, irq_name);

	if (verbose)
		label_text(name);

	status	= dmi32_init(handle);

	for (i = 0; (status == GSC_SUCCESS) && (avail) && (i < (int) set->size); i++)
	{
		status		= dmi32_config(	handle,
									set->parm,
									which,
									set->list[i].set,
									&get);

		if ((status == GSC_SUCCESS) && (get != set->list[i].get))
			status	= GSC_FAILED;

		if (status != GSC_SUCCESS)
			break;
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (verbose)
	{
		if (avail == 0)
			printf("SKIPPED (interrupt unavailable)\n");
		else if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_callback_arg_tests
*
*	Purpose:
*
*		Perform Interrupt Callback Argument tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_callback_arg_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set								get								*/
		{ 1024,								1024							},
		{ GSC_NO_CHANGE,					1024							},
		{ GSC_DEFAULT,						DMI32_IRQ_CALLBACK_ARG_DEFAULT	},
		{ 0,								0								},
		{ GSC_NO_CHANGE,					0								},
		{ GSC_DEFAULT,						DMI32_IRQ_CALLBACK_ARG_DEFAULT	},
		{ 2048,								2048							},
		{ GSC_NO_CHANGE,					2048							},
		{ GSC_DEFAULT,						DMI32_IRQ_CALLBACK_ARG_DEFAULT	},
		{ DMI32_IRQ_CALLBACK_ARG_DEFAULT,	DMI32_IRQ_CALLBACK_ARG_DEFAULT	},
		{ GSC_NO_CHANGE,					DMI32_IRQ_CALLBACK_ARG_DEFAULT	},
		{ GSC_DEFAULT,						DMI32_IRQ_CALLBACK_ARG_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Callback Argument",
		/* parm	*/	DMI32_IRQ_CALLBACK_ARG,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int				errs	= 0;
	unsigned long	feature;

	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_BS, 1, &set, verbose);
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_BE, 1, &set, verbose);
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_UIOA, 1, &set, verbose);
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_UIOB, 1, &set, verbose);

	DMI32_MISC_FEATURES__IRQ_MBUR(handle, &feature);
	feature	= (feature == DMI32_MISC_FEATURES_PRESENT) ? 1 : 0;
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_MBUR, feature, &set, verbose);

	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_callback_func_tests
*
*	Purpose:
*
*		Perform Interrupt Callback Function tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static void _callback(U32 arg1, U32 arg2, U32 arg3) {}

static int _irq_callback_func_tests(void* handle, int verbose)
{
	static set_get_t	list[]	=
	{
		/* set								get								*/
		{ (unsigned long) _callback,		(unsigned long) _callback		},
		{ GSC_NO_CHANGE,					(unsigned long) _callback		},
		{ GSC_DEFAULT,						DMI32_IRQ_CALLBACK_FUNC_DEFAULT	},
		{ (unsigned long) NULL,				(unsigned long) NULL			},
		{ GSC_NO_CHANGE,					(unsigned long) NULL			},
		{ GSC_DEFAULT,						DMI32_IRQ_CALLBACK_FUNC_DEFAULT	},
		{ DMI32_IRQ_CALLBACK_FUNC_DEFAULT,	DMI32_IRQ_CALLBACK_FUNC_DEFAULT	},
		{ GSC_NO_CHANGE,					DMI32_IRQ_CALLBACK_FUNC_DEFAULT	},
		{ GSC_DEFAULT,						DMI32_IRQ_CALLBACK_FUNC_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Callback Function",
		/* parm	*/	DMI32_IRQ_CALLBACK_ARG,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int				errs	= 0;
	unsigned long	feature;

	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_BS, 1, &set, verbose);
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_BE, 1, &set, verbose);
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_UIOA, 1, &set, verbose);
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_UIOB, 1, &set, verbose);

	DMI32_MISC_FEATURES__IRQ_MBUR(handle, &feature);
	feature	= (feature == DMI32_MISC_FEATURES_PRESENT) ? 1 : 0;
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_MBUR, feature, &set, verbose);

	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_enable_tests
*
*	Purpose:
*
*		Perform Interrupt Enable tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_enable_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set						get							*/
		{ DMI32_IRQ_ENABLE_NO,		DMI32_IRQ_ENABLE_NO			},
		{ GSC_NO_CHANGE,			DMI32_IRQ_ENABLE_NO			},
		{ GSC_DEFAULT,				DMI32_IRQ_ENABLE_DEFAULT	},
		{ DMI32_IRQ_ENABLE_YES,		DMI32_IRQ_ENABLE_YES		},
		{ GSC_NO_CHANGE,			DMI32_IRQ_ENABLE_YES		},
		{ GSC_DEFAULT,				DMI32_IRQ_ENABLE_DEFAULT	},
		{ DMI32_IRQ_ENABLE_DEFAULT,	DMI32_IRQ_ENABLE_DEFAULT	},
		{ GSC_NO_CHANGE,			DMI32_IRQ_ENABLE_DEFAULT	},
		{ GSC_DEFAULT,				DMI32_IRQ_ENABLE_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Enable",
		/* parm	*/	DMI32_IRQ_ENABLE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int				errs	= 0;
	unsigned long	feature;

	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_BS, 1, &set, verbose);
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_BE, 1, &set, verbose);
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_UIOA, 1, &set, verbose);
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_UIOB, 1, &set, verbose);

	DMI32_MISC_FEATURES__IRQ_MBUR(handle, &feature);
	feature	= (feature == DMI32_MISC_FEATURES_PRESENT) ? 1 : 0;
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_MBUR, feature, &set, verbose);

	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_state_tests
*
*	Purpose:
*
*		Perform Interrupt State tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_state_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set				get							*/
		{ 0,				DMI32_IRQ_STATE_INACTIVE	},
		{ GSC_NO_CHANGE,	DMI32_IRQ_STATE_INACTIVE	},
		{ GSC_DEFAULT,		DMI32_IRQ_STATE_INACTIVE	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"State",
		/* parm	*/	DMI32_IRQ_STATE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int				errs	= 0;
	unsigned long	feature;

	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_BS, 1, &set, verbose);
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_BE, 1, &set, verbose);
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_UIOA, 1, &set, verbose);
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_UIOB, 1, &set, verbose);

	DMI32_MISC_FEATURES__IRQ_MBUR(handle, &feature);
	feature	= (feature == DMI32_MISC_FEATURES_PRESENT) ? 1 : 0;
	errs	+= _irq_parm_list_test(handle, DMI32_WHICH_IRQ_MBUR, feature, &set, verbose);

	return(errs);
}



/*****************************************************************************
*
*	Function:	irq_config_tests
*
*	Purpose:
*
*		Perform Interrupt configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int irq_config_tests(void* handle)
{
	int	errs	= 0;
	int	verbose	= 1;

	label_text("Interrupt Configuration");

	if (verbose)
		printf("\n");

	label_level_inc();
	errs	+= _irq_callback_arg_tests(handle, verbose);
	errs	+= _irq_callback_func_tests(handle, verbose);
	errs	+= _irq_enable_tests(handle, verbose);
	errs	+= _irq_state_tests(handle, verbose);
	label_level_dec();

	if (verbose == 0)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}


