/* Application - DMI32 - Any OS */
/* $Workfile: misc.c $ */
/* $Revision: 3 $ */
/* $Date: 10/27/07 11:02a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/sbtest/misc.c $
 *
 * 3     10/27/07 11:02a Dnewbold
 * Additional porting updates.
 *
 * 2     4/05/07 2:56p Dnewbold
 * Ported to Linux.
 *
 * 1     8/22/05 4:10p Dnewbold
*/

#include <stdio.h>

#include "main.h"
#include "util_label.h"



/* typedefs	*****************************************************************/

typedef struct
{
	U32	set;
	U32	get;
} set_get_t;

typedef struct
{
	const char*			name;
	U32					parm;
	const set_get_t*	list;
	size_t				size;
} set_get_list_t;



/*****************************************************************************
*
*	Function:	_misc_parm_list_test
*
*	Purpose:
*
*		Perform a series of tests per the given data and list.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		set		The data set to act on.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _misc_parm_list_test(void*					handle,
								const set_get_list_t*	set,
								int						verbose)
{
	int				errs	= 0;
	unsigned long	get;
	int				i;
	U32				status;

	if ((verbose) && (set->name))
		label_text(set->name);

	status	= dmi32_init(handle);

	for (i = 0; (status == GSC_SUCCESS) && (i < (int) set->size); i++)
	{
		status		= dmi32_config(	handle,
									set->parm,
									0,
									set->list[i].set,
									&get);

		if ((status == GSC_SUCCESS) && (get != set->list[i].get))
			status	= GSC_FAILED;

		if (status != GSC_SUCCESS)
			break;
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if ((verbose) && (set->name))
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_misc_board_jumpers_tests
*
*	Purpose:
*
*		Perform Miscallaneous Board Jumpers tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _misc_board_jumpers_tests(void* handle, int verbose)
{
	int				errs	= 0;
	unsigned long	get;
	U32				status	= GSC_SUCCESS;

	if (verbose)
		label_text("Board Jumpers");

	status	= dmi32_init(handle);

	if (status == GSC_SUCCESS)
	{
		status		= dmi32_config(	handle,
									DMI32_MISC_BOARD_JUMPERS,
									0,
									0,
									&get);

		if ((status == GSC_SUCCESS) && (get & 0xFFFFFFFC))
		{
			status	= GSC_FAILED;
			printf("(invalid value: 0x%lX", (long) get);
		}
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (verbose)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_misc_dram_capacity_mb_tests
*
*	Purpose:
*
*		Perform Miscallaneous DRAM Capacity MB tests.
*
*	Arguments:
*
*		handle		The handle used to access the device under test.
*
*		capacity	The capacity is stored here.
*
*		verbose		Work verbosely?
*
*	Returned:
*
*		0			All went well.
*		> 0			The number of errors seen.
*
*****************************************************************************/

static int _misc_dram_capacity_mb_tests(void* handle, int verbose, U32* capacity)
{
	int				errs	= 0;
	unsigned long	get;
	U32				status	= GSC_SUCCESS;

	if (verbose)
		label_text("DRAM Capacity");

	status	= dmi32_init(handle);

	if (status == GSC_SUCCESS)
	{
		status		= dmi32_config(	handle,
									DMI32_MISC_DRAM_CAPACITY_MB,
									0,
									0,
									&get);
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? errs : 1;

	if (verbose)
	{
		if (errs)
		{
			capacity[0]	= 0xFFFFFFFF;
			printf("FAIL <---\n");
		}
		else
		{
			capacity[0]	= get;
			printf("PASS (%ldMB, 1K = 1024)\n", (long) get);
		}
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_misc_dram_size_mb_tests
*
*	Purpose:
*
*		Perform Miscallaneous DRAM Size MB tests.
*
*	Arguments:
*
*		handle		The handle used to access the device under test.
*
*		verbose		Work verbosely?
*
*		capacity	This is the DRAM capacity.
*
*		size_mb		The computed size is recorded here.
*
*	Returned:
*
*		0			All went well.
*		> 0			The number of errors seen.
*
*****************************************************************************/

static int _misc_dram_size_mb_tests(void* handle, int verbose, U32 capacity, U32* size_mb)
{
	int				errs	= 0;
	unsigned long	get;
	U32				status	= GSC_SUCCESS;

	if (verbose)
		label_text("DRAM Size");

	status	= dmi32_init(handle);

	if (status == GSC_SUCCESS)
		errs	= dram_compute_size_mb(handle, size_mb);
	else
		errs	= 1;

	if (status == GSC_SUCCESS)
	{
		status		= dmi32_config(	handle,
									DMI32_MISC_DRAM_SIZE_MB,
									0,
									0,
									&get);

		if ((status == GSC_SUCCESS) && (get != size_mb[0]))
		{
			printf("(computed %ldMB, told %ldMB)", (long) size_mb[0], (long) get);
			status	= GSC_FAILED;
		}
	}

	if ((status == GSC_SUCCESS) && (get > capacity))
	{
		status	= GSC_FAILED;
		printf(	"(size of %ldMB exceeeds capacity of %ldMB) ",
				(long) get,
				(long) capacity);
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? errs : 1;

	if (verbose)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS (%ldMB, 1K = 1024)\n", (long) get);
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_misc_feature_tests
*
*	Purpose:
*
*		Perform Miscallaneous Features tests on the given feature.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*		feature	The feature to test. We don't care if the feature is
*				supported or not.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _misc_feature_tests(void* handle, int verbose, U32 feature)
{
	int				errs	= 0;
	unsigned long	get;
	char			progress[4]	= { 0, 0, 0, 0 };
	U32				status	= GSC_SUCCESS;

	status	= dmi32_init(handle);

	if (status == GSC_SUCCESS)
	{
		status		= dmi32_config(	handle,
									DMI32_MISC_FEATURES,
									0,
									feature,
									&get);

		if ((status == GSC_SUCCESS) &&
			(get != DMI32_MISC_FEATURES_ABSENT) &&
			(get != DMI32_MISC_FEATURES_PRESENT))
		{
			status	= GSC_FAILED;
			printf("%s (invalid value: 0x%lX", progress, (long) get);
		}
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;
	return(errs);
}



/*****************************************************************************
*
*	Function:	_misc_features_tests
*
*	Purpose:
*
*		Perform Miscallaneous Features tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _misc_features_tests(void* handle, int verbose)
{
	int	errs	= 0;

	if (verbose)
		label_text("Features");

	errs	+= _misc_feature_tests(handle, verbose, DMI32_MISC_FEATURES_FR);
	errs	+= _misc_feature_tests(handle, verbose, DMI32_MISC_FEATURES_IRQ_MBUR);
	errs	+= _misc_feature_tests(handle, verbose, DMI32_MISC_FEATURES_LMBR);

	if (verbose)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_misc_map_gsc_regs_tests
*
*	Purpose:
*
*		Perform Miscellaneous Map GSC Registers tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _misc_map_gsc_regs_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set								get								*/
		{ DMI32_MISC_MAP_GSC_REGS_DISABLE,	DMI32_MISC_MAP_GSC_REGS_DISABLE	},
		{ GSC_NO_CHANGE,					DMI32_MISC_MAP_GSC_REGS_DISABLE	},
		{ GSC_DEFAULT,						DMI32_MISC_MAP_GSC_REGS_DEFAULT	},
		{ DMI32_MISC_MAP_GSC_REGS_ENABLE,	DMI32_MISC_MAP_GSC_REGS_ENABLE	},
		{ GSC_NO_CHANGE,					DMI32_MISC_MAP_GSC_REGS_ENABLE	},
		{ GSC_DEFAULT,						DMI32_MISC_MAP_GSC_REGS_DEFAULT	},
		{ DMI32_MISC_MAP_GSC_REGS_DEFAULT,	DMI32_MISC_MAP_GSC_REGS_DEFAULT	},
		{ GSC_NO_CHANGE,					DMI32_MISC_MAP_GSC_REGS_DEFAULT	},
		{ GSC_DEFAULT,						DMI32_MISC_MAP_GSC_REGS_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Map GSC Registers",
		/* parm	*/	DMI32_MISC_MAP_GSC_REGS,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs;

	errs	= _misc_parm_list_test(handle, &set, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_misc_map_plx_regs_tests
*
*	Purpose:
*
*		Perform Miscellaneous Map PLX Registers tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _misc_map_plx_regs_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set								get								*/
		{ DMI32_MISC_MAP_PLX_REGS_DISABLE,	DMI32_MISC_MAP_PLX_REGS_DISABLE	},
		{ GSC_NO_CHANGE,					DMI32_MISC_MAP_PLX_REGS_DISABLE	},
		{ GSC_DEFAULT,						DMI32_MISC_MAP_PLX_REGS_DEFAULT	},
		{ DMI32_MISC_MAP_PLX_REGS_ENABLE,	DMI32_MISC_MAP_PLX_REGS_ENABLE	},
		{ GSC_NO_CHANGE,					DMI32_MISC_MAP_PLX_REGS_ENABLE	},
		{ GSC_DEFAULT,						DMI32_MISC_MAP_PLX_REGS_DEFAULT	},
		{ DMI32_MISC_MAP_PLX_REGS_DEFAULT,	DMI32_MISC_MAP_PLX_REGS_DEFAULT	},
		{ GSC_NO_CHANGE,					DMI32_MISC_MAP_PLX_REGS_DEFAULT	},
		{ GSC_DEFAULT,						DMI32_MISC_MAP_PLX_REGS_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Map PLX Registers",
		/* parm	*/	DMI32_MISC_MAP_PLX_REGS,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs;

	errs	= _misc_parm_list_test(handle, &set, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_misc_strict_arguments_tests
*
*	Purpose:
*
*		Perform Miscellaneous Strict Arguments tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _misc_strict_arguments_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set									get									*/
		{ DMI32_MISC_STRICT_ARGUMENTS_DISABLE,	DMI32_MISC_STRICT_ARGUMENTS_DISABLE	},
		{ GSC_NO_CHANGE,						DMI32_MISC_STRICT_ARGUMENTS_DISABLE	},
		{ GSC_DEFAULT,							DMI32_MISC_STRICT_ARGUMENTS_DEFAULT	},
		{ DMI32_MISC_STRICT_ARGUMENTS_ENABLE,	DMI32_MISC_STRICT_ARGUMENTS_ENABLE	},
		{ GSC_NO_CHANGE,						DMI32_MISC_STRICT_ARGUMENTS_ENABLE	},
		{ GSC_DEFAULT,							DMI32_MISC_STRICT_ARGUMENTS_DEFAULT	},
		{ DMI32_MISC_STRICT_ARGUMENTS_DEFAULT,	DMI32_MISC_STRICT_ARGUMENTS_DEFAULT	},
		{ GSC_NO_CHANGE,						DMI32_MISC_STRICT_ARGUMENTS_DEFAULT	},
		{ GSC_DEFAULT,							DMI32_MISC_STRICT_ARGUMENTS_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	NULL,
		/* parm	*/	DMI32_MISC_STRICT_ARGUMENTS,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	U32	after;
	U32	before;
	int	errs;
	U32	status;

	if (verbose)
		label_text("Strict Arguments");

	errs	= _misc_parm_list_test(handle, &set, verbose);

	if (errs == 0)
	{
		status	= dmi32_config(	handle,
								DMI32_MISC_STRICT_ARGUMENTS,
								0,
								DMI32_MISC_STRICT_ARGUMENTS_ENABLE,
								NULL);

		before	= dmi32_config(	handle,
								DMI32_MISC_STRICT_ARGUMENTS,
								~0,
								DMI32_MISC_STRICT_ARGUMENTS_ENABLE,
								NULL);

		if (status == GSC_SUCCESS)
		{
			status	= dmi32_config(	handle,
									DMI32_MISC_STRICT_ARGUMENTS,
									0,
									DMI32_MISC_STRICT_ARGUMENTS_DISABLE,
									NULL);
		}

		after	= dmi32_config(	handle,
								DMI32_MISC_STRICT_ARGUMENTS,
								~0,
								DMI32_MISC_STRICT_ARGUMENTS_DISABLE,
								NULL);

		if ((before == GSC_SUCCESS) || (after != GSC_SUCCESS))
			status	= GSC_FAILED;

		errs	= (status == GSC_SUCCESS) ? 0 : 1;

	}

	dmi32_init(handle);

	if (verbose)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_misc_strict_dram_limits_tests
*
*	Purpose:
*
*		Perform Miscellaneous Strict DRAM Limits tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*		size_mb	The amount of installed DRAM in megabyte units.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _misc_strict_dram_limits_tests(
	void*	handle,
	int		verbose,
	U32		size_mb)
{
	static const set_get_t	list[]	=
	{
		/* set										get										*/
		{ DMI32_MISC_STRICT_DRAM_LIMITS_DISABLE,	DMI32_MISC_STRICT_DRAM_LIMITS_DISABLE	},
		{ GSC_NO_CHANGE,							DMI32_MISC_STRICT_DRAM_LIMITS_DISABLE	},
		{ GSC_DEFAULT,								DMI32_MISC_STRICT_DRAM_LIMITS_DEFAULT	},
		{ DMI32_MISC_STRICT_DRAM_LIMITS_ENABLE,		DMI32_MISC_STRICT_DRAM_LIMITS_ENABLE	},
		{ GSC_NO_CHANGE,							DMI32_MISC_STRICT_DRAM_LIMITS_ENABLE	},
		{ GSC_DEFAULT,								DMI32_MISC_STRICT_DRAM_LIMITS_DEFAULT	},
		{ DMI32_MISC_STRICT_DRAM_LIMITS_DEFAULT,	DMI32_MISC_STRICT_DRAM_LIMITS_DEFAULT	},
		{ GSC_NO_CHANGE,							DMI32_MISC_STRICT_DRAM_LIMITS_DEFAULT	},
		{ GSC_DEFAULT,								DMI32_MISC_STRICT_DRAM_LIMITS_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	NULL,
		/* parm	*/	DMI32_MISC_STRICT_DRAM_LIMITS,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int				errs;
	U32				status;
	unsigned long	ul;

	if (verbose)
		label_text("Strict DRAM Limits");

	errs	= _misc_parm_list_test(handle, &set, verbose);

	if ((errs == 0) && (size_mb < 4096))
	{
		status	= DMI32_MISC_STRICT_DRAM_LIMITS__NO(handle);

		if (status == GSC_SUCCESS)
			status	= DMI32_XFER_OFFSET__SET(handle, DMI32_XFER_OFFSET_MAX);

		if (status == GSC_SUCCESS)
			status	= DMI32_XFER_OFFSET__GET(handle, &ul);

		if ((status == GSC_SUCCESS) && (ul != DMI32_XFER_OFFSET_MAX))
			status	= GSC_FAILED;

		status	= DMI32_MISC_STRICT_DRAM_LIMITS__YES(handle);

		if (status == GSC_SUCCESS)
			status	= DMI32_XFER_OFFSET__SET(handle, DMI32_XFER_OFFSET_MAX);

		if (status == GSC_SUCCESS)
			status	= DMI32_XFER_OFFSET__GET(handle, &ul);

		if ((status == GSC_SUCCESS) && (ul == DMI32_XFER_OFFSET_MAX))
			status	= GSC_FAILED;

		errs	= (status == GSC_SUCCESS) ? 0 : 1;
	}

	dmi32_init(handle);

	if (verbose)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	misc_config_tests
*
*	Purpose:
*
*		Perform Miscellaneous configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		size_mb	Record the memory size here.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int misc_config_tests(void* handle, U32* size_mb)
{
	U32	capacity	= 0;
	int	errs		= 0;
	int	verbose		= 1;

	label_text("Miscellaneous Configuration");

	if (verbose)
		printf("\n");

	label_level_inc();
	errs	+= _misc_board_jumpers_tests(handle, verbose);
	errs	+= _misc_dram_capacity_mb_tests(handle, verbose, &capacity);
	errs	+= _misc_dram_size_mb_tests(handle, verbose, capacity, size_mb);
	errs	+= _misc_features_tests(handle, verbose);
	errs	+= _misc_map_gsc_regs_tests(handle, verbose);
	errs	+= _misc_map_plx_regs_tests(handle, verbose);
	errs	+= _misc_strict_arguments_tests(handle, verbose);
	errs	+= _misc_strict_dram_limits_tests(handle, verbose, size_mb[0]);
	label_level_dec();

	if (verbose == 0)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}


