/* Application - DMI32 - Any OS */
/* $Workfile: user_io.c $ */
/* $Revision: 2 $ */
/* $Date: 10/27/07 11:02a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/sbtest/user_io.c $
 *
 * 2     10/27/07 11:02a Dnewbold
 * Ported to Linux and updated API.
 *
 * 1     8/22/05 4:10p Dnewbold
*/

#include <stdio.h>

#include "main.h"
#include "util_label.h"



/* typedefs	*****************************************************************/

typedef struct
{
	U32	set;
	U32	get;
} set_get_t;

typedef struct
{
	const char*			name;
	U32					parm;
	const set_get_t*	list;
	size_t				size;
} set_get_list_t;

typedef struct
{
	U32	parm;
	U32	which;
	U32	set;
	U32	reg;
	U32	mask;	// Terminate list with a ZERO which value.
	U32	value;
} set_reg_t;



/*****************************************************************************
*
*	Function:	_user_io_parm_list_test
*
*	Purpose:
*
*		Perform a series of tests per the given data and list.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	Which I/O size are these tests for?
*
*		set		The data set to act on.
*
*		reg		The register access tests to perform.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _user_io_parm_list_test(	void*					handle,
									U32						which,
									const set_get_list_t*	set,
									const set_reg_t*		reg,
									int						verbose)
{
	int				errs	= 0;
	unsigned long	get;
	int				i;
	char			name[128];
	U32				status;
	U32				u32;

	sprintf(name, "%s %s", (which & DMI32_WHICH_USER_IO_A) ? "A" : "B", set->name);

	if (verbose)
		label_text(name);

	status	= dmi32_init(handle);

	if ((status == GSC_SUCCESS) &&  (set->size == 0))
	{
		// Don't perform a check on the value retrieved.
		status		= dmi32_config(handle, set->parm, which, 0, &get);
	}

	for (i = 0; (status == GSC_SUCCESS) && (i < (int) set->size); i++)
	{
		status	= dmi32_config(	handle,
								set->parm,
								which,
								set->list[i].set,
								&get);

		if ((status == GSC_SUCCESS) && (get != set->list[i].get))
			status	= GSC_FAILED;

		if (status != GSC_SUCCESS)
			break;
	}

	if (status == GSC_SUCCESS)
		status	= dmi32_init(handle);

	if ((status == GSC_SUCCESS) && (reg))
	{
		for (i = 0; reg->mask; reg++, i++)
		{
			status	= dmi32_config(	handle,
									reg->parm,
									reg->which,
									reg->set,
									NULL);

			if (status == GSC_SUCCESS)
				status	= dmi32_reg_read(handle, reg->reg, &u32);

			if (status == GSC_SUCCESS)
			{
				u32	&= reg->mask;

				if (u32 != reg->value)
				{
					status	= GSC_FAILED;
					printf(	"(%d. expected 0x%lX, read 0x%lX) ",
							i,
							(long) reg->value,
							(long) u32);
					break;
				}
			}
		}
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (verbose)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_user_io_direction_tests
*
*	Purpose:
*
*		Perform User I/O Direction tests. This test doesn't actually verify
*		that the abort request actually works. This test simply verifies that
*		the API service operates properly when an I/O operation is not active.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _user_io_direction_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set								get								*/
		{ DMI32_USER_IO_DIRECTION_IN,		DMI32_USER_IO_DIRECTION_IN		},
		{ GSC_NO_CHANGE,					DMI32_USER_IO_DIRECTION_IN		},
		{ GSC_DEFAULT,						DMI32_USER_IO_DIRECTION_DEFAULT	},
		{ DMI32_USER_IO_DIRECTION_OUT,		DMI32_USER_IO_DIRECTION_OUT		},
		{ GSC_NO_CHANGE,					DMI32_USER_IO_DIRECTION_OUT		},
		{ GSC_DEFAULT,						DMI32_USER_IO_DIRECTION_DEFAULT	},
		{ DMI32_USER_IO_DIRECTION_DEFAULT,	DMI32_USER_IO_DIRECTION_DEFAULT	},
		{ GSC_NO_CHANGE,					DMI32_USER_IO_DIRECTION_DEFAULT	},
		{ GSC_DEFAULT,						DMI32_USER_IO_DIRECTION_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Direction",
		/* parm	*/	DMI32_USER_IO_DIRECTION,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	static const set_reg_t	a[]	=
	{
		{ // A = INPUT
		/* parm		*/	DMI32_USER_IO_DIRECTION,
		/* which	*/	DMI32_WHICH_USER_IO_A,
		/* set		*/	DMI32_USER_IO_DIRECTION_IN,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_USER_IO_A_DIR,
		/* value	*/	0
		},
		{ // A = OUTPUT
		/* parm		*/	DMI32_USER_IO_DIRECTION,
		/* which	*/	DMI32_WHICH_USER_IO_A,
		/* set		*/	DMI32_USER_IO_DIRECTION_OUT,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_USER_IO_A_DIR,
		/* value	*/	DMI32_BCR_USER_IO_A_DIR
		},
		{ 0, 0, 0, 0, 0 }	// terminate with mask of 0.
	};

	static const set_reg_t	b[]	=
	{
		{ // A = INPUT
		/* parm		*/	DMI32_USER_IO_DIRECTION,
		/* which	*/	DMI32_WHICH_USER_IO_A,
		/* set		*/	DMI32_USER_IO_DIRECTION_IN,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_USER_IO_A_DIR,
		/* value	*/	0
		},
		{ // B = OUTPUT
		/* parm		*/	DMI32_USER_IO_DIRECTION,
		/* which	*/	DMI32_WHICH_USER_IO_B,
		/* set		*/	DMI32_USER_IO_DIRECTION_OUT,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_USER_IO_A_DIR | DMI32_BCR_USER_IO_B_DIR,
		/* value	*/	DMI32_BCR_USER_IO_B_DIR
		},
		{ // A = OUTPUT
		/* parm		*/	DMI32_USER_IO_DIRECTION,
		/* which	*/	DMI32_WHICH_USER_IO_A,
		/* set		*/	DMI32_USER_IO_DIRECTION_OUT,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_USER_IO_A_DIR,
		/* value	*/	DMI32_BCR_USER_IO_A_DIR
		},
		{ // B = INPUT
		/* parm		*/	DMI32_USER_IO_DIRECTION,
		/* which	*/	DMI32_WHICH_USER_IO_B,
		/* set		*/	DMI32_USER_IO_DIRECTION_IN,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_USER_IO_A_DIR | DMI32_BCR_USER_IO_B_DIR,
		/* value	*/	DMI32_BCR_USER_IO_A_DIR
		},
		{ 0, 0, 0, 0, 0 }	// terminate with mask of 0.
	};

	int	errs	= 0;

	errs	+= _user_io_parm_list_test(handle, DMI32_WHICH_USER_IO_A, &set, a, verbose);
	errs	+= _user_io_parm_list_test(handle, DMI32_WHICH_USER_IO_B, &set, b, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_user_io_input_tests
*
*	Purpose:
*
*		Perform User I/O Input tests. This test doesn't actually verify that
*		the abort request actually works. This test simply verifies that the
*		API service operates properly when an I/O operation is not active.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _user_io_input_tests(void* handle, int verbose)
{
	static const set_get_list_t	set	=
	{
		/* name	*/	"Input",
		/* parm	*/	DMI32_USER_IO_INPUT,
		/* list	*/	NULL,
		/* size	*/	0
	};

	int	errs	= 0;

	errs	+= _user_io_parm_list_test(handle, DMI32_WHICH_USER_IO_A, &set, NULL, verbose);
	errs	+= _user_io_parm_list_test(handle, DMI32_WHICH_USER_IO_B, &set, NULL, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_user_io_output_tests
*
*	Purpose:
*
*		Perform User I/O Output tests. This test doesn't actually verify that
*		the abort request actually works. This test simply verifies that the
*		API service operates properly when an I/O operation is not active.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _user_io_output_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set							get								*/
		{ 0,							0								},
		{ GSC_NO_CHANGE,				0								},
		{ GSC_DEFAULT,					DMI32_USER_IO_OUTPUT_DEFAULT	},
		{ DMI32_USER_IO_OUTPUT_MASK,	DMI32_USER_IO_OUTPUT_MASK		},
		{ GSC_NO_CHANGE,				DMI32_USER_IO_OUTPUT_MASK		},
		{ GSC_DEFAULT,					DMI32_USER_IO_OUTPUT_DEFAULT	},
		{ DMI32_USER_IO_OUTPUT_DEFAULT,	DMI32_USER_IO_OUTPUT_DEFAULT	},
		{ GSC_NO_CHANGE,				DMI32_USER_IO_OUTPUT_DEFAULT	},
		{ GSC_DEFAULT,					DMI32_USER_IO_OUTPUT_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Output",
		/* parm	*/	DMI32_USER_IO_OUTPUT,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	static const set_reg_t	a[]	=
	{
		{ // A = all bits set
		/* parm		*/	DMI32_USER_IO_OUTPUT,
		/* which	*/	DMI32_WHICH_USER_IO_A,
		/* set		*/	DMI32_USER_IO_OUTPUT_MASK,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_USER_IO_A_OUTPUT,
		/* value	*/	DMI32_BCR_USER_IO_A_OUTPUT
		},
		{ // A = 0
		/* parm		*/	DMI32_USER_IO_OUTPUT,
		/* which	*/	DMI32_WHICH_USER_IO_A,
		/* set		*/	0,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_USER_IO_A_OUTPUT,
		/* value	*/	0
		},
		{ 0, 0, 0, 0, 0 }	// terminate with mask of 0.
	};

	static const set_reg_t	b[]	=
	{
		{ // B = all bits set
		/* parm		*/	DMI32_USER_IO_OUTPUT,
		/* which	*/	DMI32_WHICH_USER_IO_B,
		/* set		*/	DMI32_USER_IO_OUTPUT_MASK,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_USER_IO_B_OUTPUT,
		/* value	*/	DMI32_BCR_USER_IO_B_OUTPUT
		},
		{ // B = 0
		/* parm		*/	DMI32_USER_IO_OUTPUT,
		/* which	*/	DMI32_WHICH_USER_IO_B,
		/* set		*/	0,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_USER_IO_B_OUTPUT,
		/* value	*/	0
		},
		{ 0, 0, 0, 0, 0 }	// terminate with mask of 0.
	};

	int	errs	= 0;

	errs	+= _user_io_parm_list_test(handle, DMI32_WHICH_USER_IO_A, &set, a, verbose);
	errs	+= _user_io_parm_list_test(handle, DMI32_WHICH_USER_IO_B, &set, b, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	user_io_config_tests
*
*	Purpose:
*
*		Perform User I/O configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int user_io_config_tests(void* handle)
{
	int	errs	= 0;
	int	verbose	= 1;

	label_text("User I/O Configuration");

	if (verbose)
		printf("\n");

	label_level_inc();
	errs	+= _user_io_direction_tests(handle, verbose);
	errs	+= _user_io_input_tests(handle, verbose);
	errs	+= _user_io_output_tests(handle, verbose);
	label_level_dec();

	if (verbose == 0)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}


