/* Application - DMI32 - Any OS */
/* $Workfile: xfer.c $ */
/* $Revision: 2 $ */
/* $Date: 10/27/07 11:02a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/sbtest/xfer.c $
 *
 * 2     10/27/07 11:02a Dnewbold
 * Ported to Linux and updated API.
 *
 * 1     8/22/05 4:10p Dnewbold
*/

#include <stdio.h>

#include "main.h"
#include "util_label.h"



/* typedefs	*****************************************************************/

typedef struct
{
	U32	set;
	U32	get;
} set_get_t;

typedef struct
{
	const char*			name;
	U32					parm;
	const set_get_t*	list;
	size_t				size;
} set_get_list_t;

typedef struct
{
	U32	parm;
	U32	set;
	U32	reg;
	U32	mask;	// Terminate list with a ZERO which value.
	U32	value;
} set_reg_t;



/*****************************************************************************
*
*	Function:	_xfer_parm_list_test
*
*	Purpose:
*
*		Perform a series of tests per the given data and list.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		set		The data set to act on.
*
*		reg		The register access tests to perform.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _xfer_parm_list_test(	void*					handle,
									const set_get_list_t*	set,
									const set_reg_t*		reg,
									int						verbose)
{
	int				errs	= 0;
	unsigned long	get;
	int				i;
	U32				status;
	U32				u32;

	if (verbose)
		label_text(set->name);

	status	= dmi32_init(handle);

	if ((status == GSC_SUCCESS) && (set->size == 0))
	{
		// Don't perform a check on the value retrieved.
		status	= dmi32_config(handle, set->parm, 0, 0, &get);
	}

	for (i = 0; (status == GSC_SUCCESS) && (i < (int) set->size); i++)
	{
		status	= dmi32_config(handle, set->parm, 0, set->list[i].set, &get);

		if ((status == GSC_SUCCESS) && (get != set->list[i].get))
			status	= GSC_FAILED;

		if (status != GSC_SUCCESS)
			break;
	}

	if (status == GSC_SUCCESS)
		status	= dmi32_init(handle);

	if ((status == GSC_SUCCESS) && (reg))
	{
		for (i = 0; reg->mask; reg++, i++)
		{
			status	= dmi32_config(	handle,
									reg->parm,
									0,
									reg->set,
									NULL);

			if (status == GSC_SUCCESS)
				status	= dmi32_reg_read(handle, reg->reg, &u32);

			if (status == GSC_SUCCESS)
			{
				u32	&= reg->mask;

				if (u32 != reg->value)
				{
					status	= GSC_FAILED;
					printf(	"(%d. expected 0x%lX, read 0x%lX) ",
							i,
							(long) reg->value,
							(long) u32);
					break;
				}
			}
		}
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (verbose)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_xfer_direction_tests
*
*	Purpose:
*
*		Perform Transfer Direction tests. This test doesn't actually verify
*		that the abort request actually works. This test simply verifies that
*		the API service operates properly when an I/O operation is not active.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _xfer_direction_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set							get								*/
		{ DMI32_XFER_DIRECTION_RX,		DMI32_XFER_DIRECTION_RX			},
		{ GSC_NO_CHANGE,				DMI32_XFER_DIRECTION_RX			},
		{ GSC_DEFAULT,					DMI32_XFER_DIRECTION_DEFAULT	},
		{ DMI32_XFER_DIRECTION_TX,		DMI32_XFER_DIRECTION_TX			},
		{ GSC_NO_CHANGE,				DMI32_XFER_DIRECTION_TX			},
		{ GSC_DEFAULT,					DMI32_XFER_DIRECTION_DEFAULT	},
		{ DMI32_XFER_DIRECTION_DEFAULT,	DMI32_XFER_DIRECTION_DEFAULT	},
		{ GSC_NO_CHANGE,				DMI32_XFER_DIRECTION_DEFAULT	},
		{ GSC_DEFAULT,					DMI32_XFER_DIRECTION_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Direction",
		/* parm	*/	DMI32_XFER_DIRECTION,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	static const set_reg_t	reg[]	=
	{
		{ // Direction = INPUT
		/* parm		*/	DMI32_XFER_DIRECTION,
		/* set		*/	DMI32_XFER_DIRECTION_RX,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_XFER_DIR,
		/* value	*/	0
		},
		{ // Direction = OUTPUT
		/* parm		*/	DMI32_XFER_DIRECTION,
		/* set		*/	DMI32_XFER_DIRECTION_TX,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_XFER_DIR,
		/* value	*/	DMI32_BCR_XFER_DIR
		},
		{ 0, 0, 0, 0, 0 }	// terminate with mask of 0.
	};

	int	errs	= 0;

	errs	+= _xfer_parm_list_test(handle, &set, reg, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_xfer_internal_clock_speed_tests
*
*	Purpose:
*
*		Perform Transfer Internal Clock Speed tests. This test doesn't
*		actually verify that the abort request actually works. This test
*		simply verifies that the API service operates properly when an I/O
*		operation is not active.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _xfer_internal_clock_speed_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set										get										*/
		{ DMI32_XFER_INTERNAL_CLOCK_SPEED_HIGH,		DMI32_XFER_INTERNAL_CLOCK_SPEED_HIGH	},
		{ GSC_NO_CHANGE,							DMI32_XFER_INTERNAL_CLOCK_SPEED_HIGH	},
		{ GSC_DEFAULT,								DMI32_XFER_INTERNAL_CLOCK_SPEED_DEFAULT	},
		{ DMI32_XFER_INTERNAL_CLOCK_SPEED_LOW,		DMI32_XFER_INTERNAL_CLOCK_SPEED_LOW		},
		{ GSC_NO_CHANGE,							DMI32_XFER_INTERNAL_CLOCK_SPEED_LOW		},
		{ GSC_DEFAULT,								DMI32_XFER_INTERNAL_CLOCK_SPEED_DEFAULT	},
		{ DMI32_XFER_INTERNAL_CLOCK_SPEED_DEFAULT,	DMI32_XFER_INTERNAL_CLOCK_SPEED_DEFAULT	},
		{ GSC_NO_CHANGE,							DMI32_XFER_INTERNAL_CLOCK_SPEED_DEFAULT	},
		{ GSC_DEFAULT,								DMI32_XFER_INTERNAL_CLOCK_SPEED_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Internal Clock Speed",
		/* parm	*/	DMI32_XFER_INTERNAL_CLOCK_SPEED,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	static const set_reg_t	reg[]	=
	{
		{ // Direction = INPUT
		/* parm		*/	DMI32_XFER_INTERNAL_CLOCK_SPEED,
		/* set		*/	DMI32_XFER_INTERNAL_CLOCK_SPEED_HIGH,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_INT_CLOCK_SPEED,
		/* value	*/	0
		},
		{ // Direction = OUTPUT
		/* parm		*/	DMI32_XFER_INTERNAL_CLOCK_SPEED,
		/* set		*/	DMI32_XFER_INTERNAL_CLOCK_SPEED_LOW,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_INT_CLOCK_SPEED,
		/* value	*/	DMI32_BCR_INT_CLOCK_SPEED
		},
		{ 0, 0, 0, 0, 0 }	// terminate with mask of 0.
	};

	int	errs	= 0;

	errs	+= _xfer_parm_list_test(handle, &set, reg, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_xfer_mode_tests
*
*	Purpose:
*
*		Perform Transfer Mode tests. This test doesn't actually verify that
*		the abort request actually works. This test simply verifies that the
*		API service operates properly when an I/O operation is not active.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _xfer_mode_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set						get						*/
		{ DMI32_XFER_MODE_SINGLE,	DMI32_XFER_MODE_SINGLE	},
		{ GSC_NO_CHANGE,			DMI32_XFER_MODE_SINGLE	},
		{ GSC_DEFAULT,				DMI32_XFER_MODE_DEFAULT	},
		{ DMI32_XFER_MODE_MULTI,	DMI32_XFER_MODE_MULTI	},
		{ GSC_NO_CHANGE,			DMI32_XFER_MODE_MULTI	},
		{ GSC_DEFAULT,				DMI32_XFER_MODE_DEFAULT	},
		{ DMI32_XFER_MODE_DEFAULT,	DMI32_XFER_MODE_DEFAULT	},
		{ GSC_NO_CHANGE,			DMI32_XFER_MODE_DEFAULT	},
		{ GSC_DEFAULT,				DMI32_XFER_MODE_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Mode",
		/* parm	*/	DMI32_XFER_MODE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	static const set_reg_t	reg[]	=
	{
		{ // Direction = INPUT
		/* parm		*/	DMI32_XFER_MODE,
		/* set		*/	DMI32_XFER_MODE_SINGLE,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_XFER_MODE,
		/* value	*/	0
		},
		{ // Direction = OUTPUT
		/* parm		*/	DMI32_XFER_MODE,
		/* set		*/	DMI32_XFER_MODE_MULTI,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_XFER_MODE,
		/* value	*/	DMI32_BCR_XFER_MODE
		},
		{ 0, 0, 0, 0, 0 }	// terminate with mask of 0.
	};

	int	errs	= 0;

	errs	+= _xfer_parm_list_test(handle, &set, reg, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_xfer_offset_tests
*
*	Purpose:
*
*		Perform Transfer Offset tests. This test doesn't actually verify that
*		the abort request actually works. This test simply verifies that the
*		API service operates properly when an I/O operation is not active.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _xfer_offset_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set							get							*/
		{ 0xFF0,						0xFF0						},
		{ GSC_NO_CHANGE,				0xFF0						},
		{ GSC_DEFAULT,					DMI32_XFER_OFFSET_DEFAULT	},
		{ 0xFFF0,						0xFFF0						},
		{ GSC_NO_CHANGE,				0xFFF0					},
		{ GSC_DEFAULT,					DMI32_XFER_OFFSET_DEFAULT	},
		{ DMI32_XFER_OFFSET_DEFAULT,	DMI32_XFER_OFFSET_DEFAULT	},
		{ GSC_NO_CHANGE,				DMI32_XFER_OFFSET_DEFAULT	},
		{ GSC_DEFAULT,					DMI32_XFER_OFFSET_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Offset",
		/* parm	*/	DMI32_XFER_OFFSET,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	static const set_reg_t	reg[]	=
	{
		{ // Direction = INPUT
		/* parm		*/	DMI32_XFER_OFFSET,
		/* set		*/	0xFF0,
		/* reg		*/	DMI32_TBBAR,
		/* mask		*/	0xFFFFFFFF,
		/* value	*/	0xFF0
		},
		{ // Direction = OUTPUT
		/* parm		*/	DMI32_XFER_OFFSET,
		/* set		*/	0xFFF0,
		/* reg		*/	DMI32_TBBAR,
		/* mask		*/	0xFFFFFFFF,
		/* value	*/	0xFFF0
		},
		{ 0, 0, 0, 0, 0 }	// terminate with mask of 0.
	};

	int	errs	= 0;

	errs	+= _xfer_parm_list_test(handle, &set, reg, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_xfer_size_tests
*
*	Purpose:
*
*		Perform Transfer Size tests. This test doesn't actually verify that
*		the abort request actually works. This test simply verifies that the
*		API service operates properly when an I/O operation is not active.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _xfer_size_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set						get						*/
		{ 0xFF0,					0xFF0					},
		{ GSC_NO_CHANGE,			0xFF0					},
		{ GSC_DEFAULT,				DMI32_XFER_SIZE_DEFAULT	},
		{ 0xFFF0,					0xFFF0					},
		{ GSC_NO_CHANGE,			0xFFF0					},
		{ GSC_DEFAULT,				DMI32_XFER_SIZE_DEFAULT	},
		{ DMI32_XFER_SIZE_DEFAULT,	DMI32_XFER_SIZE_DEFAULT	},
		{ GSC_NO_CHANGE,			DMI32_XFER_SIZE_DEFAULT	},
		{ GSC_DEFAULT,				DMI32_XFER_SIZE_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Size",
		/* parm	*/	DMI32_XFER_SIZE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	static const set_reg_t	reg[]	=
	{
		{ // Direction = INPUT
		/* parm		*/	DMI32_XFER_SIZE,
		/* set		*/	0xFF0,
		/* reg		*/	DMI32_TBSR,
		/* mask		*/	0xFFFFFFFF,
		/* value	*/	0xFF0
		},
		{ // Direction = OUTPUT
		/* parm		*/	DMI32_XFER_SIZE,
		/* set		*/	0xFFF0,
		/* reg		*/	DMI32_TBSR,
		/* mask		*/	0xFFFFFFFF,
		/* value	*/	0xFFF0
		},
		{ 0, 0, 0, 0, 0 }	// terminate with mask of 0.
	};

	int	errs	= 0;

	errs	+= _xfer_parm_list_test(handle, &set, reg, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_xfer_start_word_tests
*
*	Purpose:
*
*		Perform Transfer Start Word tests. This test doesn't actually verify
*		that the abort request actually works. This test simply verifies that
*		the API service operates properly when an I/O operation is not active.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _xfer_start_word_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set								get								*/
		{ 0,								0								},
		{ GSC_NO_CHANGE,					0								},
		{ GSC_DEFAULT,						DMI32_XFER_START_WORD_DEFAULT	},
		{ DMI32_XFER_START_WORD_MASK,		DMI32_XFER_START_WORD_MASK		},
		{ GSC_NO_CHANGE,					DMI32_XFER_START_WORD_MASK		},
		{ GSC_DEFAULT,						DMI32_XFER_START_WORD_DEFAULT	},
		{ DMI32_XFER_START_WORD_DEFAULT,	DMI32_XFER_START_WORD_DEFAULT	},
		{ GSC_NO_CHANGE,					DMI32_XFER_START_WORD_DEFAULT	},
		{ GSC_DEFAULT,						DMI32_XFER_START_WORD_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Start Word",
		/* parm	*/	DMI32_XFER_START_WORD,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	static const set_reg_t	reg[]	=
	{
		{
		/* parm		*/	DMI32_XFER_START_WORD,
		/* set		*/	0,
		/* reg		*/	DMI32_TSWR,
		/* mask		*/	DMI32_XFER_START_WORD_MASK,
		/* value	*/	0
		},
		{
		/* parm		*/	DMI32_XFER_START_WORD,
		/* set		*/	DMI32_XFER_START_WORD_DEFAULT,
		/* reg		*/	DMI32_TSWR,
		/* mask		*/	DMI32_XFER_START_WORD_MASK,
		/* value	*/	DMI32_XFER_START_WORD_DEFAULT
		},
		{
		/* parm		*/	DMI32_XFER_START_WORD,
		/* set		*/	DMI32_XFER_START_WORD_MASK,
		/* reg		*/	DMI32_TSWR,
		/* mask		*/	DMI32_XFER_START_WORD_MASK,
		/* value	*/	DMI32_XFER_START_WORD_MASK
		},
		{ 0, 0, 0, 0, 0 }	// terminate with mask of 0.
	};

	int	errs;

	errs	= _xfer_parm_list_test(handle, &set, reg, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_xfer_start_word_direction_tests
*
*	Purpose:
*
*		Perform Transfer Start Word Direction tests. This test doesn't
*		actually verify that the abort request actually works. This test
*		simply verifies that the API service operates properly when an I/O
*		operation is not active.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _xfer_start_word_direction_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set									get									*/
		{ DMI32_XFER_START_WORD_DIR_RX,			DMI32_XFER_START_WORD_DIR_RX		},
		{ GSC_NO_CHANGE,						DMI32_XFER_START_WORD_DIR_RX		},
		{ GSC_DEFAULT,							DMI32_XFER_START_WORD_DIR_DEFAULT	},
		{ DMI32_XFER_START_WORD_DIR_TX,			DMI32_XFER_START_WORD_DIR_TX		},
		{ GSC_NO_CHANGE,						DMI32_XFER_START_WORD_DIR_TX		},
		{ GSC_DEFAULT,							DMI32_XFER_START_WORD_DIR_DEFAULT	},
		{ DMI32_XFER_START_WORD_DIR_DEFAULT,	DMI32_XFER_START_WORD_DIR_DEFAULT	},
		{ GSC_NO_CHANGE,						DMI32_XFER_START_WORD_DIR_DEFAULT	},
		{ GSC_DEFAULT,							DMI32_XFER_START_WORD_DIR_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Start Word Direction",
		/* parm	*/	DMI32_XFER_START_WORD_DIR,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	static const set_reg_t	reg[]	=
	{
		{ // Direction = INPUT
		/* parm		*/	DMI32_XFER_START_WORD_DIR,
		/* set		*/	DMI32_XFER_START_WORD_DIR_TX,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_START_WORD_DIR,
		/* value	*/	0
		},
		{ // Direction = OUTPUT
		/* parm		*/	DMI32_XFER_START_WORD_DIR,
		/* set		*/	DMI32_XFER_START_WORD_DIR_RX,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_START_WORD_DIR,
		/* value	*/	DMI32_BCR_START_WORD_DIR
		},
		{ 0, 0, 0, 0, 0 }	// terminate with mask of 0.
	};

	int				errs	= 0;
	unsigned long	feature;
	U32				status;

	status	= DMI32_MISC_FEATURES__SW_DIR(handle, &feature);

	if (status != GSC_SUCCESS)
	{
		errs	= 1;
		label_text(set.name);
		printf("FAIL <--- (feature access)\n");
	}
	else if (feature != DMI32_MISC_FEATURES_PRESENT)
	{
		label_text(set.name);
		printf("SKIPPED (feature unavailable)\n");
	}
	else
	{
		errs	+= _xfer_parm_list_test(handle, &set, reg, verbose);
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_xfer_test_mode_tests
*
*	Purpose:
*
*		Perform Transfer Test Mode tests. This test doesn't actually verify
*		that the abort request actually works. This test simply verifies that
*		the API service operates properly when an I/O operation is not
*		active.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _xfer_test_mode_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set							get								*/
		{ DMI32_XFER_TEST_MODE_YES,		DMI32_XFER_TEST_MODE_YES		},
		{ GSC_NO_CHANGE,				DMI32_XFER_TEST_MODE_YES		},
		{ GSC_DEFAULT,					DMI32_XFER_TEST_MODE_DEFAULT	},
		{ DMI32_XFER_TEST_MODE_NO,		DMI32_XFER_TEST_MODE_NO			},
		{ GSC_NO_CHANGE,				DMI32_XFER_TEST_MODE_NO			},
		{ GSC_DEFAULT,					DMI32_XFER_TEST_MODE_DEFAULT	},
		{ DMI32_XFER_TEST_MODE_DEFAULT,	DMI32_XFER_TEST_MODE_DEFAULT	},
		{ GSC_NO_CHANGE,				DMI32_XFER_TEST_MODE_DEFAULT	},
		{ GSC_DEFAULT,					DMI32_XFER_TEST_MODE_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Test Mode",
		/* parm	*/	DMI32_XFER_TEST_MODE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	static const set_reg_t	reg[]	=
	{
		{ // Direction = INPUT
		/* parm		*/	DMI32_XFER_TEST_MODE,
		/* set		*/	DMI32_XFER_TEST_MODE_NO,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_TEST_MODE,
		/* value	*/	0
		},
		{ // Direction = OUTPUT
		/* parm		*/	DMI32_XFER_TEST_MODE,
		/* set		*/	DMI32_XFER_TEST_MODE_YES,
		/* reg		*/	DMI32_BCR,
		/* mask		*/	DMI32_BCR_TEST_MODE,
		/* value	*/	DMI32_BCR_TEST_MODE
		},
		{ 0, 0, 0, 0, 0 }	// terminate with mask of 0.
	};

	int	errs	= 0;

	errs	+= _xfer_parm_list_test(handle, &set, reg, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	xfer_config_tests
*
*	Purpose:
*
*		Perform Transfer configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int xfer_config_tests(void* handle)
{
	int	errs	= 0;
	int	verbose	= 1;

	label_text("Transfer Configuration");

	if (verbose)
		printf("\n");

	label_level_inc();
	// We don't test Transfer CLKIN Test as we need a clock.
	errs	+= _xfer_direction_tests(handle, verbose);
	errs	+= _xfer_internal_clock_speed_tests(handle, verbose);
	// We don't test Transfer Enable as we need a clock.
	errs	+= _xfer_mode_tests(handle, verbose);
	errs	+= _xfer_offset_tests(handle, verbose);
	errs	+= _xfer_size_tests(handle, verbose);
	errs	+= _xfer_start_word_tests(handle, verbose);
	errs	+= _xfer_start_word_direction_tests(handle, verbose);
	// We don't test Transfer State as we need a clock.
	// We don't test Transfer State Rx as we need a clock.
	// We don't test Transfer State Tx as we need a clock.
	errs	+= _xfer_test_mode_tests(handle, verbose);
	label_level_dec();

	if (verbose == 0)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}


